//========================================================================================
//
//	File:				MovieFac.h
//	Release Version:	$ 1.0d1 $
//
//	Author:				Lonnie Millett
//	Creation Date:		3/28/94
//
//	Copyright:	 1993, 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef MOVIEFAC_H
#define MOVIEFAC_H

#ifndef FWFACET_H
#include <FWFacet.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__MOVIES__)
#include <Movies.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__ALIASES__)
#include <Aliases.h>
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class CMovieFacet;

//==============================================================================
// Classes used by this interface
//==============================================================================

class XMPFacet;
class XMPShape;
class FW_CRect;
class CMovieFrame;
class CMoviePart;
class XMPDragItemIterator;

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// CMovieFacet
//==============================================================================

class CMovieFacet : public FW_CFacet
{
public:
	CMovieFacet();
	virtual ~CMovieFacet();
	void InitMovieFacet(XMPFacet* xmpFacet, CMovieFrame* movieFrame);
	
	CMovieFrame* GetMovieFrame() const;
	CMoviePart* GetMoviePart() const;
	
	Movie GetMovie();
	virtual void SetMovie(Movie newMovie);
	
	void AdjustMovieBox();
	virtual void  ClipShapeChanged();
	void GetMovieBoundingBox(FW_CRect* boundingBox);
	
	virtual void StartMovie();
	virtual void StopMovie();
	virtual void SetMovieLooping(FW_Boolean loop);
	
	virtual void Draw(FW_CGraphicContext *gc);
	virtual void  GeometryChanged();
	
	virtual void WindowActivated();
	virtual void WindowDeActivated();
	
	virtual FW_Boolean DoMouseDown(const FW_CPoint& where, XMPEventData event);
	FW_Boolean DoMovieKey(XMPEventData event);
	void DoMovieSetVisible(FW_Boolean visible);
	FW_Boolean DoMovieIdle();
	void DoMovieActivate(FW_Boolean activate);
	
	FW_Boolean IsControllerVisible();
	
	//--- Selection ------
	void GetCurrentSelection(TimeValue* currentTime, TimeValue* duration);
	void SetCurrentSelection(TimeValue currentTime, TimeValue duration);
	FW_Boolean IsEmpty();
	void CloseSelection();
	void ClearSelection(TimeValue currentTime, TimeValue duration);
	void SelectAll();
	FW_PlatformHandle CopySelection();
	
	//--- Drag and Drop ------
	virtual void DragEnter(XMPDragItemIterator* dragInfo, const FW_CPoint& where);
	virtual FW_Boolean DrawDragHiliteInside();
	PicHandle GetMovieAsPict();
	void PasteMovieToSelection(Movie newMovie, TimeValue currentTime, TimeValue duration);
	void PasteHandleToSelection(FW_PlatformHandle newHandle, OSType handleType, TimeValue currentTime, TimeValue duration);

private:
	CMovieFrame* fMovieFrame;
	Movie fMovie;
	MovieController fMovieController;
};

//----------------------------------------------------------------------------------------
// CMovieFacet::GetMovieFrame()
//----------------------------------------------------------------------------------------
inline CMovieFrame* CMovieFacet::GetMovieFrame() const
{
	return fMovieFrame;
}

//----------------------------------------------------------------------------------------
// CMovieFacet::GetMovie
//----------------------------------------------------------------------------------------
inline Movie CMovieFacet::GetMovie()
{
	return fMovie;
}

#endif
