//========================================================================================
//
//	File:							MoviePar.h
//	Release Version:	$ 1.0d1 $
//
//	Author:						Lonnie Millett
//	Creation Date:		3/28/94
//
//	Copyright:	 1993, 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef MOVIEPAR_H
#define MOVIEPAR_H

#ifndef MOVIEDEF_H
#include "MovieDef.h"
#endif

#ifndef FWPART_H
#include <FWPart.h>
#endif

// ----- Graphic Includes -----

#ifndef FWPICSHP_H
#include "FWPicShp.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(__MOVIES__)
#include <Movies.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__ALIASES__)
#include <Aliases.h>
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class CMoviePart;

//==============================================================================
// Classes used by this interface
//==============================================================================

class FW_CPart;
class CMovieFrame;
class CMovieFacet;
class XMPStorageUnit;
class XMPSession;
class XMPFrame;
class XMPFacet;
class FW_CSelection;

//==============================================================================
// Constants
//==============================================================================

const XMPCommandID cChooseMovie = 10100;
const XMPCommandID cStartMovie = 10101;
const XMPCommandID cStopMovie = 10102;
const XMPCommandID cSetMovieLooping = 10103;

//==============================================================================
// CClockPart
//==============================================================================

class CMoviePart : public FW_CPart
{

public:
	CMoviePart();
	virtual ~ CDECL CMoviePart();

	virtual void Release();

	virtual void Initialize();
	
	virtual void InternalizeContent(XMPStorageUnit* storageUnit);
	virtual void ExternalizeContent(XMPStorageUnit* storageUnit);
	
	virtual void AddProperties(XMPStorageUnit* storageUnit);
	virtual XMPValueType GetContentPropertyValueType() const;
	
	FW_Boolean IsQuickTimeInstalled();
	FW_PlatformHandle GetMovieHandle() const;
	void SetMovieHandle(FW_PlatformHandle movieHandle);
	FW_Boolean GetMovieLooping() const;
	void SetMovieLooping(FW_Boolean loop);
	Movie GetNewMovieFromMovieHandle();
	FW_CPictShape GetDefaultPicture() const;
	
	virtual FW_CFrame* NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	virtual FW_CSelection* NewSelection();


	virtual FW_Boolean DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	virtual FW_Boolean DoIdle();
	virtual void DoAdjustMenus(XMPMenuBar* menuBar);

	AliasHandle GetMovieFileFromUser();
	Movie GetMovieFromAlias(AliasHandle movieAlias);
	
	void ToggleLooping();
	
	FW_CFacet* GetAnyExistingFacet();
	
private:
	FW_Boolean fQuickTimeInstalled;
	FW_PlatformHandle fMovieHandle;
	FW_Boolean fLooping;
	FW_CPictShape fDefaultPicture;
};

//----------------------------------------------------------------------------------------
// CMoviePart::IsQuickTimeInstalled
//----------------------------------------------------------------------------------------
inline FW_Boolean CMoviePart::IsQuickTimeInstalled()
{
	return fQuickTimeInstalled;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetMovieHandle
//----------------------------------------------------------------------------------------
inline FW_PlatformHandle CMoviePart::GetMovieHandle() const
{
	return fMovieHandle;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetMovieLooping
//----------------------------------------------------------------------------------------
inline FW_Boolean CMoviePart::GetMovieLooping() const
{
	return fLooping;
}

//----------------------------------------------------------------------------------------
// CMoviePart::GetMovieLooping
//----------------------------------------------------------------------------------------
inline FW_CPictShape CMoviePart::GetDefaultPicture() const
{
	return fDefaultPicture;
}

#endif
