//========================================================================================
//
//	File:				SndPart.h
//	Release Version:	$ 1.0d1 $
//
//	Author:				Lonnie Millett
//	Creation Date:		3/28/94
//
//	Copyright:	 1993, 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef SNDPART_H
#define SNDPART_H

#ifndef SNDPARTDEF_H
#include "SndPartDef.h"
#endif

// ----- Framework Includes -----

#ifndef FWCTLMGR_H
#include "FWCtlMgr.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

//==============================================================================
//	 class CSndPart
//==============================================================================

class CSndPart : public FW_CCtlMgrPart
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CSndPart();
	virtual CDECL ~CSndPart();
	
//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual XMPValueType		GetContentPropertyValueType() const;

	virtual FW_CFrame*				NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	virtual FW_CSelection*		NewSelection();
	
	virtual void 				InternalizeContent(XMPStorageUnit* storageUnit);
	virtual void 				ExternalizeContent(XMPStorageUnit* storageUnit);
	
	virtual void 				DoClick(const FW_CPoint& where, XMPEventData event);
	
	FW_PlatformHandle						GetSoundHandle() const;
	void						SetSoundHandle(FW_PlatformHandle soundHandle);
	
protected:
	virtual ControlHandle		CreateControlHandle(XMPPlatformWindow window, 
													const FW_CRect& controlRect);
	

private:
	FW_PlatformHandle fSoundHandle;
									
};

//----------------------------------------------------------------------------------------
// CSndPart::GetSoundHandle
//----------------------------------------------------------------------------------------
inline FW_PlatformHandle CSndPart::GetSoundHandle() const
{
	return fSoundHandle;
}

//==============================================================================
//	 class CSndFrame
//==============================================================================

#define kCSndFrameID "appl:CSndframe$class,1.0.0"

class CSndFrame : public FW_CCtlMgrFrame
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	CSndFrame();
	virtual void InitSndFrame(XMPFrame* frame, FW_CPart* part);	
	virtual CDECL ~CSndFrame();

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual void			UpdateUsedShape();

};

//==============================================================================
//	 class CSndSelection
//==============================================================================

class CSndSelection : public FW_CSelection
{
	
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//------------------------------------------------------------------------------
public:
					CSndSelection();
	void			InitSndSelection(CSndPart* sndPart);
	virtual 		~CSndSelection();
	
//------------------------------------------------------------------------------
//	 Inherited API
//------------------------------------------------------------------------------
public:
	virtual void		CloseSelection();
	virtual void		SelectAll();
	virtual FW_Boolean	IsEmpty() const;
	virtual FW_Boolean	DoClear();
	
	virtual void		ExternalizeSelection(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual FW_Boolean	InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);

//------------------------------------------------------------------------------
//	 Data Members
//------------------------------------------------------------------------------
private:
	CSndPart*			fSndPart;
};

#endif
