//========================================================================================
//
//	File:							EmbedRun.h
//	Release Version:	$ 1.0d1 $
//
//	Author:						Anthone Burbidge
//	Creation Date:		3/28/94
//
//	Copyright:	 1993, 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef _EMBEDRUN_
#define _EMBEDRUN_

#ifndef _GRFXRUN_
#include "GrfxRun.h"
#endif

//========================================================================================
// Forward class declarations
//========================================================================================

class CTextProxyRun;


//========================================================================================
// Constant definitions
//========================================================================================

const ClassId kEmbeddedPartClassId = 'EMBD';
const ClassId kProxyRunAttr = 'PART';


//========================================================================================
// CLASS CEmbeddedRun
//========================================================================================

class CEmbeddedRun : public CBetterGraphicsRun
{
// ----- Constants
public:
	enum { kSelectionWidth = 4 };

// ----- Initialization/destruction
public:		
	CEmbeddedRun();
	void InitEmbeddedRun();
	virtual ~CEmbeddedRun();

// ----- Inherited from CGraphicsRun
public:
	virtual CAttrObject* CreateNew() const;
	virtual ClassId GetClassId() const;
	virtual void GetAttrIDs(short* countAttr, AttrId* attrArray) const;
	virtual void FreeData();

	virtual OSErr WriteToStream(CStream* outputStream);
	virtual OSErr ReadFromStream(CStream* inputStream);

	virtual void DrawSelection(char hiliteStat, const RunPositionPB* paramPb);
	virtual void DoClick(ClickPB*, TClickCommandInfo*, Boolean* abortDrag);
	virtual Boolean PointInResizeHandle(ClickPB* clickPB) const;
	virtual void Resize(ClickPB* clickPB);
	virtual void DrawResizeFeedback(const Rect& rect);

protected:
	virtual void BufferToAttribute(AttrId theAttr, const void* attrBuffer);
	virtual void AttributeToBuffer(AttrId theAttr, void* attrBuffer) const;
	virtual long GetAttrFlags(AttrId theAttr, void* attrBuffer, long updateMessage=0) const;
	virtual Boolean GetDimensions(short* hite, short* width);
	virtual void Draw(const Rect* theRect);
	
// ----- New API
public:
	void SetProxyRun(CTextProxyRun* proxyRun);
	CTextProxyRun* GetProxyRun() const;
	virtual void ChangePosition(const Point& pt);
	virtual void ChangeShape(const Rect& rt);

// ----- New fields
private:
	CTextProxyRun*		fProxyRun;
};

//----------------------------------------------------------------------------------------
// CEmbeddedRun::SetProxyRun
//----------------------------------------------------------------------------------------

void CEmbeddedRun::SetProxyRun(CTextProxyRun* proxyRun)
{
	fProxyRun = proxyRun;
}
//----------------------------------------------------------------------------------------
// CEmbeddedRun::GetProxyRun
//----------------------------------------------------------------------------------------

CTextProxyRun* CEmbeddedRun::GetProxyRun() const
{
	return fProxyRun;
}

#endif
