//========================================================================================
//
//	File:							TextSelc.h
//	Release Version:	$ 1.0d1 $
//
//	Author:						Anthone Burbidge
//	Creation Date:		3/28/94
//
//	Copyright:	 1993, 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef _TEXTSELC_
#define _TEXTSELC_

// ----- Framework Includes -----

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

//========================================================================================
// Forward class declarations
//========================================================================================

class CTextPart;


//========================================================================================
// CLASS CTextSelection
//========================================================================================

class CTextSelection : public FW_CSelection
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
						CTextSelection();
	void				InitTextSelection(CTextPart* textPart);
	virtual				~CTextSelection();
	
//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual void		CloseSelection();
	virtual void		SelectAll();
	virtual FW_Boolean	IsEmpty() const;

	// ----- Cut/Copy/Paste/Clear -----
	virtual FW_Boolean	DoClear();

	virtual FW_CProxyRun*	IsSelectionOnlyOneProxyRun() const;

	virtual FW_Boolean	EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape);

	virtual void		ExternalizeSelection_ContentPropertyValueType
							(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual void		ExternalizeSelection_TEXT
							(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual void		ExternalizeSelection
							(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind);
	virtual FW_Boolean	InternalizeSelection_ContentPropertyValueType
							(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);
	virtual FW_Boolean	InternalizeSelection_TEXT
							(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);
	virtual FW_Boolean	InternalizeSelection
							(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);

	// ----- Drag & Drop -----
	virtual XMPRgnHandle CalcDragRgn(FW_CFacet* facet);
	virtual FW_Boolean	DoDrop(XMPStorageUnit* dropSU, FW_CFacet* facet, const FW_CPoint& originPoint, const FW_CPoint& dropPoint);
	virtual FW_Boolean	DoDroppedInSameFrame(XMPStorageUnit* dropSU, FW_CFacet* facet, const FW_CPoint& originPoint, const FW_CPoint& dropPoint);
	
	// ----- Parameter accessors -----
	FW_Boolean			IsCloning() const
							{ return fIsCloning; }
	FW_CFrame*				GetCommandFrame() const
							{ return fCommandFrame; }
	XMPCloneKind		GetCloneKind() const
							{ return fCloneKind; }

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	virtual FW_Boolean	IsPointInSelection(const FW_CPoint& xmpPt);
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	CTextPart*			fTextPart;

	// The following fields are used to pass parameters from a text part through
	// textension to an embedded part.
	
	FW_Boolean			fIsCloning;
	FW_CFrame*				fCommandFrame;
	XMPCloneKind		fCloneKind;
};

#endif
