//  The C++ Booch Components (Version 2.1)
//  (C) Copyright 1990-1993 Grady Booch. All Rights Reserved.
//
//  Restricted Rights Legend
//  Use, duplication, or disclosure is subject to restrictions as set forth 
//  in subdivision (c)(1)(ii) of the Rights in Technical Data and Computer 
//  Software clause at DFARS 252.227-7013. 
//
//  BCOColU.cpp
//
//  This file contains the definitions for the unbounded ordered collection.

#include "BCOColU.h"

template<class Item, class StorageManager>
BC_TUnboundedOrderedCollection<Item, StorageManager>::
  BC_TUnboundedOrderedCollection() {}

template<class Item, class StorageManager>
BC_TUnboundedOrderedCollection<Item, StorageManager>::
  BC_TUnboundedOrderedCollection
    (const BC_TUnboundedOrderedCollection<Item, StorageManager>& c)
      : BC_TOrderedCollection<Item>(c),
        fRep(c.fRep) {}

template<class Item, class StorageManager>
BC_TUnboundedOrderedCollection<Item, StorageManager>::
  ~BC_TUnboundedOrderedCollection() {}

template<class Item, class StorageManager>
BC_TCollection<Item>& BC_TUnboundedOrderedCollection<Item, StorageManager>::
  operator=(const BC_TCollection<Item>& c)
{
  return BC_TCollection<Item>::operator=(c);
}

template<class Item, class StorageManager>
BC_TCollection<Item>& BC_TUnboundedOrderedCollection<Item, StorageManager>::
  operator=(const BC_TUnboundedOrderedCollection<Item, StorageManager>& c)
{
  return BC_TCollection<Item>::operator=(c);
}

template<class Item, class StorageManager>
BC_Boolean BC_TUnboundedOrderedCollection<Item, StorageManager>::
  operator==(const BC_TCollection<Item>& c) const
{
  return BC_TCollection<Item>::operator==(c);
}

template<class Item, class StorageManager>
BC_Boolean BC_TUnboundedOrderedCollection<Item, StorageManager>::
  operator==(const BC_TUnboundedOrderedCollection<Item, StorageManager>& c) const
{
  return (fRep == c.fRep);
}

template<class Item, class StorageManager>
BC_Boolean BC_TUnboundedOrderedCollection<Item, StorageManager>::
  operator!=(const BC_TUnboundedOrderedCollection<Item, StorageManager>& c) const
{
  return !operator==(c);
}

template<class Item, class StorageManager>
const Item& BC_TUnboundedOrderedCollection<Item, StorageManager>::
  operator[](BC_Index index) const
{
  return fRep[index];
}

template<class Item, class StorageManager>
Item& BC_TUnboundedOrderedCollection<Item, StorageManager>::operator[](BC_Index index)
{
  return fRep[index];
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::Clear()
{
  fRep.Clear();
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::Insert(const Item& item)
{
  for (BC_Index index = 0; (index < fRep.Length()); index++)
    if (fOrder(item) <= fOrder(fRep[index])) {
      fRep.Insert(item, index);
      return;
    }
  fRep.Append(item);  
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::
  Insert(const Item& item, BC_Index before)
{
  if (fOrder(item) == fOrder(fRep[before]))
    fRep.Insert(item, before);
  else {
    for (BC_Index index = 0; (index < fRep.Length()); index++)
      if (fOrder(item) <= fOrder(fRep[index])) {
        fRep.Insert(item, index);
        return;
      }
    fRep.Append(item);  
  }
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::Append(const Item& item)
{
  for (BC_Index index = 0; (index < fRep.Length()); index++)
    if (fOrder(item) < fOrder(fRep[index])) {
      fRep.Insert(item, index);
      return;
    }
  fRep.Append(item);  
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::
  Append(const Item& item, BC_Index after)
{
  if (fOrder(item) == fOrder(fRep[after]))
    fRep.Append(item, after);
  else {
    for (BC_Index index = 0; (index < fRep.Length()); index++)
      if (fOrder(item) < fOrder(fRep[index])) {
        fRep.Insert(item, index);
        return;
      }
    fRep.Append(item);  
  }
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::Remove(BC_Index at)
{
  fRep.Remove(at);
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::
  Replace(BC_Index at, const Item& item)
{
  fRep.Remove(at);
  for (BC_Index index = 0; (index < fRep.Length()); index++)
    if (fOrder(item) <= fOrder(fRep[index])) {
      fRep.Insert(item, index);
      return;
    }
  fRep.Append(item);  
}

template<class Item, class StorageManager>
BC_Index BC_TUnboundedOrderedCollection<Item, StorageManager>::Length() const
{
  return fRep.Length();
}

template<class Item, class StorageManager>
BC_Boolean BC_TUnboundedOrderedCollection<Item, StorageManager>::IsEmpty() const
{
  return (fRep.Length() == 0);
}

template<class Item, class StorageManager>
const Item& BC_TUnboundedOrderedCollection<Item, StorageManager>::First() const
{
  return fRep.First();
}

template<class Item, class StorageManager>
Item& BC_TUnboundedOrderedCollection<Item, StorageManager>::First()
{
  return fRep.First();
}

template<class Item, class StorageManager>
const Item& BC_TUnboundedOrderedCollection<Item, StorageManager>::Last() const
{
  return fRep.Last();
}

template<class Item, class StorageManager>
Item& BC_TUnboundedOrderedCollection<Item, StorageManager>::Last()
{
  return fRep.Last();
}

template<class Item, class StorageManager>
BC_ExtendedIndex BC_TUnboundedOrderedCollection<Item, StorageManager>::
  Location(const Item& item) const
{
  return fRep.Location(item);
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::Purge()
{
  fRep.Clear();
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::Add(const Item& item)
{
  for (BC_Index index = 0; (index < fRep.Length()); index++)
    if (fOrder(item) <= fOrder(fRep[index])) {
      fRep.Insert(item, index);
      return;
    }
  fRep.Append(item);  
}

template<class Item, class StorageManager>
BC_Index BC_TUnboundedOrderedCollection<Item, StorageManager>::Cardinality() const
{
  return fRep.Length();
}

template<class Item, class StorageManager>
const Item& BC_TUnboundedOrderedCollection<Item, StorageManager>::
  ItemAt(BC_Index index) const
{
  return fRep.ItemAt(index);
}

template<class Item, class StorageManager>
void* BC_TUnboundedOrderedCollection<Item, StorageManager>::operator new(size_t s)
{
  return StorageManager::Allocate(s);
}

template<class Item, class StorageManager>
void BC_TUnboundedOrderedCollection<Item, StorageManager>::
  operator delete(void* p, size_t s)
{
  StorageManager::Deallocate(p, s);
}
