//  The C++ Booch Components (Version 2.1)
//  (C) Copyright 1990-1993 Grady Booch. All Rights Reserved.
//
//  BCMapD.h
//
//  This file contains the declaration of the dynamic map.

#ifndef BCMAPD_H
#define BCMAPD_H 1

#include "BCNodes.h"
#include "BCDynami.h"
#include "BCHashTa.h"
#include "BCMap.h"

// Dynamic map

template<class Item, class Value, BC_Index Buckets, class StorageManager>
class BC_TDynamicMap : public BC_TMap<Item, Value> {
public:

  BC_TDynamicMap();
  BC_TDynamicMap(BC_Index (*)(const Item&));
  BC_TDynamicMap(BC_Index chunkSize);
  BC_TDynamicMap(const BC_TDynamicMap<Item, Value, Buckets, StorageManager>&);
  virtual ~BC_TDynamicMap();

  virtual BC_TMap<Item, Value>& operator=(const BC_TMap<Item, Value>&);
  virtual BC_TMap<Item, Value>&
    operator=(const BC_TDynamicMap<Item, Value, Buckets, StorageManager>&);
  virtual BC_Boolean operator==(const BC_TMap<Item, Value>&) const;
  virtual BC_Boolean 
    operator==(const BC_TDynamicMap<Item, Value, Buckets, StorageManager>&) const;
  BC_Boolean 
    operator!=(const BC_TDynamicMap<Item, Value, Buckets, StorageManager>&) const;

  virtual void SetHashFunction(BC_Index (*)(const Item&));
  virtual void SetChunkSize(BC_Index chunkSize);
  virtual void Clear();
  virtual void Preallocate(BC_Index new_length);
  virtual BC_Boolean Bind(const Item&, const Value&);
  virtual BC_Boolean Rebind(const Item&, const Value&);
  virtual BC_Boolean Unbind(const Item&);

  virtual BC_Index ChunkSize() const;
  virtual BC_Index Extent() const;
  virtual BC_Boolean IsEmpty() const;
  virtual BC_Boolean IsBound(const Item&) const;
  virtual const Value* ValueOf(const Item&) const;
  virtual Value* ValueOf(const Item&);

  static void* operator new(size_t);
  static void operator delete(void*, size_t);

protected:

  BC_TTable<Item, Value, Buckets, BC_TDynamic<BC_TPair<Item, Value>, StorageManager> > 
    fRep;

  virtual void Purge();
  virtual BC_Boolean Attach(const Item&, const Value&);
  virtual BC_Index Cardinality() const;
  virtual BC_Index NumberOfBuckets() const;
  virtual BC_Index Length(BC_Index bucket) const;
  virtual BC_Boolean Exists(const Item&) const;
  virtual const Item& ItemAt(BC_Index bucket, BC_Index index) const;
  virtual const Value& ValueAt(BC_Index bucket, BC_Index index) const;

};

#endif
