//  The C++ Booch Components (Version 2.1)
//  (C) Copyright 1990-1993 Grady Booch. All Rights Reserved.
//
//  BCQue.h
//
//  This file contains the declaration of the queue abstract base class
//  and its iterators.

#ifndef BCQUE_H
#define BCQUE_H 1

#include "BCType.h"

template<class Item>
class BC_TQueueActiveIterator;

template<class Item>
class BC_TQueuePassiveIterator;

template<class Item, class Structure>
class BC_TPersist;

// Queue abstract base class

template<class Item>
class BC_TQueue {
public:

  BC_TQueue();
  BC_TQueue(const BC_TQueue<Item>&);
  virtual ~BC_TQueue();

  virtual BC_TQueue<Item>& operator=(const BC_TQueue<Item>&);
  virtual BC_Boolean operator==(const BC_TQueue<Item>&) const;
  BC_Boolean operator!=(const BC_TQueue<Item>&) const;
  
  virtual void Clear() = 0;
  virtual void Append(const Item&) = 0;
  virtual void Pop() = 0;
  virtual void Remove(BC_Index at) = 0;

  virtual BC_Index Length() const = 0;
  virtual BC_Boolean IsEmpty() const = 0;
  virtual const Item& Front() const = 0;
  virtual Item& Front() = 0;
  virtual BC_ExtendedIndex Location(const Item&) const = 0;

protected:

  virtual void Purge() = 0;
  virtual void Add(const Item&) = 0;
  virtual BC_Index Cardinality() const = 0;
  virtual const Item& ItemAt(BC_Index) const = 0;

  virtual void Lock();
  virtual void Unlock();

private:

  friend class BC_TQueueActiveIterator<Item>;
  friend class BC_TQueuePassiveIterator<Item>;
  
  friend class BC_TPersist<Item, BC_TQueue<Item> >;

};

// Queue iterators

template <class Item>
class BC_TQueueActiveIterator {
public:

  BC_TQueueActiveIterator(const BC_TQueue<Item>&);
  ~BC_TQueueActiveIterator();
  
  void Reset();
  BC_Boolean Next();

  BC_Boolean IsDone() const;
  const Item* CurrentItem() const;
  Item* CurrentItem();
  
protected:

  const BC_TQueue<Item>& fQueue;
  BC_ExtendedIndex fIndex;
  
};

template <class Item>
class BC_TQueuePassiveIterator {
public:

  BC_TQueuePassiveIterator(const BC_TQueue<Item>&);
  ~BC_TQueuePassiveIterator();
  
  BC_Boolean Apply(BC_Boolean (*)(const Item&));
  BC_Boolean Apply(BC_Boolean (*)(Item&));
  
protected:

  const BC_TQueue<Item>& fQueue;

};

#endif
