//  The C++ Booch Components (Version 2.1)
//  (C) Copyright 1990-1993 Grady Booch. All Rights Reserved.
//
//  BCQueD.h
//
//  This file contains the declaration of the dynamic queue.

#ifndef BCQUED_H
#define BCQUED_H 1

#include "BCDynami.h"
#include "BCQue.h"

// Dynamic queue

template<class Item, class StorageManager>
class BC_TDynamicQueue : public BC_TQueue<Item> {
public:

  BC_TDynamicQueue();
  BC_TDynamicQueue(BC_Index chunkSize);
  BC_TDynamicQueue(const BC_TDynamicQueue<Item, StorageManager>&);
  virtual ~BC_TDynamicQueue();

  virtual BC_TQueue<Item>& operator=(const BC_TQueue<Item>&);
  virtual BC_TQueue<Item>& operator=(const BC_TDynamicQueue<Item, StorageManager>&);
  virtual BC_Boolean operator==(const BC_TQueue<Item>&) const;
  virtual BC_Boolean operator==(const BC_TDynamicQueue<Item, StorageManager>&) const;
  BC_Boolean operator!=(const BC_TDynamicQueue<Item, StorageManager>&) const;

  virtual void SetChunkSize(BC_Index chunkSize);
  virtual void Preallocate(BC_Index new_length);
  virtual void Clear();
  virtual void Append(const Item&);
  virtual void Pop();
  virtual void Remove(BC_Index at);

  virtual BC_Index ChunkSize() const;
  virtual BC_Index Length() const;
  virtual BC_Boolean IsEmpty() const;
  virtual const Item& Front() const;
  virtual Item& Front();
  virtual BC_ExtendedIndex Location(const Item&) const;

  static void* operator new(size_t);
  static void operator delete(void*, size_t);

protected:

  BC_TDynamic<Item, StorageManager> fRep;

  virtual void Purge();
  virtual void Add(const Item&);
  virtual BC_Index Cardinality() const;
  virtual const Item& ItemAt(BC_Index) const;
  
};

#endif
