//  The C++ Booch Components (Version 2.1)
//  (C) Copyright 1990-1993 Grady Booch. All Rights Reserved.
//
//  BCSetU.h
//
//  This file contains the declaration of the unbounded set.

#ifndef BCSETU_H
#define BCSETU_H 1

#include "BCNodes.h"
#include "BCUnboun.h"
#include "BCHashTa.h"
#include "BCSet.h"

// Unbounded set

template<class Item, BC_Index Buckets, class StorageManager>
class BC_TUnboundedSet : public BC_TSet<Item> {
public:

  BC_TUnboundedSet();
  BC_TUnboundedSet(BC_Index (*)(const Item&));
  BC_TUnboundedSet(const BC_TUnboundedSet<Item, Buckets, StorageManager>&);
  virtual ~BC_TUnboundedSet();

  virtual BC_TSet<Item>& operator=(const BC_TSet<Item>&);
  virtual BC_TSet<Item>&
    operator=(const BC_TUnboundedSet<Item, Buckets, StorageManager>&);
  virtual BC_Boolean operator==(const BC_TSet<Item>&) const;
  virtual BC_Boolean
    operator==(const BC_TUnboundedSet<Item, Buckets, StorageManager>&) const;
  BC_Boolean operator!=(const BC_TUnboundedSet<Item, Buckets, StorageManager>&) const;

  virtual void SetHashFunction(BC_Index (*)(const Item&));
  virtual void Clear();
  virtual BC_Boolean Add(const Item&);
  virtual BC_Boolean Remove(const Item& at);

  virtual BC_Index Extent() const;
  virtual BC_Boolean IsEmpty() const;
  virtual BC_Boolean IsMember(const Item&) const;
  
  static void* operator new(size_t);
  static void operator delete(void*, size_t);

protected:

  BC_TTable<Item, BC_Boolean, Buckets,
            BC_TUnbounded<BC_TPair<Item, BC_Boolean>, StorageManager> > fRep;

  virtual void Purge();
  virtual BC_Boolean Attach(const Item&, const BC_Boolean&);
  virtual BC_Boolean Attach(const Item&);
  virtual BC_Boolean Detach(const Item&);
  virtual BC_Index Cardinality() const;
  virtual BC_Index NumberOfBuckets() const;
  virtual BC_Index Length(BC_Index bucket) const;
  virtual BC_Boolean Exists(const Item&) const;
  virtual const Item& ItemAt(BC_Index bucket, BC_Index index) const;
  
};

#endif
