//  The C++ Booch Components (Version 2.1)
//  (C) Copyright 1990-1993 Grady Booch. All Rights Reserved.
//
//  Restricted Rights Legend
//  Use, duplication, or disclosure is subject to restrictions as set forth 
//  in subdivision (c)(1)(ii) of the Rights in Technical Data and Computer 
//  Software clause at DFARS 252.227-7013. 
//
//  BCSortV.cpp
//
//  This file contains the definitions for the vector sorting tools.

#include "BCSortV.h"

template<class Item, class Sequence>
BC_TSort<Item, Sequence>::BC_TSort()
  : fIsLessThan(0) {}

template<class Item, class Sequence>
BC_TSort<Item, Sequence>::
  BC_TSort(BC_Boolean (*IsLessThan)(const Item& x, const Item& y))
    : fIsLessThan(IsLessThan) {}

template<class Item, class Sequence>
BC_TSort<Item, Sequence>::~BC_TSort() {}

template<class Item, class Sequence>
void BC_TSort<Item, Sequence>::
  SetIsLessThanFunction(BC_Boolean (*IsLessThan)(const Item& x, const Item& y))
{
  fIsLessThan = IsLessThan;
}

template<class Item, class Sequence>
void BC_TSort<Item, Sequence>::Swap(Sequence& s, BC_Index i, BC_Index j)
{
  Item temp = s[i];
  (Item&)(s[i]) = s[j];
  (Item&)(s[j]) = temp;
}

template<class Item, class Sequence>
BC_TQuickSort<Item, Sequence>::BC_TQuickSort() {}

template<class Item, class Sequence>
BC_TQuickSort<Item, Sequence>::
  BC_TQuickSort(BC_Boolean (*IsLessThan)(const Item& x, const Item& y))
    : BC_TSort<Item, Sequence>(IsLessThan) {}

template<class Item, class Sequence>
BC_TQuickSort<Item, Sequence>::~BC_TQuickSort() {}

template<class Item, class Sequence>
void BC_TQuickSort<Item, Sequence>::Sort(Sequence& s)
{
  RecursiveSort(s, 0, s.Length() - 1, fIsLessThan);
}

template<class Item, class Sequence>
void BC_TQuickSort<Item, Sequence>::
  RecursiveSort(Sequence& s, BC_Index left, BC_Index right,
                BC_Boolean (*IsLessThan)(const Item& x, const Item& y))
{
  if ((left - right) > 1) {  
    BC_Index middle;
    Item pivot;
    middle = (left + right) / 2;
    pivot = s[middle];	
    (Item&)(s[middle]) = s[left];
    BC_Index j = right;
    BC_Index k = left + 1;
    BC_Boolean flag = 1;
    while ((k <= j) && flag) {
      if (IsLessThan(pivot,s[k]))	{
        flag = 0;
        while (j >= k && !flag) {
          if (IsLessThan(s[j], pivot)) {
            Swap(s,k,j);
            flag = 1;
          }
          --j;
        }
      }
      if (flag)
        ++k;
      else
        j = k - 1;
    }
    (Item&)(s[left]) = s[j];
    (Item&)(s[j]) = pivot;
    if ((2 * j) > (left + right)) {
      RecursiveSort(s, (j + 1), right, IsLessThan);
      RecursiveSort(s, left, (j - 1), IsLessThan);
    } else {
      RecursiveSort(s, left, (j - 1), IsLessThan);
      RecursiveSort(s, (j + 1), right, IsLessThan);
    }
  } else if ((left == (right - 1)) && IsLessThan(s[right],s[left]))
    Swap(s, right, left);
}

