#ifndef FWARDYNA_H
#define FWARDYNA_H
//========================================================================================
//
//	File:				FWArDyna.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef BCMAPD_H
#include "BCMapD.h"
#endif

#ifndef BCSTOREU_H
#include "BCStoreU.h"
#endif

#ifndef FWCLAINF_H
#include "FWClaInf.h"
#endif

#ifndef FWARSTRM_H
#include "FWArStrm.h"
#endif

//========================================================================================
// MACROS
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver
//
// TWM - Instances of Booch maps caused the compiler to issue an internal-type error
// message when the names were too long. Remove these macros when the bug is fixed.
//----------------------------------------------------------------------------------------

#define FW_CDynamicArchiver FWQ
#define FW_SPrivArcStr FWAS
#define FW_SPrivArcFun FWAF

//----------------------------------------------------------------------------------------
// FW_REGISTER_ARCHIVE_CLASS
//----------------------------------------------------------------------------------------

#define FW_REGISTER_ARCHIVE_CLASS(classLabel, classType, inputFunction, outputFunction) \
	FW_CDynamicArchiver FWArchive##classLabel(#classLabel, \
											  FW_CLASSNAME_FROM_TYPE(classType), \
											  FW_CDynamicArchiver::GetLocalMapNameToLabel(), \
											  FW_CDynamicArchiver::GetLocalMapLabelToIOFunction(), \
											  inputFunction, \
											  outputFunction);

//----------------------------------------------------------------------------------------
// FW_REGISTER_ARCHIVE_OLD_VERSION_LABEL
//----------------------------------------------------------------------------------------

#define FW_REGISTER_ARCHIVE_OLD_VERSION_LABEL(oldVersionClassLabel, inputFunction) \
	FW_CDynamicArchiver FWArchive##oldVersionClassLabel(#oldVersionClassLabel, \
											  FW_CDynamicArchiver::GetLocalMapLabelToIOFunction(), \
											  inputFunction);


//========================================================================================
// Forward Declarations
//========================================================================================

#ifndef FW_HAS_INSTANCE_DATA
struct FW_SPrivDynamicArchiverGlobals;
#endif

//========================================================================================
//	Struct FW_SPrivArcStr
//	An internal implementation class used in maps for string-type keys and values
//========================================================================================

struct FW_SPrivArcStr
{
	FW_SPrivArcStr();
	FW_SPrivArcStr(const FW_SPrivArcStr& string);
	FW_SPrivArcStr(const char* string);
	FW_SPrivArcStr& operator=(const FW_SPrivArcStr& string);
	BC_Boolean operator==(const FW_SPrivArcStr& string) const;
	BC_Boolean operator!=(const FW_SPrivArcStr& string) const;

	const char* fString;
};


//========================================================================================
//	Struct FW_SPrivArcFun
//	An internal implementation class used in maps for I/O function pointer values
//========================================================================================

struct FW_SPrivArcFun
{
	FW_SPrivArcFun();
	FW_SPrivArcFun(const FW_SPrivArcFun& ioFunctions);
	FW_SPrivArcFun(FW_CReadableArchive::InputFunction inputFunction,
		 FW_CWritableArchive::OutputFunction outputFunction);
	FW_SPrivArcFun& operator=(const FW_SPrivArcFun& ioFunctions);
	BC_Boolean operator==(const FW_SPrivArcFun& ioFunctions) const;
	BC_Boolean operator!=(const FW_SPrivArcFun& ioFunctions) const;

	FW_CReadableArchive::InputFunction fInputFunction;
	FW_CWritableArchive::OutputFunction fOutputFunction;
};

//========================================================================================
//	Class FW_CDynamicArchiver
//========================================================================================

class FW_CDynamicArchiver
{
	
public:

	static void InputObject(FW_CReadableArchive& readableArchive,
							void*& object);
		// Input an object of a dynamic class from the readableArchive.

	static void OutputObject(FW_CWritableArchive& writableArchive,
							 const void* object,
							 const char* className);
		// Output an object of a dynamic class to the writableArchive.
	
	static void MergeArchiverMaps();
		// Merge application's or component's archiver map into global archiver map.

protected:

	enum
	{
		kMaxClassLabelLength = 255	// Maximum allowed length of a class label
	};
	
	static void* ReadClassLabel(FW_CReadableArchive& readableArchive);

	static void WriteClassLabel(FW_CWritableArchive& writableArchive,
								const void* classLabel);


	// ----- Task global implementation items

public:

#ifndef FW_HAS_INSTANCE_DATA
	static void Initialize(FW_SPrivDynamicArchiverGlobals& globals);
	static void Terminate();
	static FW_SPrivDynamicArchiverGlobals& GetDynamicArchiverGlobals();
#endif

protected:

	enum
	{
		kDynamicArchiverGlobalsOffset = 158
	};


	// ----- Dynamic map implementation items

public:

	enum
	{
		kPrime = 101,				// Prime number for count of buckets in hash table
		kChunkSize = 50				// Chunk size for dynamic maps
	};

	typedef BC_TDynamicMap<FW_SPrivArcStr, FW_SPrivArcStr, (BC_Index)FW_CDynamicArchiver::kPrime, BC_CUnmanaged> MapNameToLabel;
	typedef BC_TDynamicMap<FW_SPrivArcStr, FW_SPrivArcFun, (BC_Index)FW_CDynamicArchiver::kPrime, BC_CUnmanaged> MapLabelToIOFunction;

	// ----- Internal methods that must be public

	static MapNameToLabel& GetLocalMapNameToLabel();
	
	static MapLabelToIOFunction& GetLocalMapLabelToIOFunction();

	FW_CDynamicArchiver(const char* classLabel,
						const char* className,
						FW_CDynamicArchiver::MapNameToLabel& localNameToLabelMap,
						FW_CDynamicArchiver::MapLabelToIOFunction& localLabelToIOFunctionMap,
						FW_CReadableArchive::InputFunction inputFunction,
						FW_CWritableArchive::OutputFunction outputFunction);
		// Register the <classLabel, className, inputFunction, outputFunction>
		// 4-tuple of a dynamic class with the archiver.

	FW_CDynamicArchiver(const char* oldVersionClassLabel,
						FW_CDynamicArchiver::MapLabelToIOFunction& localLabelToIOFunctionMap,
						FW_CReadableArchive::InputFunction inputFunction);
		// Registers the <classLabel, inputFunction> for an old label.

protected:

	static BC_Index HashFunction(const FW_SPrivArcStr& string);
	
	static MapNameToLabel& GetMapNameToLabel();
	
	static MapLabelToIOFunction& GetMapLabelToIOFunction();

	static MapNameToLabel * gLocalMapClassNameToClassLabel;
	
	static MapLabelToIOFunction * gLocalMapClassLabelToObjectIOFunction;

	static void MergeNameToLabelMaps(FW_CDynamicArchiver::MapNameToLabel& local,
									FW_CDynamicArchiver::MapNameToLabel& global);
	
	static void MergeLabelToIOFunctionMaps(FW_CDynamicArchiver::MapLabelToIOFunction& local,
									FW_CDynamicArchiver::MapLabelToIOFunction& global);

private:
	FW_CDynamicArchiver(const FW_CDynamicArchiver& archiver);
	FW_CDynamicArchiver& operator=(const FW_CDynamicArchiver& archiver);
		// Shouldn't need to copy this class.
};


#ifndef FW_HAS_INSTANCE_DATA
//========================================================================================
// STRUCT FW_SPrivDynamicArchiverGlobals
//========================================================================================

struct FW_SPrivDynamicArchiverGlobals
{
	FW_CDynamicArchiver::MapNameToLabel * gMapClassNameToClassLabel;
	FW_CDynamicArchiver::MapLabelToIOFunction * gMapClassLabelToObjectIOFunction;
};
#endif

#endif
