//========================================================================================
//
//	File:				FWArStrm.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWARSTRM_H
#include "FWArStrm.h"
#endif

#ifndef FWOBJREG_H
#include "FWObjReg.h"
#endif

#pragma segment FWArchiv


//========================================================================================
//	Constants having file scope
//========================================================================================

enum
{
	kIDOnly, kIDAndValue
};

//========================================================================================
//	Class FW_CReadableArchive
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CReadableArchive::FW_CReadableArchive
//----------------------------------------------------------------------------------------

FW_CReadableArchive::FW_CReadableArchive(FW_CSink* sink,
										 FW_CReadableStreamFormatter* formatter,
										 FW_CObjectRegistry* objectRegistry) :
	FW_CReadableStream(sink, formatter),
	fArchiveCreatedObjectRegistry(objectRegistry ? FALSE : TRUE),
	fObjectRegistry(objectRegistry)
{
	if (fArchiveCreatedObjectRegistry)
		fObjectRegistry = new FW_CBasicObjectRegistry();

	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
// FW_CReadableArchive::~FW_CReadableArchive
//----------------------------------------------------------------------------------------

FW_CReadableArchive::~FW_CReadableArchive()
{
	FW_START_DESTRUCTOR
	
	if (fArchiveCreatedObjectRegistry)
		delete fObjectRegistry;
}

//----------------------------------------------------------------------------------------
// FW_CReadableArchive::InputObject
//----------------------------------------------------------------------------------------

void* FW_CReadableArchive::InputObject(InputFunction inputFunction)
{
	FW_CObjectRegistry::ID id;
	short byValue;
	void* objectPtr;

	*this >> byValue;
	*this >> id;

	switch (byValue)
	{
		case kIDAndValue:
			objectPtr = inputFunction(*this);
			fObjectRegistry->Register(objectPtr, id);
			break;
		case kIDOnly:
			objectPtr = (void*)fObjectRegistry->Lookup(id);
			break;
		default:
			FW_ASSERT(FALSE);
			break;
	}

	return objectPtr;
}


//========================================================================================
//	Class FW_CWritableArchive
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CWritableArchive::FW_CWritableArchive
//----------------------------------------------------------------------------------------

FW_CWritableArchive::FW_CWritableArchive(FW_CSink* sink,
										 FW_CWritableStreamFormatter* formatter,
										 FW_CObjectRegistry* objectRegistry) :
	FW_CWritableStream(sink, formatter),
	fArchiveCreatedObjectRegistry(objectRegistry ? FALSE : TRUE),
	fObjectRegistry(objectRegistry)
{
	if (fArchiveCreatedObjectRegistry)
		fObjectRegistry = new FW_CBasicObjectRegistry();

	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
// FW_CWritableArchive::~FW_CWritableArchive
//----------------------------------------------------------------------------------------

FW_CWritableArchive::~FW_CWritableArchive()
{
	FW_START_DESTRUCTOR
	
	if (fArchiveCreatedObjectRegistry)
		delete fObjectRegistry;
}

//----------------------------------------------------------------------------------------
// FW_CWritableArchive::OutputObject
//----------------------------------------------------------------------------------------

void FW_CWritableArchive::OutputObject(OutputFunction outputFunction,
									   const void* object)
{
	FW_CObjectRegistry::ID id = fObjectRegistry->Lookup(object);

	if (id == FW_CObjectRegistry::kNotInRegistry)
	{
		id = fObjectRegistry->Register(object);
		*this << ((short)kIDAndValue) << id;
		// Explicitly cast the kIDOnly into a short because on the mac
		// an enum can be a char.

		outputFunction(*this, object);
	}
	else
	{
		*this << ((short)kIDOnly) << id;
		// Explicitly cast the kIDOnly into a short because on the mac
		// an enum can be a char.
	}
}
