#ifdef FW_DEBUG
// This file is only used for debugging
//========================================================================================
//
//	File:				FWPriDeb.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPRIDEB_H
#include "FWPriDeb.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(__TYPES__)
#include <Types.h>
#endif

#if defined(THINK_CPLUS) && !defined(__PASCAL__)
#include <Pascal.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(_INC_WINDOWS)
#include <Windows.h>
#endif

#ifndef FWTRACET_H
#include "FWTraceT.h"
#endif

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::FW_CDebugConsole
//----------------------------------------------------------------------------------------

FW_CDebugConsole::FW_CDebugConsole()
	: fBufferStream(fBuffer, kBufferSize)
{
	fFatalErrorStream = &fBufferStream;
	fWarningStream = &fNullStream;
	fNotificationStream = &fNullStream;
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::~FW_CDebugConsole
//----------------------------------------------------------------------------------------

FW_CDebugConsole::~FW_CDebugConsole()
{
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::GetConsole
//----------------------------------------------------------------------------------------

FW_CDebugConsole* FW_CDebugConsole::GetConsole()
{
	return FW_CPrivTraceTaskGlobals::GetTraceGlobals().gDebugConsole;
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::SetConsole
//----------------------------------------------------------------------------------------

FW_CDebugConsole* FW_CDebugConsole::SetConsole(FW_CDebugConsole* console)
{
	FW_CDebugConsole*& theConsole = FW_CPrivTraceTaskGlobals::GetTraceGlobals().gDebugConsole;
	FW_CDebugConsole* prior = theConsole;
	theConsole = console;
	return prior;
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::DoLogMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::DoLogMessage(const char* message)
{
	PrivDebugMessage(message);
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::DoDebugMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::DoDebugMessage(const char* message)
{
	PrivDebugMessage(message);
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::DoDebugger
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::DoDebugger()
{
	PrivDebugger();
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::LogMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::LogMessage(const char* message)
{
	FW_CDebugConsole *console = GetConsole();
	if (console)
		console->DoLogMessage(message);
	else
		PrivDebugMessage(message);
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::DebugMessage
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::DebugMessage(const char* message)
{
	FW_CDebugConsole *console = GetConsole();
	if (console)
		console->DoDebugMessage(message);
	else
		PrivDebugMessage(message);
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::Debugger
//----------------------------------------------------------------------------------------

void FW_CDebugConsole::Debugger()
{
	FW_CDebugConsole *console = GetConsole();
	if (console)
		console->DoDebugger();
	else
		PrivDebugger();
}

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::PrivDebugger
//----------------------------------------------------------------------------------------
void FW_CDebugConsole::PrivDebugger()
{
#if defined(FW_BUILD_WIN)
#ifndef FW_BUILD_DOS
	::DebugBreak();
#endif

#elif defined(FW_BUILD_MAC)
	::Debugger();
#endif
}

#ifdef THINK_CPLUS
//----------------------------------------------------------------------------------------
// debugstr
//----------------------------------------------------------------------------------------
void debugstr(const char *message)
{
	CtoPstr((char*) message);
	DebugStr((const unsigned char*) message);
	PtoCstr((unsigned char*) message);
}
#endif

//----------------------------------------------------------------------------------------
// FW_CDebugConsole::PrivDebugMessage
//----------------------------------------------------------------------------------------
void FW_CDebugConsole::PrivDebugMessage(const char *message)
{
#if defined(FW_BUILD_WIN)
#ifndef FW_BUILD_DOS
	::OutputDebugString(message);
	::OutputDebugString("\n\r");
	::DebugBreak();
#endif

#elif defined(THINK_CPLUS)
	::debugstr(message);
#elif defined(FW_BUILD_MAC)
	::debugstr((char*)message);
#endif
}


#endif

