//========================================================================================
//
//	File:				FWDelSta.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef   FWDELSTA_H
#include "FWDelSta.h"
#endif

#ifndef   FWEXCRUN_H
#include "FWExcRun.h"
#endif

#ifndef   FWPRIDEB_H
#include "FWPriDeb.h"
#endif

#ifndef   FWPRIMEM_H
#include "FWPriMem.h"
#endif

#ifndef   FWAUTODE_H
#include "FWAutoDe.h"
#endif

#ifndef   FWTRYBLO_H
#include "FWTryBlo.h"
#endif

#ifndef   FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef   FWCLAINF_H
#include "FWClaInf.h"
#endif

#ifdef FW_BUILD_MAC
#pragma segment BEL
#endif

//========================================================================================
// CLASS _FW_CDeleteStack
//========================================================================================

//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::Initialize
//----------------------------------------------------------------------------------------
void _FW_CDeleteStack::Initialize(FW_SPrivExceptionGlobals& globals)
{
	globals.gCurrentContext = NULL;
	globals.gBaseContext
		= (_FW_CTryBlockContext *) ::FW_PrimitiveAllocateBlock(sizeof(_FW_CTryBlockContext));
	FW_PRIV_ASSERT(globals.gBaseContext != NULL);
	_FW_CTryBlockContext *baseContext
		= new(globals.gBaseContext) _FW_CTryBlockContext(globals, 0);
	FW_PRIV_ASSERT(baseContext == globals.gCurrentContext);
	
#ifdef FW_DEBUG
	globals.gStackBase = (_FW_CDeleteEntry *)::FW_PrimitiveAllocateBlock(kStepByteSize+sizeof(_FW_CDeleteEntry));
#else
	globals.gStackBase = (_FW_CDeleteEntry *)::FW_PrimitiveAllocateBlock(kStepByteSize);
#endif

	FW_PRIV_ASSERT(globals.gStackBase != NULL);

	globals.gCurrentStackSize = kNumberOfObjectsToGrowStack;
	SetStackTop(globals, globals.gStackBase);
	globals.gStackLimit = globals.gStackBase + kNumberOfObjectsToGrowStack;
	
#ifdef FW_DEBUG
	CheckClassInvariants(globals);
#endif
}

//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::Terminate
//----------------------------------------------------------------------------------------
void _FW_CDeleteStack::Terminate()
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();
	::FW_PrimitiveFreeBlock(globals.gStackBase);
	::FW_PrimitiveFreeBlock(globals.gBaseContext);
}

//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::Push
//----------------------------------------------------------------------------------------
// Bottleneck
void _FW_CDeleteStack::Push(void* aGuardOrObject,
							_FW_CDeleteEntry::__DeleteType type)
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();

#ifdef FW_DEBUG
	CheckClassInvariants(globals);
#endif

	if (globals.gStackTop == globals.gStackLimit)
	{
		// Must grow stack before pushing next entry
		_FW_StackEntries prevSize = globals.gCurrentStackSize;
		_FW_StackEntries newSize = prevSize + kNumberOfObjectsToGrowStack;
#ifdef FW_DEBUG
		size_t newByteSize = (newSize+1) * sizeof(_FW_CDeleteEntry);
#else
		size_t newByteSize = newSize * sizeof(_FW_CDeleteEntry);
#endif

		void* p = ::FW_PrimitiveResizeBlock((void*)globals.gStackBase, newByteSize);

		FW_PRIV_ASSERT(p != NULL);

		globals.gStackBase = (_FW_CDeleteEntry *)p;
		globals.gStackTop = globals.gStackBase + prevSize;
		globals.gCurrentStackSize = newSize;
		globals.gStackLimit = globals.gStackBase + newSize;
	}
	_FW_CDeleteEntry * nextOpening = globals.gStackTop;

	nextOpening->fType = type;
	nextOpening->fObject = (_FW_CAutoDestructObject *)aGuardOrObject;
	if (aGuardOrObject != NULL)
		nextOpening->fObjectVTable = *(void**)aGuardOrObject;

	SetStackTop(globals, nextOpening + 1);

#ifdef FW_DEBUG
	if (type == _FW_CDeleteEntry::__kObject)
		globals.gCurrentContext->fAutomaticsInMyScope++;
	CheckClassInvariants(globals);
#endif
}

//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::IsClassExpectedInContext
//----------------------------------------------------------------------------------------
int _FW_CDeleteStack::IsClassExpectedInContext(_FW_CTryBlockContext *context,
											   FW_ClassReference  thrownType)
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();

	_FW_CDeleteEntry * contextLimit = globals.gStackBase + context->fDeleteStackLevel;
	_FW_CDeleteEntry * next = globals.gStackTop;

	FW_PRIV_ASSERT(globals.gStackBase != NULL);

	while (next > contextLimit)
	{
		next--;

		if (next->GetType() == _FW_CDeleteEntry::__kGuard)
		{
			FW_ClassReference  theGuardClass = next->GetGuard();
			if (!theGuardClass || !thrownType->IsKindOf(theGuardClass))
			{
				return 0;
			}
		}
	}
	return 1;
}

//----------------------------------------------------------------------------------------
// __CallDestructorWithNoVirtualBaseDelete (Macintosh version)
//----------------------------------------------------------------------------------------
#ifdef FW_BUILD_MAC
extern "C"
{
	#pragma parameter __CallDestructorWithNoVirtualBaseDelete(__D1)
	void __CallDestructorWithNoVirtualBaseDelete(void *)
		= {0x7000, 0x2f00, 0x2041, 0x2250, 0xd0e9, 0x0008,
			0x2f08, 0x2041, 0x2050, 0x2268, 0x000c, 0x4e91, 0x508f};
}
/*
00000                          	CallDestructorWithNoVirtualBaseDelete PROC
00000                          			; D1 has object
00000   7000                   			moveq	#0,d0				; don't delete virtual bases
00002   2F00                   			move.l	d0,-(a7)			; arguement to destructor
00004   2041                   			movea.l	d1,a0
00006   2250                   			movea.l	(a0),a1				; vtable for object
00008   D0E9 0008              			adda.w	8(a1),a0			; get offset to start of object
0000C   2F08                   			move.l	a0,-(a7)			; push object address
0000E   2041                   			movea.l	d1,a0
00010   2050                   			movea.l	(a0),a0				; vtable
00012   2268 000C              			movea.l	12(a0),a1			; destructor address
00016   4E91                   			jsr		(a1)
00018	508F							addq.l	#8,a7				; clean stack
0001A                          		ENDPROC
*/
#endif

//----------------------------------------------------------------------------------------
// __CallDestructorWithNoVirtualBaseDelete (Windows version)
//----------------------------------------------------------------------------------------
#ifdef __ZTCWIN__
void __CallDestructorWithNoVirtualBaseDelete(void * p)
{
    asm(0xC4, 0x5E, 0x06);          //  les     BX, [BP+6]      ; ES:BX <-- pObject
    asm(0x33, 0xC0);                //  xor     AX, AX          ; AX    <-- 0
    asm(0x50);                      //  push    AX              ; Push dtor argument
    asm(0x06);                      //  push    ES              ; Push pObject
    asm(0x53);                      //  push    BX
    asm(0x26, 0xC4, 0x1F);          //  les     BX, [ES:BX]     ; ES:BX <-- vmtPtr
    asm(0x26, 0xFF, 0x5F, 0x04);    //  call    [DWORD ES:BX+4] ; Call the dtor
}
#endif

//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::PopOffAndDeleteObjectsInContext
//----------------------------------------------------------------------------------------
void _FW_CDeleteStack::PopOffAndDeleteObjectsInContext(FW_SPrivExceptionGlobals& globals,
														_FW_CTryBlockContext *context)
{
#ifdef FW_DEBUG
	CheckClassInvariants(globals);
#endif

	_FW_StackEntries offset = context->fDeleteStackLevel;

	_FW_CDeleteEntry * popLimit = globals.gStackBase + offset;
	_FW_CDeleteEntry * popMe = globals.gStackTop;
	FW_PRIV_ASSERT(popMe >= popLimit);
	
	globals.gIsThrowing = 1;

	while (popMe > popLimit)
	{
		popMe--;

		if (popMe->GetType() == _FW_CDeleteEntry::__kObject)
		{
			_FW_CAutoDestructObject * lastDeleted = popMe->GetObject();
			FW_PRIV_ASSERT(lastDeleted != NULL);
			*(void**)lastDeleted = popMe->fObjectVTable;
	
			// Global must be set properly before __Delete(), which calls PopIfTopEquals()
			SetStackTop(globals, popMe+1);
#if 0
			// Disabled, but may need to restore this code later.
			__CallDestructorWithNoVirtualBaseDelete((void*)lastDeleted);
#else
			lastDeleted->__Delete();	// Virtual dispatch to object's destructor
#endif
	
			// The destructor of the deleted object will have popped one or more objects 
			// off the stack so we have to query the stack to find out who to pop next.
			popMe = globals.gStackTop;
		}
	}

	FW_PRIV_ASSERT(popMe == popLimit);
	SetStackTop(globals, popLimit);
	globals.gIsThrowing = 0;

#ifdef FW_DEBUG
	CheckClassInvariants(globals);
#endif
}


//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::PopIfTopEquals
//----------------------------------------------------------------------------------------
// Bottleneck
void _FW_CDeleteStack::PopIfTopEquals(_FW_CAutoDestructObject* anObject)
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();

#ifdef FW_DEBUG
	CheckClassInvariants(globals);
#endif

	_FW_CTryBlockContext *context = globals.gCurrentContext;
	_FW_StackEntries offset = context->fDeleteStackLevel;
	_FW_CDeleteEntry * popLimit = globals.gStackBase + offset;
	_FW_CDeleteEntry * popMe = globals.gStackTop;
	FW_PRIV_ASSERT(popMe > popLimit);

	// Normally, the top item on the stack will be the entry for anObject.
	// However, we may be deleting out of order, and there may be guards.
	// So, we must clean any gaurds and previously deleted items off the top
	// of the stack.  If the first kObject entry is our object, then we clean
	// it off too and exit.  If the first kObject entry is not our object, then
	// we need to search through the stack for our object and mark it deleted.
	// During the search, the stack top is not changed.

	// Clean off guards and deleted items.  Stack top is changed.
	while (popMe > popLimit)
	{
		popMe--;
		if (popMe->GetType() == _FW_CDeleteEntry::__kObject)
			break;
	}
	
	// There must still be at list one entry in this context.
	FW_PRIV_ASSERT(popMe >= popLimit);
	
	// And the top one must be an undeleted object entry.
	FW_PRIV_ASSERT(popMe->GetType() == _FW_CDeleteEntry::__kObject);

	FW_Boolean objectDeleted = FALSE;
	if (popMe->GetObject() == anObject)
	{
		// All is cool.  First kObject entry is our object.  Pop off entry.
		SetStackTop(globals, popMe);
		objectDeleted = TRUE;
	}
	else
	{
		// Need to remember where the stack top is now located.
		SetStackTop(globals, popMe+1);

		// Destroying out of order.  Search for our object and mark it destroyed.
		// Do not change stack top.

		while (popMe > popLimit)
		{
			popMe--;
			if (popMe->GetType() == _FW_CDeleteEntry::__kObject && popMe->GetObject() == anObject)
			{
				popMe->SetType(_FW_CDeleteEntry::__kDeleted);
				objectDeleted = TRUE;
				break;
			}
		}
	}

#ifdef FW_DEBUG
	if (objectDeleted)
		context->fAutomaticsInMyScope--;
	CheckClassInvariants(globals);
#endif
}

//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::PopGuard
//----------------------------------------------------------------------------------------
FW_ClassReference  _FW_CDeleteStack::PopGuard()
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();

#ifdef FW_DEBUG
	CheckClassInvariants(globals);
#endif

	_FW_CDeleteEntry * popMe = globals.gStackTop - 1;
	FW_ClassReference result = popMe->GetGuard();
	SetStackTop(globals, popMe);

#ifdef FW_DEBUG
	CheckClassInvariants(globals);
#endif

	return result;
}

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// _FW_CDeleteStack::CheckClassInvariants
//----------------------------------------------------------------------------------------
void _FW_CDeleteStack::CheckClassInvariants(FW_SPrivExceptionGlobals& globals)
{
	// Ensure class invariants
	FW_PRIV_ASSERT(globals.gStackBase != NULL);
	FW_PRIV_ASSERT(globals.gStackBase <= globals.gStackTop);
	FW_PRIV_ASSERT(globals.gStackTop <= globals.gStackLimit);
	FW_PRIV_ASSERT(globals.gCurrentStackSize >= kNumberOfObjectsToGrowStack);
	FW_PRIV_ASSERT(globals.gStackBase+globals.gCurrentStackSize == globals.gStackLimit);
	FW_PRIV_ASSERT(globals.gStackTop->GetType() == _FW_CDeleteEntry::__kInvalid);

	_FW_CTryBlockContext *context = globals.gCurrentContext;
	if (context != NULL)
	{
		context->CheckClassInvariants(globals);
		FW_PRIV_ASSERT(RelativeTopOfStack(globals) >= context->fDeleteStackLevel);
		FW_PRIV_ASSERT(RelativeTopOfStack(globals)-context->fDeleteStackLevel >= context->fAutomaticsInMyScope);
	}
}
#endif

//========================================================================================
// CLASS _FW_SThrowGuard
//========================================================================================

//----------------------------------------------------------------------------------------
// _FW_SThrowGuard::_FW_SThrowGuard
//----------------------------------------------------------------------------------------
_FW_SThrowGuard::_FW_SThrowGuard()
{
    _FW_CDeleteStack::Push((FW_ClassReference ) NULL);
#ifdef FW_DEBUG
	fGuardClass = (FW_ClassReference ) NULL;
#endif
}

//----------------------------------------------------------------------------------------
// _FW_SThrowGuard::_FW_SThrowGuard
//----------------------------------------------------------------------------------------
_FW_SThrowGuard::_FW_SThrowGuard(FW_ClassReference  guard)
{
    _FW_CDeleteStack::Push(guard);
#ifdef FW_DEBUG
	fGuardClass = guard;
#endif
}

//----------------------------------------------------------------------------------------
// _FW_SThrowGuard::~_FW_SThrowGuard
//----------------------------------------------------------------------------------------
_FW_SThrowGuard::~_FW_SThrowGuard()
{
    FW_ClassReference  aClass = _FW_CDeleteStack::PopGuard();
	FW_PRIV_ASSERT(aClass == fGuardClass);
}

