//========================================================================================
//
//	File:				FWNewHel.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWNEWHEL_H
#include "FWNewHel.h"
#endif

#ifndef FWEXCDEF_H
#include "FWExcDef.h"
#endif

#ifndef   FWPRIMEM_H
#include "FWPriMem.h"
#endif

#ifdef FW_BUILD_MAC
#pragma segment BEL
#endif

//========================================================================================
//	CLASS _FW_CPrivSubHelper
//========================================================================================

void _FW_CPrivSubHelper::InitializeStorageArray(FW_SPrivExceptionGlobals& globals,
												_FW_CPrivSubHelper*array, int size)
{
	for (int i=0; i<size-1; i++)
	{
		array[i].fSiblings = array+i+1;
#ifdef FW_DEBUG
		array[i].fSubObject = 0;
		array[i].fSubObjectSize = 0;
		array[i].fSubObjectVTable = 0;
#endif
	}
	array[size-1].fSiblings = NULL;
	globals.gSubHelperFreeListHead = array;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivSubHelper::operator new
//----------------------------------------------------------------------------------------

void* _FW_CPrivSubHelper::operator new(size_t size)
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();
	_FW_CPrivSubHelper* head = globals.gSubHelperFreeListHead;
	FW_PRIV_ASSERT(head != NULL);
	globals.gSubHelperFreeListHead = head->fSiblings;
	return (void*) head;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivSubHelper::operator delete
//----------------------------------------------------------------------------------------

void  _FW_CPrivSubHelper::operator delete(void* object)
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();
	_FW_CPrivSubHelper* subHelper = (_FW_CPrivSubHelper*) object;
	_FW_CPrivSubHelper* head = globals.gSubHelperFreeListHead;
	subHelper->fSiblings = head;
#ifdef FW_DEBUG
	subHelper->fSubObject = 0;
	subHelper->fSubObjectSize = 0;
	subHelper->fSubObjectVTable = 0;
#endif
	globals.gSubHelperFreeListHead = subHelper;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivSubHelper::_FW_CPrivSubHelper
//----------------------------------------------------------------------------------------

_FW_CPrivSubHelper::_FW_CPrivSubHelper(_FW_CAutoDestructObject *subObject, 
									   size_t subObjectSize,
									   _FW_CPrivSubHelper *nextSubHelper)
{
	fSubObject = subObject;
	fSubObjectSize = subObjectSize;
	fSubObjectVTable = *(void**)subObject;
	fSiblings = nextSubHelper;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivSubHelper::~_FW_CPrivSubHelper
//----------------------------------------------------------------------------------------

_FW_CPrivSubHelper::~_FW_CPrivSubHelper()
{
}

//========================================================================================
//	CLASS _FW_CPrivNewHelper
//========================================================================================

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::Initialize
//----------------------------------------------------------------------------------------

void _FW_CPrivNewHelper::Initialize(FW_SPrivExceptionGlobals& globals)
{
	globals.gNewHelperStackBottom = (_FW_CPrivNewHelper **) 
						    ::FW_PrimitiveAllocateBlock(kNewHelperStackSize);

	globals.gNewHelperStackTop = globals.gNewHelperStackBottom;
	
	globals.gSubHelperStorageArray = (_FW_CPrivSubHelper *) 
							::FW_PrimitiveAllocateBlock(kSubHelperArrayByteSize);

	_FW_CPrivSubHelper::InitializeStorageArray(globals,
							globals.gSubHelperStorageArray,
							kSubHelperArraySize);
}
	
//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::Terminate
//----------------------------------------------------------------------------------------

void _FW_CPrivNewHelper::Terminate()
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();
	::FW_PrimitiveFreeBlock(globals.gSubHelperStorageArray);
	::FW_PrimitiveFreeBlock(globals.gNewHelperStackBottom);
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::_FW_CPrivNewHelper
//----------------------------------------------------------------------------------------

_FW_CPrivNewHelper::_FW_CPrivNewHelper(__FW_OperatorNewHandler newHandler,
					  				__FW_OperatorDeleteHandler deleteHandler) :
	fWatchedObject(0),
	fObjectVTable(0),
	fWatchedSize(0),
	fSubHelperList(0),
	fNewHandler(newHandler),
	fDeleteHandler(deleteHandler)
{
	PushNewHelper();
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::~_FW_CPrivNewHelper
//----------------------------------------------------------------------------------------

_FW_CPrivNewHelper::~_FW_CPrivNewHelper()
{
	FW_START_DESTRUCTOR
	if (fWatchedObject != 0)
	{
		DeleteWatchedObject();
		_FW_CPrivNewHelper *temp = _FW_CPrivNewHelper::PopNewHelper();
		FW_PRIV_ASSERT(temp == this);
	}
	else
	{
		FW_PRIV_ASSERT(fSubHelperList == 0);
	}
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::DeleteWatchedObject
//----------------------------------------------------------------------------------------

void _FW_CPrivNewHelper::DeleteWatchedObject()
{
	_FW_CPrivSubHelper	*next = fSubHelperList;
	while (next != NULL)
	{
		_FW_CAutoDestructObject *subObject = next->fSubObject;
		*(void**)subObject = next->fSubObjectVTable;
		subObject->__Delete();

		// Save next->fSiblings before deleting next! (see operator delete)
		_FW_CPrivSubHelper	*temp = next;
		next = next->fSiblings;
		delete temp;
	}
	if (fObjectVTable != 0)
	{
		*(void**)fWatchedObject = fObjectVTable;
		fWatchedObject->__Delete();
	}
	_FW_CAutoDestructObject::operator delete((void*) fWatchedObject);
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::WatchObject
//
//		This method is not really const, 
//		but it reduces unnecessary warnings to declare it so.
//----------------------------------------------------------------------------------------

_FW_CAutoDestructObject * _FW_CPrivNewHelper::WatchObject(
								_FW_CAutoDestructObject *watchedObject,
								size_t watchedSize) const
{
	_FW_CPrivNewHelper *self = (_FW_CPrivNewHelper *) this;
	_FW_CAutoDestructObject *previous = fWatchedObject;
	self->fWatchedObject = watchedObject;
	self->fWatchedSize = watchedSize;
	return previous;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::ForgetObject
//
//		This method is not really const, 
//		but it reduces unnecessary warnings to declare it so.
//----------------------------------------------------------------------------------------

_FW_CAutoDestructObject * _FW_CPrivNewHelper::ForgetObject() const
{
	_FW_CPrivNewHelper *self = (_FW_CPrivNewHelper *) this;
	_FW_CAutoDestructObject *previous = fWatchedObject;
	self->fWatchedObject = 0;
	FW_PRIV_ASSERT(fSubHelperList == 0);
	return previous;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::PopNewHelper
//----------------------------------------------------------------------------------------

_FW_CPrivNewHelper* _FW_CPrivNewHelper::PopNewHelper()
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();
	_FW_CPrivNewHelper** top = globals.gNewHelperStackTop;
#ifdef FW_DEBUG
	_FW_CPrivNewHelper** bottom = globals.gNewHelperStackBottom;
	FW_PRIV_ASSERT(top > bottom);
#endif
	_FW_CPrivNewHelper*	helper = *--top;
	globals.gNewHelperStackTop = top;
	return helper;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::TopNewHelper
//----------------------------------------------------------------------------------------

_FW_CPrivNewHelper* _FW_CPrivNewHelper::TopNewHelper()
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();
	_FW_CPrivNewHelper** top = globals.gNewHelperStackTop;
	_FW_CPrivNewHelper** bottom = globals.gNewHelperStackBottom;
	_FW_CPrivNewHelper*	helper = 0;
	if (top > bottom)
		helper = top[-1];
	return helper;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::PushNewHelper
//----------------------------------------------------------------------------------------

void _FW_CPrivNewHelper::PushNewHelper()
{
	FW_SPrivExceptionGlobals& globals = FW_CExceptionTaskGlobals::GetExceptionGlobals();
	_FW_CPrivNewHelper** top = globals.gNewHelperStackTop;
#ifdef FW_DEBUG
	_FW_CPrivNewHelper** bottom = globals.gNewHelperStackBottom;
	FW_PRIV_ASSERT(top < bottom + kMaxNewHelperStackItems);
#endif
	*top++ = this;
	globals.gNewHelperStackTop = top;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivSubHelper::InList
//----------------------------------------------------------------------------------------

_FW_CPrivSubHelper* _FW_CPrivNewHelper::InList(_FW_CAutoDestructObject *subObject)
{
	_FW_CPrivSubHelper *next = fSubHelperList;
	while (next != NULL)
	{
		if (next->fSubObject == subObject)
			break;
		next = next->fSiblings;
	}
	return next;
}

//----------------------------------------------------------------------------------------
//	_FW_CPrivNewHelper::UpdateForEndConstructor
//----------------------------------------------------------------------------------------

void _FW_CPrivNewHelper::UpdateForEndConstructor(_FW_CAutoDestructObject *object, size_t size)
{
	if (object == fWatchedObject)
	{
		// Object is the object that was FW_NEWed.
		// Update the helper's copy of the vtbl pointer.
		fObjectVTable = *(void**)fWatchedObject;
		FW_PRIV_ASSERT(fObjectVTable != 0);
	}
	else
	{
		// Object must be a subobject of the object that was FW_NEWed.
		FW_PRIV_ASSERT(IsSubObject(object, fWatchedObject, fWatchedSize));
		
		// Is this subobject already attached, partially constructed?
		_FW_CPrivSubHelper* subHelper = InList(object);
		if (subHelper==NULL)
		{
			// No, create a new helper to track this subobject
			fSubHelperList = new _FW_CPrivSubHelper(object, size, fSubHelperList);
		}
		else
		{
			// Yes, update its vtable
			subHelper->fSubObjectVTable = *(void**)object;
			FW_PRIV_ASSERT(subHelper->fSubObjectVTable != 0);
		}
	}
		
	// Now, release any nested subobjects of this object
	// First, remove from head of list, updating fSubHelperList
	while (fSubHelperList != NULL && IsSubObject(fSubHelperList->fSubObject, object, size))
	{
		// Save fSubHelperList->fSiblings before deleting fSubHelperList!
		_FW_CPrivSubHelper *temp = fSubHelperList;
		fSubHelperList = fSubHelperList->fSiblings;
		delete temp;
	}
	
	// Second, remove from internal to list, with fSubHelperList frozen
	if (fSubHelperList != NULL)
	{
		FW_PRIV_ASSERT(!IsSubObject(fSubHelperList->fSubObject, object, size));
		_FW_CPrivSubHelper* prev = fSubHelperList;
		_FW_CPrivSubHelper* next = prev->fSiblings;
		while (next!=NULL)
		{
			if (IsSubObject(next->fSubObject, object, size))
			{
				// unlink & delete next.
				prev->fSiblings = next->fSiblings;
				delete next;
				next = prev->fSiblings;
			}
			else
			{
				// advance prev to next
				prev = next;
				next = prev->fSiblings;
			}
		}
	}
}

