//========================================================================================
//
//	File:				FWTryBlo.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef   FWTRYBLO_H
#include "FWTryBlo.h"
#endif

#ifndef   FWEXCRUN_H
#include "FWExcRun.h"
#endif

#ifndef   FWDELSTA_H
#include "FWDelSta.h"
#endif

#ifdef FW_BUILD_MAC
#pragma segment BEL
#endif

//========================================================================================
// CLASS _FW_CTryBlockContext
//========================================================================================

#pragma trace off

//----------------------------------------------------------------------------------------
// _FW_CTryBlockContext::~_FW_CTryBlockContext
//----------------------------------------------------------------------------------------
_FW_CTryBlockContext::~_FW_CTryBlockContext()
{
#ifdef FW_DEBUG
	CheckClassInvariants(fGlobals);
	fGlobals.gContextDepth--;
	FW_PRIV_ASSERT(fAutomaticsInMyScope == 0);
#endif
		
    _FW_CTryBlockContext *priorContext = MakeCurrent(fGlobals, fPriorContext);
	FW_PRIV_ASSERT(priorContext == this);
}

//----------------------------------------------------------------------------------------
// _FW_CTryBlockContext::_FW_CTryBlockContext
//----------------------------------------------------------------------------------------
_FW_CTryBlockContext::_FW_CTryBlockContext(FW_SPrivExceptionGlobals& globals, jmp_buf buffer) :
	fGlobals(globals),
	fJumpBuffer((jmp_buf *) buffer)
{
    fPriorContext = MakeCurrent(fGlobals, this);
    fDeleteStackLevel = _FW_CDeleteStack::RelativeTopOfStack(fGlobals);

#ifdef FW_DEBUG
	fGlobals.gContextDepth++;
	fAutomaticsInMyScope = 0;
	CheckClassInvariants(fGlobals);
#endif
}

//----------------------------------------------------------------------------------------
// _FW_CTryBlockContext::MakeCurrent
//----------------------------------------------------------------------------------------
class _FW_CTryBlockContext *_FW_CTryBlockContext::MakeCurrent(
											FW_SPrivExceptionGlobals& globals,
											_FW_CTryBlockContext *context)
{
	_FW_CTryBlockContext *tmpContext = globals.gCurrentContext;
	globals.gCurrentContext = context;
	return tmpContext;
}

#ifdef FW_DEBUG
//----------------------------------------------------------------------------------------
// _FW_CTryBlockContext::CheckClassInvariants
//
//		Should only be called on current context.
//----------------------------------------------------------------------------------------
void _FW_CTryBlockContext::CheckClassInvariants(FW_SPrivExceptionGlobals& globals)
{
	// Ensure class invariants
	FW_PRIV_ASSERT(&globals == &fGlobals);
	FW_PRIV_ASSERT(globals.gContextDepth>=1);
	FW_PRIV_ASSERT(globals.gContextDepth>1 || fPriorContext==NULL);
	FW_PRIV_ASSERT(globals.gContextDepth==1 || fPriorContext!=NULL);
	FW_PRIV_ASSERT(fAutomaticsInMyScope >= 0);
	FW_PRIV_ASSERT(fDeleteStackLevel >= 0);
	FW_PRIV_ASSERT(globals.gCurrentContext == this);
}
#endif

