//========================================================================================
//
//	File:				FWBaseFr.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWBASEFR_H
#define FWBASEFR_H

#ifndef FWSTDDEF_H
#include <FWStdDef.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _FRAME_
#include <Frame.h>
#endif

//========================================================================================
//  Forward Declaration
//========================================================================================

class XMPShape;
class XMPWindow;
class XMPStorageUnit;
struct FW_CPoint;
struct FW_CRect;

//========================================================================================
//  FW_CBaseFrame Definition
//========================================================================================

#define kFW_CBaseFrameID "appl:framework:cbaseframe$class,1.0.0"

class FW_CBaseFrame
{
//----------------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	FW_CBaseFrame();
	void InitBaseFrame(XMPFrame* frame);
	virtual ~ CDECL FW_CBaseFrame();

//----------------------------------------------------------------------------------------
//	 API from XMPFrame (inlines)
//
public:
	unsigned long		GetFrameGroup() const;
	void				SetFrameGroup(unsigned long groupID);
						
	FW_Boolean			IsRoot() const;

	FW_Boolean			IsSubframe() const;
	void				SetSubframe(FW_Boolean isSubframe);

	FW_Boolean			IsOverlaid() const;

	FW_Boolean			IsFrozen() const;
	void				SetFrozen(FW_Boolean isFrozen);

	FW_Boolean			DoesPropagateEvents() const;
	void				SetPropagateEvents(FW_Boolean doesPropagateEvents);

	XMPTypeToken		GetPresentation() const;
	XMPTypeToken		GetViewType() const;

	void 				SetDragging(FW_Boolean dragging);
	void				SetDroppable(FW_Boolean droppable);
		
	XMPShape*			GetUsedShape() const;
	XMPTransform* 		GetInternalTransform() const;
	
//----------------------------------------------------------------------------------------
//	 New API
//
public:
	// ----- Conversion -----
	operator XMPFrame*() const
		{return fXMPFrame;}
		
	// ----- Geometry -----
	void				GetFrameShape(XMPShape* shape) const;
	void				GetFrameShapeBounds(FW_CRect* rect) const;
	
	void				GetExtendShape(XMPShape* shape) const;
	void 				GetExtendRect(FW_CRect* rect) const;
	
	virtual void		PlaceFrame(const FW_CPoint& position);
	virtual void		OffsetFrame(const FW_CPoint& offset);
	virtual void		ResizeFrame(XMPShape* newShape);
	
	// ----- Drawing -----
	void				DrawAllFacets(XMPShape *invalidShape);
	
	virtual void 		Invalidate(XMPShape *invalidShape);
	virtual void 		Validate(XMPShape *validShape);
	
	// ----- Getters/Setters -----
	XMPFrame*			GetXMPFrame() const;
	XMPPart*			GetXMPPart() const;
	
//----------------------------------------------------------------------------------------
//	 Data Members
//
private:
	XMPFrame			*fXMPFrame;
};

//========================================================================================
//  Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::GetXMPFrame
//----------------------------------------------------------------------------------------
inline XMPFrame* FW_CBaseFrame::GetXMPFrame() const
{
	return fXMPFrame;
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::GetXMPPart
//----------------------------------------------------------------------------------------
inline XMPPart* FW_CBaseFrame::GetXMPPart() const
{
	return fXMPFrame->GetPart();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::GetFrameGroup
//----------------------------------------------------------------------------------------
inline unsigned long FW_CBaseFrame::GetFrameGroup() const
{
	return fXMPFrame->GetFrameGroup();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::SetFrameGroup
//----------------------------------------------------------------------------------------
inline void FW_CBaseFrame::SetFrameGroup(unsigned long groupID)
{
	fXMPFrame->SetFrameGroup(groupID);
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::IsRoot
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CBaseFrame::IsRoot() const
{
	return fXMPFrame->IsRoot();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::IsOverlaid
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CBaseFrame::IsOverlaid() const
{
	return fXMPFrame->IsOverlaid();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::IsFrozen
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CBaseFrame::IsFrozen() const
{
	return fXMPFrame->IsFrozen();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::SetFrozen
//----------------------------------------------------------------------------------------
inline void FW_CBaseFrame::SetFrozen(FW_Boolean isFrozen)
{
	fXMPFrame->SetFrozen(isFrozen);
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::DoesPropagateEvents
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CBaseFrame::DoesPropagateEvents() const
{
	return fXMPFrame->DoesPropagateEvents();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::SetPropagateEvents
//----------------------------------------------------------------------------------------
inline void FW_CBaseFrame::SetPropagateEvents(FW_Boolean doesPropagateEvents)
{
	fXMPFrame->SetPropagateEvents(doesPropagateEvents);
}

//----------------------------------------------------------------------------------------
//	 FW_CBaseFrame::GetViewType
//----------------------------------------------------------------------------------------
inline XMPTypeToken FW_CBaseFrame::GetViewType() const
{
	return fXMPFrame->GetViewType();
}

//----------------------------------------------------------------------------------------
//	 FW_CBaseFrame::GetPresentation
//----------------------------------------------------------------------------------------
inline XMPTypeToken FW_CBaseFrame::GetPresentation() const
{
	return fXMPFrame->GetPresentation();
}

//----------------------------------------------------------------------------------------
//	 FW_CBaseFrame::SetDragging
//----------------------------------------------------------------------------------------
inline void FW_CBaseFrame::SetDragging(FW_Boolean dragging)
{
	fXMPFrame->SetDragging(dragging);
}

//----------------------------------------------------------------------------------------
//	 FW_CBaseFrame::SetDroppable
//----------------------------------------------------------------------------------------
inline void FW_CBaseFrame::SetDroppable(FW_Boolean droppable)
{
	fXMPFrame->SetDroppable(droppable);
}

//----------------------------------------------------------------------------------------
//	 FW_CBaseFrame::IsSubframe
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CBaseFrame::IsSubframe() const
{
	return fXMPFrame->IsSubframe();
}

//----------------------------------------------------------------------------------------
//	 FW_CBaseFrame::SetSubframe
//----------------------------------------------------------------------------------------
inline void FW_CBaseFrame::SetSubframe(FW_Boolean isSubframe)
{
	fXMPFrame->SetSubframe(isSubframe);
}

//----------------------------------------------------------------------------------------
//	 FW_CBaseFrame::SetSubframe
//----------------------------------------------------------------------------------------
inline XMPShape* FW_CBaseFrame::GetUsedShape() const
{
	return fXMPFrame->GetUsedShape();
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::GetInternalTransform()
//----------------------------------------------------------------------------------------
inline XMPTransform* FW_CBaseFrame::GetInternalTransform() const
{
	return fXMPFrame->GetInternalTransform();
}


#endif
