//========================================================================================
//
//	File:				FWBorder.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWBORDER_H
#define FWBORDER_H

// ----- Foundation Includes -----

#ifndef FWSTDDEF_H
#include <FWStdDef.h>
#endif

#ifndef FWGRDEF_H
#include "FWGrDef.h"
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class FW_CProxyRun;
class FW_CPoint;
class FW_CRect;
class XMPShape;
class FW_CFacet;
class XMPFacet;

//==============================================================================
//	 Constants
//==============================================================================

const short kXMPBorderWidth = 5;
const short kXMPHandleLenMultiplier = 3;

const short kInTopLeftCorner = 1;
const short kInTopRightCorner = 2;
const short kInBottomLeftCorner = 3;
const short kInBottomRightCorner = 4;

const short kInTopEdge = 5;
const short kInRightEdge = 6;
const short kInBottomEdge = 7;
const short kInLeftEdge = 8;

const short kInDrag = 100;

//==============================================================================
//	 class FW_CBaseProxyBorder
//==============================================================================

#define kFW_CBaseProxyBorderID "appl:framework:cbaseproxyborder$class,1.0.0"

class FW_CBaseProxyBorder
{
//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder Initialization/Destruction
//
public:
	FW_CBaseProxyBorder();
	void InitBaseProxyBorder(FW_CProxyRun* proxyRun);
	virtual ~FW_CBaseProxyBorder();

//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder New API
//
public:
	// ----- Drawing -----
	virtual void		DrawProxyBorder(FW_CFacet* facet, XMPFacet* embeddedFacet) = 0;
	virtual void		EraseProxyBorder(FW_CFacet* facet, XMPFacet* embeddedFacet) = 0;
	
	// ----- Hit Test -----
	virtual short	WhereInProxyBorder(FW_CFacet* facet, const FW_CPoint& mouse) const = 0;
	
	// ----- Invalid Border Cache -----
	void				BorderHasChanged(FW_Boolean state);
	FW_Boolean			HasBorderChanged();
	
	// ----- Getters/Setters -----
	FW_CProxyRun*			GetProxyRun() const;
							
//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder Data Members
//
private:	
	FW_CProxyRun			*fProxyRun;
	FW_Boolean			fBorderHasChanged;
};

//==============================================================================
//	 class FW_CProxyBorder
//==============================================================================

class FW_CProxyBorder : public FW_CBaseProxyBorder
{
//------------------------------------------------------------------------------
//	 FW_CProxyBorder Initialization/Destruction
//
public:
	FW_CProxyBorder();
	void InitProxyBorder(FW_CProxyRun* proxyRun);
	virtual ~FW_CProxyBorder();

//------------------------------------------------------------------------------
//	 FW_CProxyBorder Inherited API
//
public:
	// ----- Drawing -----
	void				DrawProxyBorder(FW_CFacet* facet, XMPFacet* embeddedFacet);
	void				EraseProxyBorder(FW_CFacet* facet, XMPFacet* embeddedFacet);
	
	// ----- Hit Test -----
	short			WhereInProxyBorder(FW_CFacet* facet, const FW_CPoint& mouse) const;
	
//------------------------------------------------------------------------------
//	 FW_CProxyBorder New API
//
public:
	// ----- Calc/Dispose -----
	void				CalcProxyBorder(FW_CFacet* facet, XMPFacet *embeddedXMPFacet);
	virtual void 		GetContentClip(FW_CFacet* facet, const FW_CRect& borderBox, XMPShape* clip);

private:
	void				DisposeSelectionRgn();

//------------------------------------------------------------------------------
//	 FW_CProxyBorder Data Members
//
private:	
	FW_PlatformRegion	fSelectRgn;
	FW_PlatformRegion	fCornerHandleRgn;
	FW_PlatformRegion	fEdgeHandleRgn;
};

//==============================================================================
//	 FW_CBaseProxyBorder Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder::BorderHasChanged
//------------------------------------------------------------------------------
inline void FW_CBaseProxyBorder::BorderHasChanged(FW_Boolean state)
{
	fBorderHasChanged = state;
}

//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder::HasBorderChanged
//------------------------------------------------------------------------------
inline FW_Boolean FW_CBaseProxyBorder::HasBorderChanged()
{
	return fBorderHasChanged;
}
	
//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder::GetProxyRun
//------------------------------------------------------------------------------
inline FW_CProxyRun* FW_CBaseProxyBorder::GetProxyRun() const
{
	return  fProxyRun;
}

#endif

