//========================================================================================
//
//	File:				FWCtlMgr.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCTLMGR_H
#define FWCTLMGR_H

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

// ----- Graphic Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__CONTROLS__)
#include <Controls.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class FW_CGraphicContext;

//==============================================================================
//	 class FW_CCtlMgrPart
//==============================================================================

class FW_CCtlMgrPart : public FW_CPart
{

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart Initialization/Destruction
//
public:
	virtual ~ CDECL FW_CCtlMgrPart();

protected:
	FW_CCtlMgrPart();
	
	virtual void Initialize();

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart Inherited API
//
public:
	virtual FW_CFrame*			NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation);
	
//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart New API
//
public:
	ControlHandle				GetControlHandle() const;
									
	void						SetControlSize(const FW_CPoint& controlSize);
	void						GetControlSize(FW_CPoint& controlSize) const;

	void						SetAsDefaultButton(FW_Boolean state);
	FW_Boolean					IsDefaultButton() const;
	virtual void				DoClick(const FW_CPoint& where, XMPEventData event);

protected:
	virtual ControlHandle		CreateControlHandle(XMPPlatformWindow window, const FW_CRect& controlRect) = 0;
									
//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart Data Members
//
private:
	ControlHandle		fControlHandle;
	FW_CPoint			fControlSize;
	FW_Boolean 			fDefaultButton;
};

//==============================================================================
//	 class FW_CCtlMgrFrame
//==============================================================================

#define kFW_CCtlMgrFrameID "appl:framework:cctlmgrframe$class,1.0.0"

class FW_CCtlMgrFrame : public FW_CFrame
{
//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame Initialization/Destruction
//
public:
	FW_CCtlMgrFrame();
	virtual void InitCtlMgrFrame(XMPFrame* frame, FW_CPart* part);
	virtual ~ CDECL FW_CCtlMgrFrame();
		
//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame Inherited API
//
public:
	virtual FW_CFacet*		NewFacet(XMPFacet* xmpFacet);
	virtual void			FacetAdded(FW_CFacet* facet);
	virtual void			FrameShapeChanged();	
};

//==============================================================================
//	 class FW_CCtlMgrFacet
//==============================================================================

#define kFW_CCtlMgrFacetID "appl:framework:cctlmgrfacet$class,1.0.0"

class FW_CCtlMgrFacet : public FW_CFacet
{

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet Initialization/Destruction
//
public:
	FW_CCtlMgrFacet();
	virtual void InitCtlMgrFacet(XMPFacet* xmpFacet);
	virtual ~FW_CCtlMgrFacet();
		

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet Inherited API
//
public:
	virtual void		Draw(FW_CGraphicContext *gc);
	virtual FW_Boolean DoMouseDown(const FW_CPoint& where, XMPEventData event);


//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet New API
//
public:
	ControlHandle		GetControlHandle() const;

protected:
	ControlHandle		SetupControl();
	void				RestoreControl();
};




//==============================================================================
//	 class FW_CCtlMgrPart inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::GetControlHandle
//------------------------------------------------------------------------------

inline ControlHandle FW_CCtlMgrPart::GetControlHandle() const
{
	return fControlHandle;
}
									
//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::GetControlSize
//------------------------------------------------------------------------------

inline void FW_CCtlMgrPart::GetControlSize(FW_CPoint& controlSize) const
{
	controlSize = fControlSize;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::SetAsDefaultButton
//------------------------------------------------------------------------------
inline void FW_CCtlMgrPart::SetAsDefaultButton(FW_Boolean state)
{
	fDefaultButton = state;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::IsDefaultButton
//------------------------------------------------------------------------------
inline FW_Boolean FW_CCtlMgrPart::IsDefaultButton() const
{
	return fDefaultButton;
}



//==============================================================================
//	 class FW_CCtlMgrFacet inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::GetControlHandle
//------------------------------------------------------------------------------
inline ControlHandle FW_CCtlMgrFacet::GetControlHandle() const
{
	return ((FW_CCtlMgrPart*)GetFrame()->GetPart())->GetControlHandle();
}


#endif
