//========================================================================================
//
//	File:				FWFacet.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFACET_H
#define FWFACET_H

#ifndef FWSTDDEF_H
#include <FWStdDef.h>
#endif

#ifndef FWGRDEF_H
#include "FWGrDef.h"
#endif
// ----- OpenDoc Includes -----

#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPWindow;
class XMPShape;
class FW_CFrame;
class FW_CProxyFrame;
class FW_CProxyRun;
class FW_CWindow;
class FW_CDropTracker;
class FW_CGraphicContext;
struct FW_CPoint;

//==============================================================================
//	 class FW_CFacet
//==============================================================================

#define kFW_CFacetID "appl:framework:cfacet$class,1.0.0"

class FW_CFacet
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	FW_CFacet();
	void InitFacet(XMPFacet* xmpFacet);
	virtual ~FW_CFacet();

//------------------------------------------------------------------------------
//	 New API
//
public:
	virtual void 			Draw(FW_CGraphicContext* gc);

	void					Invalidate(XMPShape *invalidShape);
	void					Validate(XMPShape *validShape);
	
	void					InvalidateAllFacets(XMPShape* invalidShape, FW_Boolean skipThis);
	void					ValidateAllFacets(XMPShape* validShape, FW_Boolean skipThis);
	
	// ----- Conversion -----
	operator XMPFacet*() const
		{return fXMPFacet;}
		
	// ----- Drag And Drop -----
	virtual FW_PlatformRegion 		CalcDragHiliteRgn();
	virtual FW_Boolean		DrawDragHiliteInside();
	virtual void 			DragEnter(XMPDragItemIterator* dragInfo,  
										const FW_CPoint& where);
	virtual void 			DragWithin(XMPDragItemIterator* dragInfo,
										const FW_CPoint& where);
	virtual void 			DragLeave(const FW_CPoint& where);
	virtual XMPDropResult 	Drop(XMPDragItemIterator* dropInfo,
								  const FW_CPoint& where);
	virtual FW_CDropTracker* NewDropTracker(FW_CFacet* facet);

	// ----- Events -----
	virtual FW_Boolean 	HandleMouseDown(XMPFacet* embeddedXMPFacet, XMPEventData event);
	virtual FW_Boolean 	DoMouseDown(const FW_CPoint& where,
										XMPEventData event);
	
	virtual FW_Boolean 	HandleMouseUp(XMPEventData event);
	virtual FW_Boolean 	DoMouseUp(const FW_CPoint& where,
										XMPEventData event);

	virtual void			MouseEnter(const FW_CPoint& where);
	virtual void			MouseWithin(const FW_CPoint& where);
	virtual void			MouseLeave();

	virtual FW_Boolean		DoMouseDownInEmbeddedFrame(XMPFacet* embeddedXMPFacet, 
														const FW_CPoint& where, 
														XMPEventData event);
	virtual FW_Boolean		HandleMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, 
																 XMPEventData event);
	virtual FW_Boolean		DoMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, 
															const FW_CPoint& where, 
															XMPEventData event);
	// ----- Imaging -----
	virtual void			GeometryChanged();
	virtual void 			ClipEmbeddedFacets();
	virtual void 			UpdateActiveShape();


	// ----- Activation -----
	virtual void 			WindowActivated();
	virtual void			WindowDeActivated();
	
	// ----- Getters/Setters -----
	FW_CFrame*				GetFrame() const;
	XMPShape*				GetClipShape() const;
	
	// ----- XMP/FW_CFacet mapping -----
	static FW_CFacet* 		XMPtoFWFacet(XMPFacet* xmpFacet);
	
	// ----- Facet Hierarchy -----
	XMPFacet* 				GetEmbeddedXMPFacet(FW_CProxyRun* proxyRun);
	void					RemoveEmbeddedFacets();
	virtual XMPFacet*		CreateEmbeddedFacet(FW_CProxyFrame *itsFrame, XMPShape* clipShape, XMPTransform* externalTransform, 
						   		 				XMPFacet* siblingFacet, XMPFramePosition position);
	
	// ----- Transforms -----
	XMPTransform* 			GetExternalTransform() const;
	XMPTransform* 			GetFrameTransform() const;
	XMPTransform* 			GetContentTransform() const;
	XMPTransform* 			GetWindowFrameTransform() const;
	XMPTransform* 			GetWindowContentTransform() const;
	
	// ----- Getters/Setters -----
	FW_CWindow*				GetWindow() const;
	XMPWindow*				GetXMPWindow() const;
	XMPFacet*				GetXMPFacet() const;
	FW_Boolean				IsInFloatingWindow() const;
	FW_Boolean				GetCursorInContent() const;
	void					SetCursorInContent(FW_Boolean inContent);
	FW_Boolean				GetCanAcceptDrop() const;
	void					SetCanAcceptDrop(FW_Boolean acceptDrop);
	
	// ----- Redraw -----
	FW_Boolean				GetRedraw() const;
	FW_Boolean				SetRedraw(FW_Boolean state);
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	XMPFacet*				fXMPFacet;	
	FW_CFrame*				fFrame;
	FW_Boolean				fRedraw;
	FW_Boolean				fCursorInContent;		// Used for drag hiliting
	FW_Boolean				fCanAcceptDrop;
	FW_CDropTracker*		fDropTracker;
};

//==============================================================================
//	 FW_CFacet Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CFacet::GetRedraw
//------------------------------------------------------------------------------
inline FW_Boolean FW_CFacet::GetRedraw() const
{
	return fRedraw;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::GetClipShape
//------------------------------------------------------------------------------
inline XMPShape* FW_CFacet::GetClipShape() const
{
	return fXMPFacet->GetClipShape();
}

//------------------------------------------------------------------------------
//	 FW_CFacet::GetFrame
//------------------------------------------------------------------------------
inline FW_CFrame* FW_CFacet::GetFrame() const
{
	return fFrame;
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetXMPFacet
//----------------------------------------------------------------------------------------
inline XMPFacet* FW_CFacet::GetXMPFacet() const
{
	return fXMPFacet;
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetXMPWindow
//----------------------------------------------------------------------------------------
inline XMPWindow* FW_CFacet::GetXMPWindow() const
{
	return fXMPFacet->GetWindow();
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetExternalTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* FW_CFacet::GetExternalTransform() const
{
	return fXMPFacet->GetExternalTransform();
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetFrameTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* FW_CFacet::GetFrameTransform() const
{
	return fXMPFacet->GetFrameTransform();
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetContentTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* FW_CFacet::GetContentTransform() const
{
	return fXMPFacet->GetContentTransform();
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetWindowFrameTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* FW_CFacet::GetWindowFrameTransform() const
{
	return fXMPFacet->GetWindowFrameTransform();
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetWindowContentTransform
//----------------------------------------------------------------------------------------
inline XMPTransform* FW_CFacet::GetWindowContentTransform() const
{
	return fXMPFacet->GetWindowContentTransform();
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::IsInFloattingWindow()
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFacet::IsInFloatingWindow() const
{
	return GetXMPWindow()->IsFloating();
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetCursorInContent()
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFacet::GetCursorInContent() const
{
	return fCursorInContent;
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::SetCursorInContent()
//----------------------------------------------------------------------------------------
inline void FW_CFacet::SetCursorInContent(FW_Boolean inContent)
{
	fCursorInContent = inContent;
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::GetCanAcceptDrop()
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFacet::GetCanAcceptDrop() const
{
	return fCanAcceptDrop;
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::SetCanAcceptDrop()
//----------------------------------------------------------------------------------------
inline void FW_CFacet::SetCanAcceptDrop(FW_Boolean acceptDrop)
{
	fCanAcceptDrop = acceptDrop;
}

#endif