//========================================================================================
//
//	File:				FWFrame.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFRAME_H
#define FWFRAME_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWBASEFR_H
#include "FWBaseFr.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

//========================================================================================
//	 Forward Declaration
//========================================================================================

class FW_CPart;
class FW_CFacet;
class XMPFacet;
class XMPFocusSet;
class XMPMenuBar;

//========================================================================================
//	 Defines
//========================================================================================

#define kNoSelection 0
#define kInactiveSelection 1
#define kActiveSelection 2

//========================================================================================
//	 Type Definitions
//========================================================================================

typedef unsigned short SelectionState;

//========================================================================================
//	 class FW_CFrame
//========================================================================================

#define kFW_CFrameID "appl:framework:cframe$class,1.0.0"

class FW_CFrame : public FW_CBaseFrame, public FW_MEventHandler
{
//----------------------------------------------------------------------------------------
//	 New API
//
public:
	FW_CFrame();
	void InitFrame(XMPFrame* frame, FW_CPart* part);
	virtual ~ CDECL FW_CFrame();

//----------------------------------------------------------------------------------------
//	 New API
//
public:
	static FW_CFrame* XMPtoFWFrame(XMPFrame* xmpFrame);

	// ----- Events -----
	virtual FW_Boolean 	HandleKeyDown(XMPEventData event);
	virtual FW_Boolean 	DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	virtual FW_Boolean 	DoIdle();

	// ----- Facets -----
	virtual FW_CFacet* 		NewFacet(XMPFacet* xmpFacet) = 0;
		// 	In NewFacet just create your subclass of FW_CFacet. Wait for 
		//	FacetAdded before doing anything else.
		
	virtual void 			FacetAdded(FW_CFacet* facet);
	virtual void 			FacetRemoved(FW_CFacet* facet);
		
	void					InvalidateAllFacets(FW_CFacet* skipFacet, XMPShape* invalidShape);
	void					ValidateAllFacets(FW_CFacet* skipFacet, XMPShape* validShape);

	virtual void			Invalidate(XMPShape* invalidShape);
	virtual void			Validate(XMPShape* validShape);

	// ----- Add/Remove frame notification -----
	virtual void			AddedToDisplayList();
	virtual void			CloseDisplayFrame();
	virtual void			RemoveDisplayFrame();
	
	// ----- Focus Set -----
	XMPFocusSet*			GetFocusSet() const;
	void 					AddToFocusSet(XMPTypeToken focus);	
	void 					RemoveFromFocusSet(XMPTypeToken focus);	
	
	// ----- Menus -----
	virtual void			DoAdjustMenus(XMPMenuBar* menuBar);
	virtual void			InstallMenus();
	
	// ----- Focus -----
	virtual void 			FocusStateChanged(XMPTypeToken focus, FW_Boolean newState);
	virtual FW_Boolean 	ActivateFrame(FW_CFacet* activatedFacet);
	virtual void			DeactivateFrame();
	FW_Boolean				HasSelectionFocus() const;
	
	// ----- ViewType/Presentation/Sequence -----
	virtual void 			ViewTypeChanged();
	virtual void 			PresentationChanged();
	virtual void 			SequenceChanged();

	// ----- Frame Negotiation -----
	virtual void 			FrameShapeChanged();
	XMPShape*				RequestFrameShape(XMPShape* shape);
	
	virtual void 			UpdateUsedAndActiveShapes();
	virtual void 			UpdateUsedShape();

	// ----- Persistence -----
	virtual XMPStorageUnit*	GetXMPStorageUnit() const;
	
	// ----- Window -----
	virtual XMPWindow*		CreateChildWindow(XMPISOStr windowTitle);
	
	// ----- Selection State -----
	void 					ChangeSelectionState(SelectionState state);
	virtual void			SelectionStateChanged(SelectionState oldState, SelectionState newState);
	SelectionState			GetSelectionState() const;
	
	// ----- Getter/Setter -----
	FW_CPart* 					GetPart() const;
	FW_Boolean				GetFrontClicks() const;
	void					SetFrontClicks(FW_Boolean getFrontClicks);
	FW_CFacet* 				GetActiveFacet() const;
	virtual void			SetActiveFacet(FW_CFacet* activeFacet);
	FW_Boolean				NeedsActivating() const;
	void					SetNeedsActivating(FW_Boolean state);
	FW_Boolean				CanBeActiveFrame() const;
	void					SetCanBeActiveFrame(FW_Boolean beActive);
	
	// ----- Part Info -----
	void					WritePartInfo(XMPStorageUnitView* storageUnitView);
	void					ReadPartInfo(XMPStorageUnitView* storageUnitView);
	
protected:
	// ----- Relinquish/Request Focus Set -----
	void					RelinquishFocusSet();
	FW_Boolean				RequestFocusSet();
	
//----------------------------------------------------------------------------------------
//	 Data Members
//
private:
	XMPFocusSet* 			fFocusSet;
	FW_Boolean				fGetFrontClicks;
	FW_Boolean				fNeedsActivating;	// should this be activated when its window is activated?
	FW_CFacet*				fActiveFacet;
	SelectionState			fSelectionState;
	FW_Boolean				fCanBeActiveFrame;
};

//========================================================================================
//	 FW_CFrame Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	 FW_CFrame::RequestFrameShape()
//----------------------------------------------------------------------------------------
inline XMPShape* FW_CFrame::RequestFrameShape(XMPShape* shape)
{
	return GetXMPFrame()->RequestFrameShape(shape);
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::GetFrontClicks()
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::GetFrontClicks() const
{
	return fGetFrontClicks;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::SetFrontClicks()
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetFrontClicks(FW_Boolean getFrontClicks)
{
	fGetFrontClicks = getFrontClicks;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::GetPart()
//----------------------------------------------------------------------------------------
inline FW_CPart* FW_CFrame::GetPart() const
{
	return (FW_CPart*)GetXMPFrame()->GetPart();
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::GetFocusSet()
//----------------------------------------------------------------------------------------
inline XMPFocusSet* FW_CFrame::GetFocusSet() const
{
	return fFocusSet;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::GetXMPStorageUnit
//----------------------------------------------------------------------------------------

inline XMPStorageUnit* FW_CFrame::GetXMPStorageUnit() const
{
	return GetXMPFrame()->GetStorageUnit();
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::GetActiveFacet
//----------------------------------------------------------------------------------------
inline FW_CFacet* FW_CFrame::GetActiveFacet() const
{
	return fActiveFacet;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::CanBeActiveFrame
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::CanBeActiveFrame() const
{
	return fCanBeActiveFrame;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::SetCanBeActiveFrame
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetCanBeActiveFrame(FW_Boolean beActive)
{
	fCanBeActiveFrame = beActive;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::GetSelectionState
//----------------------------------------------------------------------------------------
inline SelectionState FW_CFrame::GetSelectionState() const
{
	return fSelectionState;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::NeedsActivating
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::NeedsActivating() const
{
	return fNeedsActivating;
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::SetNeedsActivating
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetNeedsActivating(FW_Boolean state)
{
	fNeedsActivating = state;
}




//========================================================================================
//	 class FW_CFrameFacetIterator
//========================================================================================

#define kCFrameFacetIteratorID "appl:cframefacetiterator$class,1.0.0"

class FW_CFrameFacetIterator
{

public:
	FW_CFrameFacetIterator(FW_CFrame* frame);

	virtual ~FW_CFrameFacetIterator();

	FW_CFacet* CurrentItem();
	FW_Boolean Next();
	FW_Boolean IsDone();

private:
	XMPFrameFacetIterator 	fIte;
	FW_CFacet*	fCurrentFacet;
};

//----------------------------------------------------------------------------------------
//	 FW_CFrameFacetIterator::IsDone
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrameFacetIterator::IsDone()
{
	return fCurrentFacet == NULL;
}

#endif

