//========================================================================================
//
//	File:				FWPart.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPART_H
#define FWPART_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef BCSTOREU_H
#include <BCStoreU.h>
#endif

#ifndef BCCOLLD_H
#include <BCCollD.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _PART_
#include <Part.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _DOCUMENT_
#include <Document.h>
#endif

#ifndef _STORAGE_
#include <Storage.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPFrame;
class XMPMenuBar;
class XMPWindow;
class FW_CFrame;
class FW_CFacet;
class FW_CWindow;
class FW_CFrameFacetIterator;
class FW_CSelection;
class FW_CPublishLink;
class FW_CSubscribLink;
class XMPSession;
struct FW_CPoint;
struct FW_CRect;

//==============================================================================
//	 Defines
//==============================================================================

#define kNoPresentation 0
#define kContentAnnotationProp		"Apple:Framework:Proposition:ContentAnnotation"
#define kContentAnnotationValue		"Apple:Framework:Value:ContentAnnotation"

//==============================================================================
//	 class FW_CPart
//==============================================================================

class FW_CPart : public XMPPart, public FW_MEventHandler
{
	friend class XMPDraft;

//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
protected:

	FW_CPart();
	virtual ~ CDECL FW_CPart();

	virtual void Initialize();
	virtual void InitPart(XMPStorageUnit* storageUnit);
	virtual void InitPartFromStorage(XMPStorageUnit* storageUnit);

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	enum
	{
		kDisplayFramesChunkSize = 5,			// Chunk size for display frames
		kWindowsChunkSize = 3,					// Chunk size for window list
		kPublishLinksChunkSize = 5,				// Chunk size for publish links
		kSubscribeLinksChunkSize = 5			// Chunk size for subscribe links
	};

	// ----- ref counting -----
	virtual void 		Release();
	virtual void 		ReleaseAll();
	
	// ----- DragAndDrop -----
	virtual void 		FulfillPromise(XMPStorageUnitView *promiseSUView);
	virtual void 		DropCompleted(XMPPart* destPart, XMPDropResult dropResult);
	virtual void		DragEnter(XMPDragItemIterator* dragInfo,  
										XMPFacet* facet,  
										XMPPoint where);
	virtual void 		DragWithin(XMPDragItemIterator* dragInfo,
										XMPFacet* xmpFacet,
										XMPPoint where);
	virtual void 		DragLeave(XMPFacet* xmpFacet,
						   				XMPPoint where);
	virtual XMPDropResult Drop(XMPDragItemIterator* dropInfo,
								  XMPFacet* xmpFacet,
								  XMPPoint where);

	// ----- Embedding -----
	virtual void 		ContainingPartPropertiesChanged(XMPFrame* displayFrame,
													XMPStorageUnit* propertyUnit);
	virtual XMPStorageUnit* GetContainingPartProperties(XMPFrame* displayFrame);
	virtual void 		RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape);
	virtual void 		EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec);
	virtual XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(XMPFrame* displayFrame);
		
	// ----- Frame -----
	virtual void 		AddDisplayFrame(XMPFrame* xmpFrame);
	virtual void 		AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame);
	virtual void 		RemoveDisplayFrame(XMPFrame* xmpFrame);
	virtual void 		CloseDisplayFrame(XMPFrame* xmpFrame);	
	virtual void 		FrameShapeChanged(XMPFrame* xmpFrame);
	virtual void 		ViewTypeChanged(XMPFrame* frame);
	virtual void 		PresentationChanged(XMPFrame* xmpFrame);
	virtual void 		SequenceChanged(XMPFrame* xmpFrame);
	virtual void 		WritePartInfo(void* partInfo, XMPStorageUnitView* storageUnitView);
	virtual void* 		ReadPartInfo(XMPFrame* xmpFrame, XMPStorageUnitView* storageUnitView);
	virtual XMPID 		Open(XMPFrame* xmpFrame);


	virtual XMPFrame* RequestEmbeddedFrame(XMPFrame* containingFrame,
											  XMPFrame* baseFrame,
											  XMPShape* frameShape,
											  XMPPart* embedPart,
											  XMPTypeToken viewType,
											  XMPTypeToken presentation,
											  FW_Boolean isOverlaid);
	virtual void 		RemoveEmbeddedFrame(XMPFrame* xmpFrame);
	virtual XMPShape* 	RequestFrameShape(XMPFrame* xmpFrame, XMPShape* frameShape);
	virtual void 		UsedShapeChanged(XMPFrame* xmpEmbeddedFrame);
	virtual XMPShape* 	AdjustBorderShape(XMPFrame* xmpFrame, XMPShape* shape);
	
	// ----- Facet -----
	virtual void 		FacetAdded(XMPFacet* xmpFacet);
	virtual void 		FacetRemoved(XMPFacet* xmpFacet);
	virtual void 		CanvasChanged(XMPFacet* xmpFacet);
	virtual void 		GeometryChanged(XMPFacet* xmpFacet);

	// ----- Imaging -----
	virtual void 		Draw(XMPFacet* xmpFacet, XMPShape* invalidShape);
	virtual void 		CanvasUpdated(XMPCanvas* canvas);
	virtual void 		HighlightChanged(XMPFacet* xmpFacet);
	virtual unsigned long 	GetPrintResolution(XMPFrame* xmpFrame);

	// ----- Linking -----
	virtual XMPLink* 	CreateLink(void* data, unsigned long size);
	virtual void 		LinkUpdated(XMPLink* updatedLink, XMPChangeID id);
	virtual void 		RevealLink(XMPLinkSource* xmpLinkSource);
	virtual void		EmbeddedFrameChanged(XMPFrame* xmpFrame, XMPChangeID change);
	virtual void		LinkStatusChanged(XMPFrame* xmpFrame);

	// ----- Part Activation -----
	virtual FW_Boolean 	BeginRelinquishFocus(XMPTypeToken focus,
											   XMPFrame* ownerFrame,
											   XMPFrame* proposedFrame);
	virtual void 		CommitRelinquishFocus(XMPTypeToken focus, 
										  		XMPFrame* ownerFrame,
										  		XMPFrame* proposedFrame);
	virtual void 		AbortRelinquishFocus(XMPTypeToken focus,
										 		XMPFrame* ownerFrame,
										 		XMPFrame* proposedFrame);
	virtual void 		FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame);
	virtual void 		FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame);

	// ----- Storage -----
	virtual void 		Externalize();
	virtual void 		CloneInto(XMPDraftKey key, 
									XMPStorageUnit* storageUnit, 
									XMPStorageUnit* initiatingFrameSU);

	// ----- Binding -----
	virtual void 		ExternalizeKinds(XMPTypeList* kindList);
	virtual void 		ChangeKind(XMPType kind);

	// ----- UI Events -----
	virtual FW_Boolean HandleEvent(XMPEventData event,  XMPFrame* xmpFrame,  XMPFacet* xmpFacet);
	virtual FW_Boolean HandleEventInEmbedded(XMPEventData event,
												XMPFrame* frame,
												XMPFacet* facet,
												XMPFrame* embeddedFrame,
												XMPFacet* embeddedFacet);
	virtual void 		MouseEnter(XMPFacet* xmpFacet, XMPPoint where);
	virtual void 		MouseWithin(XMPFacet* xmpFacet, XMPPoint where);
	virtual void 		MouseLeave(XMPFacet* xmpFacet);
	virtual void 		AdjustMenus(XMPFrame* xmpFrame);

	// ----- Undo -----
	virtual void 		UndoAction(XMPActionData actionState);
	virtual void 		RedoAction(XMPActionData actionState);
	virtual void 		DisposeActionState(XMPActionData actionState,
									   XMPDoneState doneState);
	virtual void 		WriteActionState(void* actionState,
									 XMPStorageUnitView* storageUnitView);
	virtual void* 		ReadActionState(XMPStorageUnitView* storageUnitView);

	// ----- Memory Management -----
	virtual XMPSize 	Purge(XMPSize size);


//------------------------------------------------------------------------------
//	 New API
//
public:
	// ----- Storage Properties -----
	virtual void 			AddProperties(XMPStorageUnit* storageUnit);
	void					CheckAndAddProperties(XMPStorageUnit* storageUnit);
	
	// ----- Events -----
	virtual FW_Boolean 	HandleActivate(FW_CFrame* frame, FW_CFacet* facet, FW_Boolean state);

	virtual FW_Boolean 	HandleMenuEvent(FW_CFrame* frame, long message);
	virtual FW_Boolean 	DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);
	
	virtual FW_Boolean HandleIdle(FW_CFrame* frame);
	virtual FW_Boolean DoIdle();

	// ----- Frame -----
	virtual FW_CFrame* 	NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation) = 0;

	// ----- Menus -----
	virtual void 			InstallMenus(XMPMenuBar* menuBar);
	virtual void 			DoAdjustMenus(XMPMenuBar* menuBar);

	// ----- Validation/Invalidation -----
	void					InvalidateAllFrames(XMPTypeToken whichFrames, FW_CFacet* skipFacet, XMPShape* invalidShape);
	void					ValidateAllFrames(XMPTypeToken whichFrames, FW_CFacet* skipFacet, XMPShape* validShape);
	
	// ----- Presentation -----
	virtual XMPTypeToken	GetDefaultPresentation() const;
	
	// ----- Getters/Setters -----
	FW_CFrame* 				GetActiveFrame() const;
	virtual void			SetActiveFrame(FW_CFrame* activeFrame);
	FW_CFacet* 				GetActiveFacet() const;
	XMPMenuBar* 			GetMenuBar() const;
	XMPMenuBar* 			GetPartMenuBar() const;
	BC_TDynamicCollection<FW_CFrame*, BC_CUnmanaged>*	GetDisplayFrameList() const;
	unsigned long				CountDisplayFrame() const;
	XMPSession* 			GetSession() const;
	
	XMPTypeToken			GetSelectionFocusToken() const;
	XMPTypeToken			GetMenuFocusToken() const;
	XMPTypeToken			GetKeyFocusToken() const;

	virtual XMPValueType	GetContentPropertyValueType() const = 0;
	
	// ----- Window -----
	void					AddWindow(FW_CWindow *window);
	void					RemoveWindow(FW_CWindow *window);
	FW_CWindow*				XMPtoFWWindow(XMPWindow *xmpWindow) const;
	void					HideShowFloatingWindows(FW_Boolean state);
	
	virtual XMPWindow*		CreateRootWindow(XMPISOStr windowTitle);
	
	XMPWindow*				CreateXMPWindow(FW_CFrame* itsFrame,
											XMPTypeToken viewType,
											XMPTypeToken presentation,
											XMPISOStr windowTitle,
											const FW_CRect& windowRect,
											FW_Boolean hasCloseBox,
											FW_Boolean isResizable,
											FW_Boolean isFloating,
											FW_Boolean shouldSave,
											short procID);
											
	// ----- Saving -----
	void					Changed();
	
	// ----- Selection -----
	FW_CSelection*			GetSelection() const;
	virtual FW_CSelection*	NewSelection();
	
	// ----- Utilities -----
	FW_Boolean				HasPropertyOnClipboard(XMPPropertyName propertyName, XMPValueType type);

	// ----- Linking -----
	void					SetPendingPublish(FW_CPublishLink* publishLink);
	void					DeletePendingPublish();
	
	void					AddToPublishLinkList(FW_CPublishLink* publishLink);
	void					RemoveFromPublishLinkList(FW_CPublishLink* publishLink);

	void					AddToSubscribLinkList(FW_CSubscribLink* subscribLink);
	void					RemoveFromSubscribLinkList(FW_CSubscribLink* subscribLink);
	
	// ----- Drag&Drop -----
	virtual void			Dragging(FW_CFacet *facet, FW_Boolean dragging);
	
	// ----- Externalize Content -----
	virtual void			ExternalizeContent(XMPStorageUnit* storageUnit);
	virtual void			InternalizeContent(XMPStorageUnit* storageUnit);

	// ----- Cloning -----
	virtual XMPPart*		ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind);
	
	// ----- Can I embed? -----
	// This is temporary. It needs to be removed
	virtual FW_Boolean		IsEmbeddingPart() const;
	
protected:
	// ----- Framework data specific Ex/Internalize -----
	void					ExternalizeDisplayFrames(XMPStorageUnit* storageUnit);
	void					InternalizeDisplayFrames(XMPStorageUnit* storageUnit);
	
	// ----- Linking -----
	FW_CSubscribLink*			XMPtoSubscribLink(XMPLink* xmpLink);

//------------------------------------------------------------------------------
//	 Data members
//
private:
	XMPMenuBar*			fPartMenuBar;
	XMPMenuBar*			fMenuBar;			// This is the current menubar 
	FW_CFrame* 			fActiveFrame;
	BC_TDynamicCollection<FW_CFrame*, BC_CUnmanaged>* 	fDisplayFrames;	
	BC_TDynamicCollection<FW_CWindow*, BC_CUnmanaged>* 	fWindowList;
	
	XMPTypeToken		fSelectionFocusToken;
	XMPTypeToken		fMenuFocusToken;
	XMPTypeToken		fKeyFocusToken;
	
	FW_CSelection*		fSelection;

	FW_CPublishLink*		fPendingPublishLink;
	BC_TDynamicCollection<FW_CPublishLink*, BC_CUnmanaged>*	fPublishLinkList;
	BC_TDynamicCollection<FW_CSubscribLink*, BC_CUnmanaged>*	fSubscribLinkList;
	
	FW_Boolean			fUserPropertiesAdded;
};

//========================================================================================
//	 class FW_CPartFrameIterator
//========================================================================================

class FW_CPartFrameIterator
{
public:
	FW_CPartFrameIterator(FW_CPart* part, XMPTypeToken presentation = 0);
	virtual ~FW_CPartFrameIterator();

	FW_CFrame*			CurrentItem();
	FW_Boolean			Next();
	FW_Boolean			IsDone();

private:
	void			FindNext();
	
private:
	BC_TCollectionActiveIterator<FW_CFrame*>	fIterator;
	XMPTypeToken				fPresentation;
	FW_CFrame*						fCurrentFrame;
};

//========================================================================================
//	 class FW_CPartFacetIterator
//========================================================================================

class FW_CPartFacetIterator
{
public:
	FW_CPartFacetIterator(FW_CPart* part, XMPTypeToken presentation = 0);
	virtual ~FW_CPartFacetIterator();

	FW_CFacet*		CurrentItem();
	FW_Boolean		Next();
	FW_Boolean		IsDone();

private:
	void			NewFacetIterator();
	
private:
	FW_CPartFrameIterator	fFrameIterator;
	FW_CFrameFacetIterator*		fFacetIterator;
	FW_CFrame*						fCurrentFrame;
	FW_CFacet*						fCurrentFacet;
};

//========================================================================================
//	 Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetActiveFrame
//----------------------------------------------------------------------------------------
inline FW_CFrame* FW_CPart::GetActiveFrame() const
{
	return fActiveFrame;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetMenuBar
//----------------------------------------------------------------------------------------
inline XMPMenuBar* FW_CPart::GetMenuBar() const
{
	return fMenuBar;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetMenuBar
//----------------------------------------------------------------------------------------
inline XMPMenuBar* FW_CPart::GetPartMenuBar() const
{
	return fPartMenuBar;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetDisplayFrameList
//----------------------------------------------------------------------------------------
inline BC_TDynamicCollection<FW_CFrame*, BC_CUnmanaged>* FW_CPart::GetDisplayFrameList() const
{
	return fDisplayFrames;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetSession
//----------------------------------------------------------------------------------------
inline XMPSession* FW_CPart::GetSession() const
{
	XMPPart* self = (XMPPart *) this;
	return self->GetStorageUnit()->GetSession();
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetSelectionFocusToken
//----------------------------------------------------------------------------------------
inline XMPTypeToken FW_CPart::GetSelectionFocusToken() const
{
	return fSelectionFocusToken;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetMenuFocusToken
//----------------------------------------------------------------------------------------
inline XMPTypeToken FW_CPart::GetMenuFocusToken() const
{
	return fMenuFocusToken;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetKeyFocusToken 
//----------------------------------------------------------------------------------------
inline XMPTypeToken FW_CPart::GetKeyFocusToken() const
{
	return fKeyFocusToken;
}

//------------------------------------------------------------------------------
//	 FW_CPart::GetSelection
//------------------------------------------------------------------------------
inline FW_CSelection* FW_CPart::GetSelection() const
{
	return fSelection;
}

//------------------------------------------------------------------------------
//	 FW_CPart::SetPendingPublish
//------------------------------------------------------------------------------
inline void FW_CPart::SetPendingPublish(FW_CPublishLink* publishLink)
{
	fPendingPublishLink = publishLink;
}

//------------------------------------------------------------------------------
//	 FW_CPart::CountDisplayFrame
//------------------------------------------------------------------------------
inline unsigned long FW_CPart::CountDisplayFrame() const
{
	return fDisplayFrames->Length();
}

#endif



