//========================================================================================
//
//	File:				FWParting.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPARTING_H
#define FWPARTING_H

#ifndef FWPART_H
#include "FWPart.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef BCCOLLD_H
#include <BCCollD.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPFrame;
class XMPMenuBar;
class XMPFocusSet;
class FW_CEmbeddingFrame;
class FW_CProxyFrame;
class FW_CProxyRun;
class BC_CUnmanaged;

//==============================================================================
//	 class FW_CEmbeddingPart
//==============================================================================

class FW_CEmbeddingPart : public FW_CPart
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
protected:
	FW_CEmbeddingPart();
	virtual ~ CDECL FW_CEmbeddingPart();

	virtual void Initialize();
	
	virtual void		Release();

//------------------------------------------------------------------------------
//	 Inherited API
//
public:
	// ----- Embedding ----- 
	virtual XMPEmbeddedFramesIterator* CreateEmbeddedFramesIterator(XMPFrame* displayFrame);

	// ----- Frame ----- 	
	virtual XMPFrame* RequestEmbeddedFrame(XMPFrame* containingFrame,
											  XMPFrame* baseFrame,
											  XMPShape* frameShape,
											  XMPPart* embedPart,
											  XMPTypeToken viewType,
											  XMPTypeToken presentation,
											  FW_Boolean isOverlaid);
	virtual void 		RemoveEmbeddedFrame(XMPFrame* frame);

	virtual void		UsedShapeChanged(XMPFrame* xmpEmbeddedFrame);
	virtual void 		AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame);
	
	// ----- UI Events ----- 
	virtual void		DoAdjustMenus(XMPMenuBar* menuBar);
	virtual FW_Boolean 	HandleEvent(XMPEventData event, XMPFrame* xmpFrame, XMPFacet* xmpFacet);
	virtual FW_Boolean	DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID);

//------------------------------------------------------------------------------
//	 New API
//
public:
	enum
	{
		kChunkSize = 5				// Chunk size for proxy runs
	};

	// ----- Embedding -----
	void					EmbedPart(XMPPart *embeddedXMPPart,
										FW_CProxyRun* proxyRun, 
										XMPTypeToken inWhichPresentation, 		
										XMPTypeToken viewType,
										XMPTypeToken presentation,
										XMPTransform* externalXForm, 
										XMPShape* frameShape,
										FW_Boolean invalidate);
	void					EmbedFrame(XMPFrame* embeddedXMPFrame,
										FW_CProxyRun* proxyRun, 
										XMPTypeToken inWhichPresentation, 		
										XMPTransform* externalXForm, 
										FW_Boolean invalidate);
											
	void 					ClipEmbeddedFrames(XMPTypeToken whichFrames);
	void					RemoveEmbeddedFrames(FW_CProxyRun *proxyRun);
	
	// ----- Cloning -----
	XMPFrame*				CloneFrameFrom(XMPStorageUnit* storageUnit, XMPStorageUnitRef frameSURef, XMPCloneKind kind);
	virtual XMPPart*		ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind);
	
	
	// ----- Conversion -----
	FW_CProxyRun*			XMPFrametoProxyRun(XMPFrame* embeddedXMPFrame) const;
	FW_CProxyFrame*			XMPtoProxyFrame(XMPFrame* embeddedXMPFrame) const;
	
	FW_Boolean				HasSelectedProxyRun() const;
	
	// ----- ProxyRun List -----
	virtual FW_CProxyRun* 	NewProxyRun() = 0;	
	void					AddProxyRun(FW_CProxyRun *proxyRun);
	void					RemoveProxyRun(FW_CProxyRun *proxyRun);

	// -----
	void					SetEmbeddedFramesContainingFrame(FW_CEmbeddingFrame* frame);
	
	// ----- Drag&Drop -----
	virtual void			Dragging(FW_CFacet *facet, FW_Boolean state);

	// ----- Can I embed? -----
	// This is temporary. It needs to be removed
	virtual FW_Boolean		IsEmbeddingPart() const;

	// ----- Getters -----
	BC_TDynamicCollection<FW_CProxyRun*, BC_CUnmanaged>* GetProxyRunList() const;
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	BC_TDynamicCollection<FW_CProxyRun*, BC_CUnmanaged>*	fProxyRunList;
	unsigned long				fProxyRunCount;
	FW_Boolean				fSupportRequestEmbeddedFrame;	// Default true. Set it to false if you don't want
															// an embedded part to have multiple display frames
															// embedded in one of your frame.
};

//==============================================================================
//	 class FW_CEmbeddedXMPFramesIterator
//==============================================================================

class FW_CEmbeddedXMPFramesIterator : public XMPEmbeddedFramesIterator
{
public:
	FW_CEmbeddedXMPFramesIterator(FW_CEmbeddingPart* part, FW_CEmbeddingFrame* displayFrame = NULL);
	virtual ~FW_CEmbeddedXMPFramesIterator();

	virtual XMPFrame*		First();
	virtual XMPFrame*		Next();
	virtual FW_Boolean		IsNotComplete();

	FW_CProxyFrame*			GetCurrentProxyFrame() const
									{return fCurrentProxyFrame;}

private:
	void 					TestEmbedded();

private:
	FW_CEmbeddingFrame*				fDisplayFrame;
	FW_CEmbeddingPart*				fPart;
	FW_CProxyRun* 					fCurrentProxyRun;
	FW_CProxyFrame*					fCurrentProxyFrame;
	BC_TCollectionActiveIterator<FW_CProxyRun*>* fProxyRunIter;
	BC_TCollectionActiveIterator<FW_CProxyFrame*>*	fProxyFrameIter;
};


//==============================================================================
//	 FW_CEmbeddingPart Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::GetProxyRunList
//------------------------------------------------------------------------------
inline BC_TDynamicCollection<FW_CProxyRun*, BC_CUnmanaged>* FW_CEmbeddingPart::GetProxyRunList() const
{
	return fProxyRunList;
}

#endif
