//========================================================================================
//
//	File:				FWProxy.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPROXY_H
#define FWPROXY_H

#ifndef FWSTDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWBASEFR_H
#include "FWBaseFr.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef BCCOLLD_H
#include <BCCollD.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _PLFMTYPE_
#include <PlfmType.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

#ifndef _PART_
#include <Part.h>
#endif

#ifndef _FACET_
#include <Facet.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <Quickdraw.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPFrame;
class XMPShape;
class FW_CFrame;
class FW_CFacet;
class FW_CEmbeddingFrame;
class FW_CProxyFrame;
class FW_CEmbeddingPart;
class FW_CBaseProxyBorder;
struct FW_CPoint;
class BC_CUnmanaged;

//==============================================================================
//	 class FW_CProxyRun
//==============================================================================

#define kFW_CProxyRunID "appl:framework:cproxyrun$class,1.0.0"

class FW_CProxyRun
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	FW_CProxyRun();
	void InitProxyRun(FW_CEmbeddingPart* embeddingPart);
	virtual ~FW_CProxyRun();
	
//------------------------------------------------------------------------------
//	 New API
//
public:
	enum
	{
		kChunkSize = 3					// Chunk size for proxy frames
	};

	virtual FW_CProxyFrame* 	NewProxyFrame(XMPFrame *xmpFrame);
	virtual void			AddProxyFrame(FW_CProxyFrame* proxyFrame);
	virtual void			RemoveProxyFrame(FW_CProxyFrame* proxyFrame);
	unsigned long				CountProxyFrames() const
									{return fCount;}
	
	// ----- Query -----
	XMPFacet*				WhichEmbeddedFacet(FW_CFacet* embeddingFacet, const FW_CPoint& mouse);	
	FW_CProxyFrame*			WhichProxyFrame(FW_CEmbeddingFrame* embeddingFrame);
	
	// ----- Notification -----
	virtual void 			UsedShapeChanged(XMPFrame* xmpFrame);
	
	// ----- Persistence -----
	void					Externalize(XMPStorageUnit* storageUnit);
	void					Internalize(XMPStorageUnit* storageUnit);
	void					CloneTo(XMPStorageUnit* storageUnit, 
									  FW_CFrame* commandFrame, 
									  XMPCloneKind cloneKind);
	void					CloneInto(XMPStorageUnit* storageUnit, 
									  FW_CFrame* commandFrame, 
									  XMPCloneKind cloneKind);
	XMPFrame*				CloneFrom(XMPStorageUnit* storageUnit, 
									  XMPCloneKind cloneKind);
									  
	// ----- Selection state -----
	FW_Boolean				IsSelected() const
								{return fSelected;}
	virtual void			SetSelectState(FW_Boolean state);

	// ----- Selection Border -----
	virtual FW_CBaseProxyBorder*	NewProxyBorder();
	
	// ----- Facet External Transform -----
	virtual void 				GetFacetExternalTransform(XMPFrame* xmpFrame, 
														  XMPTransform* facetExternalTranform) = 0;
	
	// ----- Getters/Setters -----
	XMPPart*					GetXMPPart();
	BC_TDynamicCollection<FW_CProxyFrame*, BC_CUnmanaged>* GetProxyFrameList() const;
	FW_CEmbeddingPart*				GetEmbeddingPart() const;
	FW_CBaseProxyBorder*			GetProxyBorder() const;
	
										
//------------------------------------------------------------------------------
//	 Data Members
//------------------------------------------------------------------------------
private:
	FW_CProxyFrame*				fCachedProxyFrame;
	BC_TDynamicCollection<FW_CProxyFrame*, BC_CUnmanaged>*	fProxyFrameList;
	FW_CEmbeddingPart*			fEmbeddingPart;
	FW_CBaseProxyBorder*		fProxyBorder;
	FW_Boolean				fSelected;
	unsigned long				fCount;
};

//==============================================================================
//	 FW_CProxyRun Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CProxyRun::GetProxyFrameList
//------------------------------------------------------------------------------
inline BC_TDynamicCollection<FW_CProxyFrame*, BC_CUnmanaged>* FW_CProxyRun::GetProxyFrameList() const
{
	return fProxyFrameList;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::GetEmbeddingPart
//------------------------------------------------------------------------------
inline FW_CEmbeddingPart* FW_CProxyRun::GetEmbeddingPart() const
{
	return fEmbeddingPart;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::GetEmbeddingPart
//------------------------------------------------------------------------------
inline FW_CBaseProxyBorder* FW_CProxyRun::GetProxyBorder() const
{
	return fProxyBorder;
}

//==============================================================================
//	 class FW_CProxyFrame
//==============================================================================
// FW_CProxyFrame is used as a wrapper for an embedded frame

#define kCProxyFrameID "appl:cproxyframe$class,1.0.0"

class FW_CProxyFrame : public FW_CBaseFrame
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	FW_CProxyFrame();
	void InitProxyFrame(FW_CProxyRun* proxyRun,
					 	XMPFrame *xmpFrame);
	virtual ~ CDECL FW_CProxyFrame();
	
//------------------------------------------------------------------------------
//	 New API
//
	// ----- Presistence -----
	void					Externalize(XMPStorageUnit* storageUnit);
	
	// ----- Getters/Setters
	FW_CEmbeddingFrame* 		GetContainingFrame() const;
	
	FW_CBaseProxyBorder*		GetProxyBorder() const;
	FW_CProxyRun*				GetProxyRun() const
								{return fProxyRun;}

//------------------------------------------------------------------------------
//	 Data Members
//
private:
	FW_CProxyRun*				fProxyRun;
};


//==============================================================================
//	 CEmbeddedXMPFacetIterator
//==============================================================================
// Iterate trought all the facet of proxyRun embedded in facet
// If facet is Null returns all the embedded facets

class FW_CEmbeddedXMPFacetsIterator
{
public:
	FW_CEmbeddedXMPFacetsIterator(FW_CProxyRun* proxyRun, FW_CFacet* facet, XMPSiblingOrder siblingOrder);
	virtual ~FW_CEmbeddedXMPFacetsIterator();

	XMPFacet*		First();
	XMPFacet*		Next();
	FW_Boolean		IsNotComplete();

private:
	void			FindNext();
	
private:
	XMPFacetIterator	fXMPFacetIterator;
	FW_CProxyRun*			fProxyRun;
	XMPFacet*			fXMPFacet;
};

#endif

