//========================================================================================
//
//	File:				FWSelect.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWSELECT_H
#define FWSELECT_H

#ifndef FWSTDDEF_H
#include <FWStdDef.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class XMPStorageUnit;
struct FW_CPoint;
class FW_CPart;
class FW_CFacet;
class FW_CFrame;
class XMPLink;
class XMPLinkSource;
class XMPShape;
class FW_CProxyRun;
class FW_CGraphicContext;

//==============================================================================
//	 constants
//==============================================================================

const XMPPropertyName kXMPPropFrameInfo = "Framework:Property:FrameInfo";
const XMPPropertyName kXMPPropMouseDownOffset = "Framework:Property:MouseDownOffset";

const XMPValueType kQDPoint = "Framework:ValueType:FW_SPlatformPoint";

//==============================================================================
//	 class FW_CPublishLink
//==============================================================================

class FW_CPublishLink
{
public:
	FW_CPublishLink(XMPChangeID changeID);
	virtual ~ CDECL FW_CPublishLink();
	
	void			SetXMPLinkSource(XMPLinkSource* xmpLinkSource);
	XMPLinkSource*	GetXMPLinkSource() const
						{return fXMPLinkSource;}
	FW_Boolean		IsPending() const
						{return fXMPLinkSource == NULL;}
	XMPChangeID		GetChangeID() const
						{return fChangeID;}
	
	void			ContentChanged(XMPChangeID changeID);
	
	virtual void	Publish() = 0;	
	virtual void	ExternalizeLinkContent(XMPStorageUnit* linkSU) = 0;
	virtual XMPValueType GetPublishFormat() = 0;
 
private:
	XMPLinkSource	*fXMPLinkSource;
	XMPChangeID		fChangeID;
};

//==============================================================================
//	 class FW_CSubscribLink
//==============================================================================

class FW_CSubscribLink
{
public:
	FW_CSubscribLink(XMPLink *xmpLink);
	virtual ~FW_CSubscribLink();
	
	XMPLink* 		GetXMPLink() const
						{return fXMPLink;}
						
	virtual void	LinkUpdated(XMPChangeID id);
						
	virtual void	Subscrib() = 0;

private:
	XMPLink *fXMPLink;
};

//==============================================================================
//	 class FW_CSelection
//==============================================================================

class FW_CSelection
{
//------------------------------------------------------------------------------
//	 Initialization/Destruction
//
public:
	FW_CSelection();
	void		InitSelection(FW_CPart* part, FW_Boolean allowPublish, FW_Boolean allowSubscrib);
	virtual 	~FW_CSelection();

//------------------------------------------------------------------------------
//	 New API
//
public:
	virtual FW_Boolean		IsEmpty() const = 0;	
	virtual void			CloseSelection() = 0;
	virtual void			SelectAll() = 0;
	virtual XMPShape*		GetSelectionShape();
	virtual FW_CProxyRun*	IsSelectionOnlyOneProxyRun() const;
	
	// ----- Cut/Copy/Paste/Clear -----
	virtual FW_Boolean		DoCut(FW_CFrame* frame);
	virtual void			DoCopy(FW_CFrame* frame);
	virtual FW_Boolean		DoPaste();
	virtual FW_Boolean		DoPasteWithLink();
	virtual FW_Boolean		DoClear() = 0;
	
	// ----- Drag & Drop -----
	virtual FW_Boolean		Drag(FW_CFacet* facet, XMPEventData event);
	virtual XMPRgnHandle 	CalcDragRgn(FW_CFacet* facet);
	virtual FW_Boolean 	DoDroppedInSameFrame(XMPStorageUnit* dropSU, FW_CFacet* facet,
											 const FW_CPoint& originPoint, const FW_CPoint& dropPoint);
 	virtual FW_Boolean		DoDrop(XMPStorageUnit* dropSU, FW_CFacet* facet,
							   	const FW_CPoint& originPoint, const FW_CPoint& dropPoint);
 
 	// ----- Linking -----
	virtual FW_CPublishLink* 	NewPublishLink(XMPChangeID changeID);
	virtual FW_CSubscribLink*	NewSubscribLink(XMPLink *xmpLink);
	virtual FW_Boolean			IsSelectionPublishable();
	
	// ----- Externalize/Internalize Selection -----
	virtual void				ExternalizeSelection(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind) = 0;
	virtual FW_Boolean			InternalizeSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind) = 0;
	virtual FW_Boolean			EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape);

	// ----- Getters/Setters -----
	FW_CPart*					GetPart() const;
	
	// ----- Insert new Part -----
	virtual FW_Boolean 		InsertNewPart();

protected:
	void						Copy(FW_Boolean allowLinking, FW_CFrame* frame, XMPCloneKind cloneKind);
	FW_Boolean					Paste(FW_Boolean withLink, XMPCloneKind cloneKind);
	XMPPart*					ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind);
	void						WriteSelection(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind);
	FW_Boolean					ReadSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind);
	
//------------------------------------------------------------------------------
//	 Data Members
//
private:
	FW_CPart*				fPart;
	FW_Boolean				fAllowPublish;
	FW_Boolean				fAllowSubscrib;
};

//==============================================================================
//	 FW_CSelection Inlines
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CSelection::GetPart
//------------------------------------------------------------------------------
inline FW_CPart* FW_CSelection::GetPart() const
{
	return fPart;
}

#endif
