//========================================================================================
//
//	File:				FWUtil.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWUTIL_H
#define FWUTIL_H

// ----- Graphic Includes -----

#ifndef FWGC_H
#include "FWGC.h"
#endif

#ifndef FWGRUTIL_H
#include "FWGrUtil.h"
#endif

// ----- Foundation Includes -----

#ifndef BCCOLL_H
#include <BCColl.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <QuickDraw.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__EVENTS__)
#include <Events.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__TEXTEDIT__)
#include <TextEdit.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__LIBRARYMANAGERUTILITIES__)
#include <LibraryManagerUtilities.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__LIBRARYMANAGERCLASSES__)
#include <LibraryManagerClasses.h>
#endif

//========================================================================================
//	 Forward Declarations
//========================================================================================

class FW_CFrame;
class FW_CFacet;
class XMPShape;
class XMPTransform;
class XMPWindow;
class XMPStorageUnit;
struct FW_CPoint;
struct FW_CRect;

//========================================================================================
//	 Global Utilities
//========================================================================================

long 	Abs(long value);
void 		CenterRectOnScreen(FW_SPlatformRect& aRect, Boolean horizontally, Boolean vertically, Boolean forDialog);

// ----- Key pressed
FW_Boolean IsShiftKeyPressed();
FW_Boolean IsOptionKeyPressed();
FW_Boolean IsCommandKeyPressed();
FW_Boolean IsControlKeyPressed();
FW_Boolean IsKeyPressed(unsigned short theKey);

// ----- XMPShape utilities
XMPShape*	NewXMPShape(XMPStorageUnit* su);

//========================================================================================
//	 class FW_CAcquireASLMResourceAccess
//========================================================================================

class FW_CAcquireASLMResourceAccess
{
public:
	FW_CAcquireASLMResourceAccess();
	virtual ~FW_CAcquireASLMResourceAccess();

private:
	long		fShellLibRefNum;
	TLibraryFile*	fShellLibFile;
};

//========================================================================================
//	 class FW_TCollectionActiveIterator
//========================================================================================

template <class Item>
class FW_TCollectionActiveIterator
{
public:

  FW_TCollectionActiveIterator(const BC_TCollection<Item>&, BC_Boolean backward = FALSE); 
  ~FW_TCollectionActiveIterator();
  
  void Reset();
  BC_Boolean Next();
  BC_Boolean Previous();

  BC_Boolean IsDone() const;
  
  const Item* CurrentItem() const;
  Item* CurrentItem();
  
  Item* FirstItem();
  // Sets the index to the first item and returns it
  
  Item* LastItem();
  // Sets the index to the last item and returns it

protected:

  const BC_TCollection<Item>& fCollection;
  BC_ExtendedIndex fIndex;
  
};

//========================================================================================
//	 class FW_CDrawInitiator
//========================================================================================

class FW_CDrawInitiator : public FW_CGraphicContext
{
public:
	FW_CDrawInitiator(FW_CFacet *facet, XMPShape* clipShape = NULL);
	FW_CDrawInitiator(XMPWindow* xmpWindow);
	
	virtual 		~FW_CDrawInitiator();

	FW_CFacet*		GetFacet() const
						{return fFacet;}

private:
	FW_CFacet	*fFacet;
};

//========================================================================================
//	 Global Inline Methods
//========================================================================================

//----------------------------------------------------------------------------------------
// Max
//----------------------------------------------------------------------------------------

inline long Max(long a, long b)
{
	return a > b ? a : b;
}

//----------------------------------------------------------------------------------------
// Min
//----------------------------------------------------------------------------------------

inline long Min(long a, long b)
{
	return a < b ? a : b;
}

//----------------------------------------------------------------------------------------
// TestShiftKey
//----------------------------------------------------------------------------------------

inline FW_Boolean TestShiftKey(EventRecord* event)
{
	return (event->modifiers & 0x0200) != 0;
}

//----------------------------------------------------------------------------------------
// TestCommandKey
//----------------------------------------------------------------------------------------

inline FW_Boolean TestCommandKey(EventRecord* event)
{
	return (event->modifiers & 0x0100) != 0;
}

//----------------------------------------------------------------------------------------
// TestOptionKey
//----------------------------------------------------------------------------------------

inline FW_Boolean TestOptionKey(EventRecord* event)
{
	return (event->modifiers & 0x0800) != 0;
}

//----------------------------------------------------------------------------------------
// TestControlKey
//----------------------------------------------------------------------------------------

inline FW_Boolean TestControlKey(EventRecord* event)
{
	return (event->modifiers & 0x1000) != 0;
}

//----------------------------------------------------------------------------------------
// ::IsShiftKeyPressed
//----------------------------------------------------------------------------------------

inline FW_Boolean IsShiftKeyPressed()
{
	return ::IsKeyPressed(0x38);
}

//----------------------------------------------------------------------------------------
// ::IsOptionKeyPressed
//----------------------------------------------------------------------------------------

inline FW_Boolean IsOptionKeyPressed()
{
	return ::IsKeyPressed(0x3A);
}

//----------------------------------------------------------------------------------------
// ::IsCommandKeyPressed
//----------------------------------------------------------------------------------------

inline FW_Boolean IsCommandKeyPressed()
{
	return ::IsKeyPressed(0x37);
}

//----------------------------------------------------------------------------------------
// ::IsControlKeyPressed
//----------------------------------------------------------------------------------------

inline FW_Boolean IsControlKeyPressed()
{
	return ::IsKeyPressed(0x3B);
}


#endif