//	File:		FWUtil.h
//
//	Contains:	Some Utilities
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

#ifndef FWUTIL_H
#include <FWUtil.h>
#endif

template<class Item>
FW_TCollectionActiveIterator<Item>::
  FW_TCollectionActiveIterator(const BC_TCollection<Item>& c, BC_Boolean backward) 
    : fCollection(c)
{
	if (backward)
		fIndex = c.Length() ? c.Length() - 1 : -1;
	else
		fIndex = c.Length() ? 0 : -1;
}

template<class Item>
FW_TCollectionActiveIterator<Item>::~FW_TCollectionActiveIterator() {}
  
template<class Item>
void FW_TCollectionActiveIterator<Item>::Reset()
{
  fIndex = fCollection.Length() ? 0 : -1;
}

template<class Item>
BC_Boolean FW_TCollectionActiveIterator<Item>::Next()
{
  fIndex++;
  return !IsDone();
}

template<class Item>
BC_Boolean FW_TCollectionActiveIterator<Item>::Previous()
{
  fIndex--;
  return !IsDone();
}
template<class Item>
BC_Boolean FW_TCollectionActiveIterator<Item>::IsDone() const
{
  return ((fIndex < 0) || (fIndex >= fCollection.Length()));
}

template<class Item>
const Item* FW_TCollectionActiveIterator<Item>::CurrentItem() const
{
  return IsDone() ? 0 : &(fCollection[fIndex]);
}

template<class Item>
Item* FW_TCollectionActiveIterator<Item>::CurrentItem()
{
  return IsDone() ? 0 : &((Item&)(fCollection[fIndex]));
}

template<class Item>
Item* FW_TCollectionActiveIterator<Item>::FirstItem()
{
	if (fIndex > -1)
		fIndex = 0;
  return CurrentItem();
}

template<class Item>
Item* FW_TCollectionActiveIterator<Item>::LastItem()
{
	if (fIndex > -1)
		fIndex = fCollection.Length() - 1;
  return CurrentItem();
}

