//========================================================================================
//
//	File:				FWWindow.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWWINDOW_H
#define FWWINDOW_H

// ----- Graphic Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

//==============================================================================
//	 Forward Declarations
//==============================================================================

class 	FW_CPart;
class  	FW_CFacet;
class	FW_CFrame;

//==============================================================================
//	 defines
//==============================================================================

#define kWindoidWDEF			48				// the default windoid WDEF proc ID
#define kWindoidHilite			 1				// add this to kWindoidWDEF if the title bar can be unhilited
#define kWindoidTallTitleBar	 2				// add this to kWindoidWDEF if the title bar has a title
#define kWindoidNotResizable	 4				// add this to kWindoidWDEF if the windoid is NOT resizable
#define kWindoidZoomable		 8				// add this to kWindoidWDEF if the windoid is zoomable

//==============================================================================
//	 class FW_CWindow
//==============================================================================

#define kFW_CWindowID "appl:framework:cwindow$class,1.0.0"

class FW_CWindow
{
public:
	FW_CWindow();
	void InitWindow(FW_CPart* part, const FW_CRect& windowRect);
	void InitWindow(FW_CPart* part, XMPWindow* xmpWindow);
	virtual ~FW_CWindow();
	
	XMPWindow*		GetXMPWindow() const;
	
	void			FacetAdded(FW_CFacet* facet);
	void			FacetRemoved(FW_CFacet* facet);

	void			CloseWindow();	
	virtual void 	ShowWindow();
	virtual void 	HideWindow();
	
	virtual FW_Boolean IsShown() const;
	FW_Boolean 		IsFloating() const;
	
	void			InClose();
	
protected:
	XMPWindow*		fXMPWindow;
	FW_CPart*		fPart;
	FW_CRect		fWindowRect;

private:
	unsigned short		fFacetCount;
};

//------------------------------------------------------------------------------
//	 FW_CWindow::GetXMPWindow
//------------------------------------------------------------------------------
inline XMPWindow* FW_CWindow::GetXMPWindow() const
{
	return fXMPWindow;
}

//------------------------------------------------------------------------------
//	 FW_CWindow::IsFloating
//------------------------------------------------------------------------------
inline FW_Boolean FW_CWindow::IsFloating() const
{
	return fXMPWindow ? fXMPWindow->IsFloating() : FALSE;
}

//==============================================================================
//	class FW_CFloatingWindow
//==============================================================================

#define kFW_CFloatingWindowID "appl:framework:cfloatingwindow$class,1.0.0"

class FW_CFloatingWindow : public FW_CWindow
{
public:
	FW_CFloatingWindow();
	void 			InitFloatingWindow(FW_CPart* part,
										XMPTypeToken presentation,
										const FW_CRect& windowRect);
	void 			InitFloatingWindow(FW_CPart* part, XMPWindow* xmpWindow);
		
	virtual 		~FW_CFloatingWindow();
	
	virtual void 	ShowWindow();
	virtual void 	HideWindow();

	void			HideShowFloating();
	virtual FW_Boolean IsShown() const;

private:
	void			CreateFloatingWindow(XMPTypeToken presentation);
		
private:
	FW_Boolean		fFloatingShown;
	XMPTypeToken	fPresentation;
};

#endif
