//========================================================================================
//
//	File:				FWBaseFr.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWBASEFR_H
#include "FWBaseFr.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

// ----- Graphx Includes -----

#ifndef FWRECT_H
#include <FWRect.h>
#endif

#ifndef FWPOINT_H
#include <FWPoint.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _PART_
#include <Part.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <Quickdraw.h>
#endif

#pragma segment framework

//========================================================================================
//  Class FW_CBaseFrame
//========================================================================================

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::FW_CBaseFrame
//----------------------------------------------------------------------------------------

FW_CBaseFrame::FW_CBaseFrame()
{
	fXMPFrame = NULL;	
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::InitBaseFrame
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::InitBaseFrame(XMPFrame* xmpFrame)
{
	fXMPFrame = xmpFrame;
	fXMPFrame->IncrementRefCount();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::~FW_CBaseFrame
//----------------------------------------------------------------------------------------

FW_CBaseFrame::~FW_CBaseFrame()
{
	if (fXMPFrame)
		fXMPFrame->Release();
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::GetFrameShape
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::GetFrameShape(XMPShape* shape) const
{
	shape->CopyFrom(this->GetXMPFrame()->GetFrameShape());
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::GetFrameShapeBounds
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::GetFrameShapeBounds(FW_CRect* rect) const
{
	XMPShape* shape = this->GetXMPFrame()->GetFrameShape();
	shape->GetBoundingBox(rect);
}

//----------------------------------------------------------------------------------------
// FW_CBaseFrame::GetExtendShape
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::GetExtendShape(XMPShape* extent) const
{
	extent->CopyFrom(GetUsedShape());
	
	XMPTransform frameToContent;
	frameToContent.CopyFrom(GetInternalTransform());
	frameToContent.Invert();
	extent->Transform(&frameToContent);
}

//----------------------------------------------------------------------------------------
// FW_CBaseFrame::GetExtendRect
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::GetExtendRect(FW_CRect* rect) const
{
	XMPShape* extentShape = ::NewXMPShape();
	GetExtendShape(extentShape);
	extentShape->GetBoundingBox(rect);
	delete extentShape;
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::OffsetFrame
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::OffsetFrame(const FW_CPoint& offset)
{	
	if (offset.x != 0 || offset.y!=0)
	{
		XMPFrameFacetIterator i(fXMPFrame);
		
		for (XMPFacet* xmpFacet = i.First(); i.IsNotComplete(); xmpFacet = i.Next())
		{
			XMPTransform *transform = ::NewXMPTransform(xmpFacet->GetExternalTransform());
			
			transform->MoveBy(offset);
			
			xmpFacet->ChangeGeometry(NULL, transform);
		}

	}
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::PlaceFrame
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::PlaceFrame(const FW_CPoint& position)
{	
	XMPFrameFacetIterator *i = new XMPFrameFacetIterator(fXMPFrame);
	
	for (XMPFacet* xmpFacet = i->First(); i->IsNotComplete(); xmpFacet = i->Next())
	{
		XMPTransform *transform = ::NewXMPTransform(xmpFacet->GetExternalTransform());
		
		transform->Reset();
		transform->MoveBy(position);
		
		xmpFacet->ChangeGeometry(NULL, transform);
	}
	
	delete i;
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::ResizeFrame
//
//	Resize and move a frame. newShape is in this shape coordinate
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::ResizeFrame(XMPShape* newShape)
{	
	FW_CRect frameBounds;
	GetFrameShapeBounds(&frameBounds);
	
	FW_CRect shapeBox;
	newShape->GetBoundingBox(&shapeBox);
	
	FW_CPoint offset(-shapeBox.left + frameBounds.left, -shapeBox.top + frameBounds.top);
	XMPTransform *transform = ::NewXMPTransform(offset);
	newShape->Transform(transform);
	delete transform;
	
	offset.x *= -1;
	offset.y *= -1;	
	PlaceFrame(offset);
	
//	return RequestFrameShape(newShape);	// newShape is kept by the frame
	fXMPFrame->ChangeFrameShape(newShape);
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::DrawAllFacets
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::DrawAllFacets(XMPShape* invalidShape)
{
	XMPFrameFacetIterator i(fXMPFrame);
	for (XMPFacet* xmpFacet = i.First(); i.IsNotComplete(); xmpFacet = i.Next())
	{
		xmpFacet->Draw(invalidShape);
	}
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::Invalidate
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::Invalidate(XMPShape* invalidShape)
{
	GetXMPFrame()->Invalidate(invalidShape);
}

//----------------------------------------------------------------------------------------
//  FW_CBaseFrame::Validate
//----------------------------------------------------------------------------------------

void FW_CBaseFrame::Validate(XMPShape* validShape)
{
	GetXMPFrame()->Validate(validShape);
}


