//========================================================================================
//
//	File:				FRBorder.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWBORDER_H
#include "FWBorder.h"
#endif

#ifndef FWPROXY_H
#include "FWProxy.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

// ----- Graphic Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <Quickdraw.h>
#endif

#pragma segment framework

//==============================================================================
//	 Class FW_CBaseProxyBorder
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder::FW_CBaseProxyBorder
//------------------------------------------------------------------------------

FW_CBaseProxyBorder::FW_CBaseProxyBorder()
{
	fProxyRun = NULL;
	fBorderHasChanged = TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder::InitBaseProxyBorder
//------------------------------------------------------------------------------

void FW_CBaseProxyBorder::InitBaseProxyBorder(FW_CProxyRun *proxyRun)
{
	fProxyRun = proxyRun;
}

//------------------------------------------------------------------------------
//	 FW_CBaseProxyBorder::~FW_CBaseProxyBorder
//------------------------------------------------------------------------------

FW_CBaseProxyBorder::~FW_CBaseProxyBorder()
{
}

//==============================================================================
//	 class FW_CProxyBorder
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::FW_CProxyBorder
//------------------------------------------------------------------------------

FW_CProxyBorder::FW_CProxyBorder()
{
	fSelectRgn 			= NULL;
	fCornerHandleRgn 	= NULL;
	fEdgeHandleRgn 		= NULL;
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::InitProxyBorder
//------------------------------------------------------------------------------

void FW_CProxyBorder::InitProxyBorder(FW_CProxyRun *proxyRun)
{
	InitBaseProxyBorder(proxyRun);
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::~FW_CProxyBorder
//------------------------------------------------------------------------------

FW_CProxyBorder::~FW_CProxyBorder()
{
	DisposeSelectionRgn();
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::DisposeSelectionRgn
//------------------------------------------------------------------------------

void FW_CProxyBorder::DisposeSelectionRgn()
{
	if (fSelectRgn)
		DisposeRgn(fSelectRgn);
	if (fCornerHandleRgn)
		DisposeRgn(fCornerHandleRgn);
	if (fEdgeHandleRgn)
		DisposeRgn(fEdgeHandleRgn);
	
	fSelectRgn = NULL;
	fCornerHandleRgn = NULL;
	fEdgeHandleRgn = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::CalcProxyBorder
//------------------------------------------------------------------------------

void FW_CProxyBorder::CalcProxyBorder(FW_CFacet* facet, XMPFacet *emPrivdedXMPFacet)
{
	DisposeSelectionRgn();
	
	//	 get frame shape to draw border
	XMPRgnHandle border = NewRgn();
	
	{
		XMPShape* frameShape = ::NewXMPShape(emPrivdedXMPFacet->GetFrame()->GetFrameShape());
		
		frameShape->Transform(emPrivdedXMPFacet->GetExternalTransform());	// now in enclosure content coordinate
		
		FW_SPlatformRect qdRect;
		::GetXMPShapeQDBox(frameShape, qdRect);
		delete frameShape;
		
		::RectRgn(border, &qdRect);
	}
	
	
	XMPRgnHandle insideBorder = ::NewRgn();
	::CopyRgn(border, insideBorder);
	::InsetRgn(border, -kXMPBorderWidth, -kXMPBorderWidth);
	::DiffRgn(border, insideBorder, border);
	::DisposeRgn(insideBorder);
	
	FW_SPlatformRect borderRect = (**border).rgnBBox;
	
	GrafPtr svPort;
	::GetPort(&svPort);
	
	::SetPort(emPrivdedXMPFacet->GetWindow()->GetPlatformWindow());
	
	::OpenRgn();

	// top left handle
	::MoveTo(borderRect.left, borderRect.top);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.top);

	// top right handle
	::MoveTo(borderRect.right, borderRect.top);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.top + (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.top);

	// bottom right handle
	::MoveTo(borderRect.right, borderRect.bottom);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom);
	::LineTo(borderRect.right - (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.right - kXMPBorderWidth, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.right, borderRect.bottom);

	// bottom left handle
	::MoveTo(borderRect.left, borderRect.bottom);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom);
	::LineTo(borderRect.left + (kXMPHandleLenMultiplier * kXMPBorderWidth), borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.bottom - kXMPBorderWidth);
	::LineTo(borderRect.left + kXMPBorderWidth, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.bottom - (kXMPHandleLenMultiplier * kXMPBorderWidth));
	::LineTo(borderRect.left, borderRect.bottom);

	fCornerHandleRgn = NewRgn();
	::CloseRgn(fCornerHandleRgn);
	
	::UnionRgn(border,fCornerHandleRgn, border);
	
	::OpenRgn();

	if ((borderRect.right - borderRect.left) >= (5 * kXMPHandleLenMultiplier * kXMPBorderWidth)) {
	
		short hcenter = (borderRect.right - borderRect.left) / 2;
		short hdist = (kXMPHandleLenMultiplier * kXMPBorderWidth) / 2;
		
		// top middle handle
		::MoveTo(borderRect.left + hcenter - hdist, borderRect.top);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.top);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.top + kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.top + kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.top);
		
		// bottom middle handle
		::MoveTo(borderRect.left + hcenter - hdist, borderRect.bottom);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.bottom);
		::LineTo(borderRect.left + hcenter + hdist, borderRect.bottom - kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.bottom - kXMPBorderWidth);
		::LineTo(borderRect.left + hcenter - hdist, borderRect.bottom);
	}
	
	if ((borderRect.bottom - borderRect.top) >= (5 * kXMPHandleLenMultiplier * kXMPBorderWidth)) {
	
		short vcenter = (borderRect.bottom - borderRect.top) / 2;
		short vdist = (kXMPHandleLenMultiplier * kXMPBorderWidth) / 2;
		
		// left middle handle
		::MoveTo(borderRect.left, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.left, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.left + kXMPBorderWidth, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.left, borderRect.top + vcenter - vdist);
		
		// right middle handle
		::MoveTo(borderRect.right, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.right, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + vcenter + vdist);
		::LineTo(borderRect.right - kXMPBorderWidth, borderRect.top + vcenter - vdist);
		::LineTo(borderRect.right, borderRect.top + vcenter - vdist);
	}
	
	fEdgeHandleRgn = NewRgn();
	::CloseRgn(fEdgeHandleRgn);
	
	::SetPort(svPort);
	
	::UnionRgn(border,fEdgeHandleRgn, border);
	fSelectRgn = border;
	
	FW_CRect rect((*fSelectRgn)->rgnBBox);
	XMPShape* clipShape = ::NewXMPShape();
	GetContentClip(facet, rect, clipShape);
	if (!clipShape->IsEmpty())
	{
		::DiffRgn(fSelectRgn, clipShape->GetQDRegion(), fSelectRgn);
		::DiffRgn(fCornerHandleRgn, clipShape->GetQDRegion(), fCornerHandleRgn);
		::DiffRgn(fEdgeHandleRgn, clipShape->GetQDRegion(), fEdgeHandleRgn);
	}
	delete clipShape;
	
	BorderHasChanged(FALSE);
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::GetContenClip
//------------------------------------------------------------------------------

void FW_CProxyBorder::GetContentClip(FW_CFacet* facet, const FW_CRect& borderBox, XMPShape* clip)
{
	clip->Subtract(clip);

	FW_CRect shapeRect;
	XMPShape* clipShape = ::NewXMPShape();
	XMPFacetIterator* ite = facet->GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (XMPFacet* xmpFacet = ite->First(); ite->IsNotComplete(); xmpFacet = ite->Next())
	{
		if (xmpFacet->GetFrame()->GetPart() == GetProxyRun()->GetXMPPart())
			break;
		clipShape->CopyFrom(xmpFacet->GetClipShape());
		clipShape->Transform(xmpFacet->GetExternalTransform());
		clipShape->GetBoundingBox(&shapeRect);
		if (shapeRect.IsIntersecting(borderBox))
		{
			clip->Union(clipShape);
		}
	}
	delete ite;
	delete clipShape;
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::DrawProxyBorder
//------------------------------------------------------------------------------
//	Enclosure must be prepared for drawing

void FW_CProxyBorder::DrawProxyBorder(FW_CFacet* facet, XMPFacet* emPrivdedFacet)
{
FW_UNUSED(facet);

	if (GetProxyRun()->IsSelected())
	{
		if (HasBorderChanged())
			CalcProxyBorder(facet, emPrivdedFacet);
		
#ifdef THINK_CPLUS
		::FillRgn(fSelectRgn, XMPASLMQDGlobals.gray);
		::FillRgn(fCornerHandleRgn, XMPASLMQDGlobals.white);
		::FillRgn(fEdgeHandleRgn, XMPASLMQDGlobals.white);
#else
		::FillRgn(fSelectRgn, &XMPASLMQDGlobals.gray);
		::FillRgn(fCornerHandleRgn, &XMPASLMQDGlobals.white);
		::FillRgn(fEdgeHandleRgn, &XMPASLMQDGlobals.white);
#endif

		::FrameRgn(fCornerHandleRgn);
		::FrameRgn(fEdgeHandleRgn);
	}
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::EraseProxyBorder
//------------------------------------------------------------------------------
//	Enclosure must be prepared for drawing

void FW_CProxyBorder::EraseProxyBorder(FW_CFacet* facet, XMPFacet* emPrivdedFacet)
{	
FW_UNUSED(emPrivdedFacet);

	if (fSelectRgn != NULL)
	{
		FW_PlatformRegion newRgn = fSelectRgn;
		::HandToHand((FW_PlatformHandle *) &newRgn);
		
		XMPShape* borderShape = ::NewXMPShape(newRgn);
		::EraseRgn(fSelectRgn);
		facet->Invalidate(borderShape);
		delete borderShape;
	}
}

//------------------------------------------------------------------------------
//	 FW_CProxyBorder::WhereInProxyBorder
//------------------------------------------------------------------------------
//	1--5--2
//	|	  |
//	8	  6
//	|	  |
//  3--7--4

short FW_CProxyBorder::WhereInProxyBorder(FW_CFacet* facet, const FW_CPoint& mouse) const
{
FW_UNUSED(facet);
	FW_SPlatformPoint qdMouse = mouse;
	
	if (GetProxyRun()->IsSelected())
	{
		if (::PtInRgn(qdMouse, fCornerHandleRgn))
		{
			FW_SPlatformRect rect = (*fSelectRgn)->rgnBBox;
			
			FW_SPlatformPoint middle;
			middle.h  = (rect.right + rect.left) / 2;
			middle.v  = (rect.top + rect.bottom) / 2;
			
			if (qdMouse.h < middle.h)
			{
				if (qdMouse.v < middle.v)
					return kInTopLeftCorner;
				else
					return kInBottomLeftCorner;
			}
			else
			{
				if (qdMouse.v < middle.v)
					return kInTopRightCorner;
				else
					return kInBottomRightCorner;		
			}
		}
		else if (::PtInRgn(qdMouse, fEdgeHandleRgn))
		{
			return kInDrag; // [HLX] for now
		}
		else if (::PtInRgn(qdMouse, fSelectRgn))
			return kInDrag;
	}
	
	return 0;
}

