//========================================================================================
//
//	File:				FWContrH.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef _FWCONTRH_
#include "FWContrH.h"
#endif

#pragma segment framework

//========================================================================================
// CLASS FW_CControlMgrHelper
//========================================================================================

WindowPtr FW_CControlMgrHelper::gScratchWindow;

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::FW_CControlMgrHelper
//----------------------------------------------------------------------------------------

FW_CControlMgrHelper::FW_CControlMgrHelper() :
	fControlHandle(NULL)
{
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::~FW_CControlMgrHelper
//----------------------------------------------------------------------------------------

FW_CControlMgrHelper::~FW_CControlMgrHelper()
{
	if (fControlHandle)
		::DisposeControl(fControlHandle);
	
	if (gScratchWindow)
	{
		long count;
		::SetWRefCon(gScratchWindow,  count = (::GetWRefCon(gScratchWindow) - 1));
		if (count == 0)
		{
			::DisposeWindow(gScratchWindow);
			gScratchWindow = NULL;
		}
	}
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::InitControlMgrHelper
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::InitControlMgrHelper(const FW_SPlatformRect& ctrlRect, Str255 title,
												short initValue, short minValue, short maxValue,
												short ctrlType)
{	
	if (gScratchWindow == NULL)
	{
		FW_SPlatformRect rBounds;
		::SetRect(&rBounds, 0, 0, 10 , 10);
		gScratchWindow = ::NewCWindow(NULL,
										&rBounds,
										"\p",
										false,
										documentProc,
										NULL,
										false,
										0);
	}

	::SetWRefCon(gScratchWindow, ::GetWRefCon(gScratchWindow) + 1);
	
	fControlHandle = CreateControlHandle(gScratchWindow, ctrlRect, title,
										 initValue, minValue, maxValue, ctrlType);
	
	// ----- Remove it from the window -----
	((WindowPeek) gScratchWindow)->controlList = NULL;
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::SetBounds
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::SetBounds(const FW_SPlatformRect& controlBounds)
{
	(*fControlHandle)->contrlRect = controlBounds;
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::SetValue
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::SetValue(short value)
{
	::SetCtlValue(fControlHandle, value);
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::SetMin
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::SetMin(short min)
{
	::SetCtlValue(fControlHandle, min);
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::SetMax
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::SetMax(short max)
{
	::SetCtlValue(fControlHandle, max);
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::Draw
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::Draw(WindowPtr window)
{
	SetupControl(window);
	::Draw1Control(fControlHandle);
 	RestoreControl();
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::MouseDown
//----------------------------------------------------------------------------------------

Boolean FW_CControlMgrHelper::MouseDown(WindowPtr window, const FW_SPlatformPoint& where)
{
	SetupControl(window);
	Boolean mouseUpInWindow = ::TrackControl(fControlHandle, where, NULL);
	RestoreControl();
	
	return mouseUpInWindow;
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::CreateControlHandle
//----------------------------------------------------------------------------------------

ControlHandle FW_CControlMgrHelper::CreateControlHandle(WindowPtr window, const FW_SPlatformRect& ctrlRect,
														Str255 title,
														short initValue, short minValue, short maxValue,
														short ctrlType)
{
	return ::NewControl(window, &ctrlRect, title, true,
						initValue, minValue, maxValue,
						ctrlType, 0);
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::SetupControl
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::SetupControl(WindowPtr window)
{
	
	if (fControlHandle)
		(*fControlHandle)->contrlOwner = window;
}

//----------------------------------------------------------------------------------------
// FW_CControlMgrHelper::RestoreControl
//----------------------------------------------------------------------------------------

void FW_CControlMgrHelper::RestoreControl()
{
	if (fControlHandle)
		(*fControlHandle)->contrlOwner = FW_CControlMgrHelper::gScratchWindow;
}

