//========================================================================================
//
//	File:				FWCtlMgr.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCTLMGR_H
#include "FWCtlMgr.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

// ----- Graphic Includes -----

#ifndef FWGC_H
#include "FWGC.h"
#endif

#ifndef FWDFAULT_H
#include "FWDfault.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

// ----- Macintosh Includes

#if defined(FW_BUILD_MAC) && !defined(mathRoutinesIncludes)
#include <math routines.h>
#endif

#pragma segment framework

//==============================================================================
//	 class FW_CCtlMgrPart
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::FW_CCtlMgrPart
//------------------------------------------------------------------------------

FW_CCtlMgrPart::FW_CCtlMgrPart():
	fControlSize(ff(80), ff(20))
{
	fControlHandle = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::~FW_CCtlMgrPart
//------------------------------------------------------------------------------

FW_CCtlMgrPart::~FW_CCtlMgrPart()
{
	if (fControlHandle)
	{
		::DisposeControl(fControlHandle);
		fControlHandle = NULL;
	}
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::Initialize
//------------------------------------------------------------------------------

void FW_CCtlMgrPart::Initialize()
{
	FW_CPart::Initialize();
		
	FW_CRect rect(0, 0, fControlSize.x, fControlSize.y);
	fControlHandle = CreateControlHandle(gGraphicGlobales.gScratchWindow, rect);
	
	// ----- Remove it from the window -----
	((WindowPeek)gGraphicGlobales.gScratchWindow)->controlList = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::NewFrame
//------------------------------------------------------------------------------

FW_CFrame* FW_CCtlMgrPart::NewFrame(XMPFrame* xmpFrame, XMPTypeToken presentation)
{
	FW_UNUSED(presentation);
	
	FW_CCtlMgrFrame *frame = new FW_CCtlMgrFrame;
	frame->InitCtlMgrFrame(xmpFrame, this);
	return frame;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::SetControlSize
//------------------------------------------------------------------------------

void FW_CCtlMgrPart::SetControlSize(const FW_CPoint& controlSize)
{
	if (controlSize != fControlSize)
	{
		fControlSize = controlSize;
		
		FW_CRect rect(0, 0, controlSize.x, controlSize.y);
		XMPShape* controlShape = ::NewXMPShape(rect);
		
		FW_CPartFrameIterator ite(this);
		FW_CCtlMgrFrame *frame;
		while (!ite.IsDone())
		{
			frame = (FW_CCtlMgrFrame*)ite.CurrentItem();
			XMPShape *shape = ::NewXMPShape(controlShape);
			frame->RequestFrameShape(shape);
			frame->UpdateUsedAndActiveShapes();
			ite.Next();
		}
		
		delete controlShape;
	}
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrPart::DoClick
//------------------------------------------------------------------------------

void FW_CCtlMgrPart::DoClick(const FW_CPoint& where, XMPEventData event)
{
	FW_UNUSED(where);
	FW_UNUSED(event);
}

//==============================================================================
//	 class FW_CCtlMgrFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame::FW_CCtlMgrFrame
//------------------------------------------------------------------------------

FW_CCtlMgrFrame::FW_CCtlMgrFrame()
{
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame::InitCtlMgrFrame
//------------------------------------------------------------------------------

void FW_CCtlMgrFrame::InitCtlMgrFrame(XMPFrame* frame, FW_CPart* part)
{
	InitFrame(frame, part);
	SetFrontClicks(TRUE);	// I want the first click
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame::~FW_CCtlMgrFrame
//------------------------------------------------------------------------------

FW_CCtlMgrFrame::~FW_CCtlMgrFrame()
{
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame::NewFacet
//------------------------------------------------------------------------------

FW_CFacet* FW_CCtlMgrFrame::NewFacet(XMPFacet* xmpFacet)
{
	FW_CCtlMgrFacet* facet = new FW_CCtlMgrFacet;
	facet->InitCtlMgrFacet(xmpFacet);
	return facet;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame::FacetAdded
//------------------------------------------------------------------------------

void FW_CCtlMgrFrame::FacetAdded(FW_CFacet* facet)
{
	FW_UNUSED(facet);
	
	FW_CPoint controlSize;
	((FW_CCtlMgrPart*)GetPart())->GetControlSize(controlSize);
	FW_CRect rect(0, 0, controlSize.x, controlSize.y);
	XMPShape* controlShape = ::NewXMPShape(rect);

	this->RequestFrameShape(controlShape);
	this->UpdateUsedAndActiveShapes();
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFrame::FrameShapeChanged
//------------------------------------------------------------------------------

void FW_CCtlMgrFrame::FrameShapeChanged()
{
	FW_CRect rect;
	GetXMPFrame()->GetFrameShape()->GetBoundingBox(&rect);
	
	FW_CPoint controlSize(rect.right - rect.left, rect.bottom - rect.top);
	((FW_CCtlMgrPart*)GetPart())->SetControlSize(controlSize);
	
	FW_CFrame::FrameShapeChanged();
}

//==============================================================================
//	 class FW_CCtlMgrFacet
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::FW_CCtlMgrFacet
//------------------------------------------------------------------------------

FW_CCtlMgrFacet::FW_CCtlMgrFacet()
{
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::InitCtlMgrFacet
//------------------------------------------------------------------------------

void FW_CCtlMgrFacet::InitCtlMgrFacet(XMPFacet* xmpFacet)
{
	InitFacet(xmpFacet);
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::~FW_CCtlMgrFacet
//------------------------------------------------------------------------------

FW_CCtlMgrFacet::~FW_CCtlMgrFacet()
{
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::Draw
//------------------------------------------------------------------------------

void FW_CCtlMgrFacet::Draw(FW_CGraphicContext *gc)
{
FW_UNUSED(gc);

	ControlHandle hControl = SetupControl();
		
	::Draw1Control(hControl);
	
	RestoreControl();
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::DoMouseDown
//------------------------------------------------------------------------------

FW_Boolean FW_CCtlMgrFacet::DoMouseDown(const FW_CPoint& where, XMPEventData event)
{
FW_UNUSED(event);

	FW_CGraphicContext gc(this->GetXMPFacet());
	
	ControlHandle hControl = SetupControl();
	
	if (::TrackControl(hControl, where, NULL))
		((FW_CCtlMgrPart*)this->GetFrame()->GetPart())->DoClick(where, event);

	RestoreControl();
	
	return TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::SetupControl
//------------------------------------------------------------------------------

ControlHandle FW_CCtlMgrFacet::SetupControl()
{
	ControlHandle hControl = GetControlHandle();

	if (hControl)
	{
		FW_CRect rect;
		
		XMPShape* frameShape = ::NewXMPShape();
		GetFrame()->GetFrameShape(frameShape);
		frameShape->Transform(GetExternalTransform());
		frameShape->GetBoundingBox(&rect);
		delete frameShape;
		
		rect.Offset(-rect.left, -rect.top);
		(*hControl)->contrlRect = rect;	
		(*hControl)->contrlOwner = GetXMPWindow()->GetPlatformWindow();
	}
	
	return hControl;
}

//------------------------------------------------------------------------------
//	 FW_CCtlMgrFacet::RestoreControl
//------------------------------------------------------------------------------

void FW_CCtlMgrFacet::RestoreControl()
{
	ControlHandle hControl = GetControlHandle();
	if (hControl)
		(*hControl)->contrlOwner = gGraphicGlobales.gScratchWindow;	
}
