//========================================================================================
//
//	File:				FWFacet.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWPROXY_H
#include "FWProxy.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWWINDOW_H
#include "FWWindow.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWDROPTR_H
#include "FWDropTr.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef FWMEMMGR_H
#include <FWMemMgr.h>
#endif

// ----- Graphic Includes -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWGC_H
#include "FWGC.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _TRANSLAT_
#include <Translat.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _DGITMITR_
#include <DgItmItr.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <QuickDraw.h>
#endif

#pragma segment framework

//==============================================================================
//	 class FW_CFacet
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CFacet::FW_CFacet
//------------------------------------------------------------------------------

FW_CFacet::FW_CFacet()
{
	fXMPFacet = NULL;
	fRedraw = TRUE;
	fCursorInContent = FALSE;
	fCanAcceptDrop = FALSE;
	fDropTracker = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::InitFacet
//------------------------------------------------------------------------------

void FW_CFacet::InitFacet(XMPFacet* xmpFacet)
{
	fXMPFacet = xmpFacet;

	xmpFacet->SetPartInfo((XMPInfoType)this);
	
	fFrame = FW_CFrame::XMPtoFWFrame(xmpFacet->GetFrame());
	
	fDropTracker = this->NewDropTracker(this);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::~FW_CFacet
//------------------------------------------------------------------------------

FW_CFacet::~FW_CFacet()
{
	XMPFacet *xmpFacet = GetXMPFacet();
	if (xmpFacet)
		xmpFacet->SetPartInfo((XMPInfoType)NULL);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::XMPtoFWFacet
//------------------------------------------------------------------------------

FW_CFacet* FW_CFacet::XMPtoFWFacet(XMPFacet* xmpFacet)
{
	return xmpFacet != NULL ? (FW_CFacet*)xmpFacet->GetPartInfo() : (FW_CFacet*)NULL;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::CalcDragHiliteRgn
//------------------------------------------------------------------------------

FW_PlatformRegion FW_CFacet::CalcDragHiliteRgn()
{
	return this->GetXMPFacet()->GetActiveShape()->CopyQDRegion();
}

//------------------------------------------------------------------------------
//	 FW_CFacet::DragEnter
//------------------------------------------------------------------------------
// 'where' is in window coordinate

void FW_CFacet::DragEnter(XMPDragItemIterator* dragInfo, const FW_CPoint& where)
{
	XMPTranslation *translate = fFrame->GetPart()->GetSession()->GetTranslation();
	XMPType applehfs = translate->GetISOTypeFromPlatformType('hfs ', kXMPPlatformDataType);

	XMPStorageUnit *dragSU;
	HFSFlavor *valueData;
	long valueSize;
	FW_CPart *thePart = this->GetFrame()->GetPart();
	FW_CSelection* selection = thePart->GetSelection();
	
	fCanAcceptDrop = FALSE;

	for (dragSU = dragInfo->First(); dragSU; dragSU = dragInfo->Next())
		if (thePart->IsEmbeddingPart() || dragSU->Exists(kXMPPropContents, thePart->GetContentPropertyValueType(), 0))
			fCanAcceptDrop = TRUE;
		else if (dragSU->Exists(kXMPPropContents, applehfs, 0))
		{
			dragSU->Focus(kXMPPropContents, kXMPPosUndefined, applehfs, 0, kXMPPosUndefined);		
			valueSize = dragSU->GetSize();
			valueData = (HFSFlavor*)FW_CMemoryManager::AllocateBlock(valueSize);
			if (valueData)
			{
				dragSU->GetValue(valueSize, (XMPValue)valueData);
				if (valueData->fileType == 'bnto')
					fCanAcceptDrop = TRUE;
			}
			FW_CMemoryManager::FreeBlock(valueData);
		}
	
	fCursorInContent = FALSE;
	
	if (fCanAcceptDrop && fDropTracker)
		fDropTracker->HandleBeginTracking(where);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::DragWithin
//------------------------------------------------------------------------------
// 'where' is in window coordinate

void FW_CFacet::DragWithin(XMPDragItemIterator* dragInfo, const FW_CPoint& where)
{
	FW_UNUSED(dragInfo);
	
	if (!fCanAcceptDrop)
		return;
		
	DragReference theDrag;
	unsigned long attributes;
	XMPSession* session = this->GetFrame()->GetPart()->GetSession();
	
	FW_CGraphicContext di(this->GetXMPWindow());		// focus drawing to the window

	attributes = session->GetDragAndDrop()->GetDragAttributes();
	theDrag = session->GetDragAndDrop()->GetDragReference();
	
	if (this->GetXMPFacet()->ContainsPoint(where))
	{
		if (!fCursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
		{
			FW_PlatformRegion hiliteRgn = this->CalcDragHiliteRgn();
			FW_SPlatformPoint offset = this->GetWindowContentTransform()->GetQDOffset();
			::OffsetRgn(hiliteRgn, offset.h, offset.v);
			::ShowDragHilite(theDrag, hiliteRgn, this->DrawDragHiliteInside());
			fCursorInContent = TRUE;
		}
	}
	else
	{
		if (fCursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
			::HideDragHilite(theDrag);
		fCursorInContent = FALSE;
	}
	
	if (fDropTracker)
		fDropTracker->HandleContinueTracking(where);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::DragLeave
//------------------------------------------------------------------------------
// 'where' is in window coordinate

void FW_CFacet::DragLeave(const FW_CPoint& where)
{
	if (!fCanAcceptDrop)
		return;
	
	DragReference theDrag;
	unsigned long attributes;
	XMPSession* session = this->GetFrame()->GetPart()->GetSession();
	
	FW_CGraphicContext di(this->GetXMPWindow());	// focus drawing on the window

	attributes = session->GetDragAndDrop()->GetDragAttributes();
	theDrag = session->GetDragAndDrop()->GetDragReference();
	
	if (fCursorInContent && (attributes & kXMPdragHasLeftSourceFrame))
		HideDragHilite(theDrag);
		
	fCursorInContent = FALSE;
	
	if (fDropTracker)
		fDropTracker->HandleEndTracking(where);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::Drop
//------------------------------------------------------------------------------

XMPDropResult FW_CFacet::Drop(XMPDragItemIterator* dropInfo, const FW_CPoint& where)
{
	XMPStorageUnit *dropSU;
	DragReference theDrag;
	unsigned long attributes;
	FW_CPoint dragPoint, pinnedPoint;
	FW_SPlatformPoint qd;
	short mouseDownModifiers, mouseUpModifiers, movePart;
	
	if (!fCanAcceptDrop)
		return kXMPDropFail;
	
	FW_CGraphicContext di(this->GetXMPWindow());		// focus drawing to the window

	// Get the attributes for this drag
	XMPSession* session = this->GetFrame()->GetPart()->GetSession();
	attributes = session->GetDragAndDrop()->GetDragAttributes();
	theDrag = session->GetDragAndDrop()->GetDragReference();
	::GetDragModifiers(theDrag, 0L, &mouseDownModifiers, &mouseUpModifiers);
	movePart = (attributes & kXMPdragIsInSourceFrame) && (!((mouseDownModifiers & optionKey) | (mouseUpModifiers & optionKey)));

	FW_Boolean acceptedDrop = FALSE;
	for (dropSU = dropInfo->First(); dropSU && !acceptedDrop; dropSU = dropInfo->Next())
	{		
		FW_CSelection* selection = this->GetFrame()->GetPart()->GetSelection();
		if (movePart)
		{
			GetDragOrigin(theDrag, &qd);
			::GlobalToLocal(&qd);
			dragPoint = qd;
			
			GetDragMouse(theDrag, NULL, &qd);
			::GlobalToLocal(&qd);
			pinnedPoint = qd;
			
			acceptedDrop = selection->DoDroppedInSameFrame(dropSU, this, dragPoint, pinnedPoint);
		}
		else
		{
			FW_SPlatformPoint mouseDownOffset = {0,0};
			if (dropSU->Exists(kXMPPropMouseDownOffset, kQDPoint, 0))
			{
				dropSU->Focus(kXMPPropMouseDownOffset, kXMPPosUndefined, kQDPoint, 0, kXMPPosFirstSib);
				dropSU->GetValue(sizeof(FW_SPlatformPoint), (XMPValue) &mouseDownOffset);
			}
			FW_CPoint originPoint(mouseDownOffset);
			acceptedDrop = selection->DoDrop(dropSU, this, originPoint, where);
		}
	}
	this->GetFrame()->GetPart()->Changed();
	return kXMPDropCopy;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::DrawDragHiliteInside
//------------------------------------------------------------------------------

FW_Boolean FW_CFacet::DrawDragHiliteInside()
{
	return TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::NewDropTracker
//------------------------------------------------------------------------------

FW_CDropTracker* FW_CFacet::NewDropTracker(FW_CFacet* facet)
{
FW_UNUSED(facet);

	return NULL;
}

//-------------------------------------------------------------------------
// 	 FW_CFacet::HandleMouseDown
//-------------------------------------------------------------------------

FW_Boolean FW_CFacet::HandleMouseDown(XMPFacet* embeddedXMPFacet, XMPEventData event)
{
	XMPWindow* window = GetXMPWindow();

	if (!IsInFloatingWindow())
	{
		FW_Boolean firstClick = FALSE;

		if (!window->IsActive())
		{
			window->Select();
			firstClick = TRUE;
		}
		
		if (!fFrame->IsActive())
		{
			fFrame->ActivateFrame(this);
			firstClick = TRUE;
		}
				
		if (firstClick && !fFrame->GetFrontClicks())
			return TRUE;
	}
	
	FW_SPlatformPoint mousePos = event->where;
	::SetPort(window->GetPlatformWindow());
	::SetOrigin(0,0);
	::GlobalToLocal(&mousePos);
	FW_CPoint windowPoint(mousePos);	
	windowPoint = GetWindowContentTransform()->InvertPoint(windowPoint);

	if (event->what == mouseDown)
	{
		return DoMouseDown(windowPoint, event);
	}
	else
	{
		return DoMouseDownInEmbeddedFrame(embeddedXMPFacet, windowPoint, event);
	}
}

//-------------------------------------------------------------------------
//	 FW_CFacet::DoMouseDown
//-------------------------------------------------------------------------

FW_Boolean FW_CFacet::DoMouseDown(const FW_CPoint& where, XMPEventData event)
{
FW_UNUSED(where);
FW_UNUSED(event);
	return FALSE;
}

//-------------------------------------------------------------------------
//	 FW_CFacet::HandleMouseUp
//-------------------------------------------------------------------------

FW_Boolean FW_CFacet::HandleMouseUp(XMPEventData event)
{
	FW_SPlatformPoint mousePos = event->where;
	::SetPort(GetXMPWindow()->GetPlatformWindow());
	::SetOrigin(0,0);
	::GlobalToLocal(&mousePos);
	FW_CPoint windowPoint(mousePos);	
	windowPoint = GetWindowContentTransform()->InvertPoint(windowPoint);

	return DoMouseUp(windowPoint, event);
}

//-------------------------------------------------------------------------
//	 FW_CFacet::DoMouseUp
//-------------------------------------------------------------------------

FW_Boolean FW_CFacet::DoMouseUp(const FW_CPoint& where, XMPEventData event)
{
FW_UNUSED(where);
FW_UNUSED(event);
	return FALSE;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::WindowActivated
//------------------------------------------------------------------------------

void FW_CFacet::WindowActivated()
{
	FW_CFrame *frame = GetFrame();
	if (frame->NeedsActivating() && !IsInFloatingWindow()) 	// Floating window always active
	{
		frame->ActivateFrame(this);
		frame->SetNeedsActivating(FALSE);
	}
}

//------------------------------------------------------------------------------
//	 FW_CFacet::WindowDeActivated
//------------------------------------------------------------------------------

void FW_CFacet::WindowDeActivated()
{
	if (!IsInFloatingWindow())
	{
		FW_CFrame *frame = GetFrame();
		frame->SetNeedsActivating(frame->IsActive());
	}
}

//------------------------------------------------------------------------------
//	 FW_CFacet::GeometryChanged
//------------------------------------------------------------------------------

void FW_CFacet::GeometryChanged()
{
	ClipEmbeddedFacets();
}

//------------------------------------------------------------------------------
//	 FW_CFacet::ClipEmbeddedFacets
//------------------------------------------------------------------------------

void FW_CFacet::ClipEmbeddedFacets()
{
	XMPFacet* xmpFacet = GetXMPFacet();
	
	XMPShape* workingClip = NewXMPShape(xmpFacet->GetClipShape());
	
	XMPMacTransform* wclipTrans = ::NewXMPTransform(GetWindowFrameTransform());
	workingClip->Transform(wclipTrans);		// so now in window coordinate
	
	XMPFacetIterator* iter = xmpFacet->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (XMPFacet* embeddedFacet = iter->First();
		 iter->IsNotComplete();
		 embeddedFacet = iter->Next())
	{
		// ----- Set the clip to the used shape -----
		XMPShape* embeddedClipShape = ::NewXMPShape(embeddedFacet->GetFrame()->GetUsedShape());
		
		// ----- 
		wclipTrans->CopyFrom(embeddedFacet->GetWindowFrameTransform());
		wclipTrans->Invert();
		workingClip->Transform(wclipTrans);		// now in frame coordinate
		
		// ----- Set the clip of embFrame -----
		embeddedClipShape->Intersect(workingClip);
		embeddedFacet->ChangeGeometry(embeddedClipShape, NULL);
				
		// ----- remove embFrame clip from the working clip -----
		workingClip->Subtract(embeddedFacet->GetClipShape());
		
		// ----- Put back working clip in window coordinate -----
		wclipTrans->Invert();
		workingClip->Transform(wclipTrans);
	}
	delete iter;
	
	// loop to clip for part content here
	
	delete workingClip;
	delete wclipTrans;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::UpdateActiveShape
//------------------------------------------------------------------------------

void FW_CFacet::UpdateActiveShape()
{
	XMPShape* scratch = ::NewXMPShape(this->GetFrame()->GetXMPFrame()->GetFrameShape());
	this->GetXMPFacet()->ChangeActiveShape(scratch);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::CreateEmbeddedFacet
//------------------------------------------------------------------------------

XMPFacet* FW_CFacet::CreateEmbeddedFacet(FW_CProxyFrame *itsFrame, XMPShape* clipShape, XMPTransform* externalTransform, 
						   		 		XMPFacet* siblingFacet, XMPFramePosition position)
{
	return GetXMPFacet()->CreateEmbeddedFacet(itsFrame->GetXMPFrame(), clipShape, externalTransform,
												siblingFacet, position);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::GetWindow
//------------------------------------------------------------------------------

FW_CWindow* FW_CFacet::GetWindow() const
{
	return GetFrame()->GetPart()->XMPtoFWWindow(GetXMPFacet()->GetWindow());
}

//------------------------------------------------------------------------------
//	 FW_CFacet::Draw
//------------------------------------------------------------------------------

void FW_CFacet::Draw(FW_CGraphicContext *gc)
{
FW_UNUSED(gc);
}

//-------------------------------------------------------------------------
//	 FW_CFacet::MouseEnter
//-------------------------------------------------------------------------

void FW_CFacet::MouseEnter(const FW_CPoint& where)
{
FW_UNUSED(where);
	::InitCursor();
}

//-------------------------------------------------------------------------
//	 FW_CFacet::MouseWithin
//-------------------------------------------------------------------------

void FW_CFacet::MouseWithin(const FW_CPoint& where)
{
FW_UNUSED(where);
}

//-------------------------------------------------------------------------
//	 FW_CFacet::MouseLeave
//-------------------------------------------------------------------------

void FW_CFacet::MouseLeave()
{
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::DoMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFacet::DoMouseDownInEmbeddedFrame(XMPFacet* embeddedXMPFacet, const FW_CPoint& where, XMPEventData event)
{
FW_UNUSED(embeddedXMPFacet);

	return DoMouseDown(where, event);
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::HandleMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFacet::HandleMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, XMPEventData event)
{
	XMPWindow* window = GetXMPWindow();
	
	if (window->IsActive())
	{
		fFrame->ActivateFrame(this);
	}
	else
	{
		window->Select();
		return TRUE;
	}
	
	FW_SPlatformPoint mousePos = event->where;
	::SetPort(window->GetPlatformWindow());
	::SetOrigin(0,0);
	::GlobalToLocal(&mousePos);
	FW_CPoint windowPoint(mousePos);	
	windowPoint = GetWindowContentTransform()->InvertPoint(windowPoint);

	return DoMouseDownInEmbeddedFrameBorder(embeddedXMPFacet,
											 windowPoint, 
											 event);
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::DoMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFacet::DoMouseDownInEmbeddedFrameBorder(XMPFacet* embeddedXMPFacet, const FW_CPoint& where, XMPEventData event)
{
FW_UNUSED(embeddedXMPFacet);
FW_UNUSED(where);
FW_UNUSED(event);

	return FALSE;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::RemoveEmbeddedFacets
//------------------------------------------------------------------------------

void FW_CFacet::RemoveEmbeddedFacets()
{
	// ----- We need to do it in two steps
	BC_TDynamicCollection<XMPFacet*, BC_CUnmanaged> children(1);	// Assume average of single facet per frame
	XMPFacetIterator* ite = GetXMPFacet()->CreateFacetIterator(kXMPChildrenOnly, kXMPFrontToBack);
	for (XMPFacet* xmpChildFacet = ite->First(); ite->IsNotComplete(); xmpChildFacet = ite->Next())
	{
		children.Append(xmpChildFacet);
	}
	delete ite;
	
	// ----- Now we can remove them
	BC_TCollectionActiveIterator<XMPFacet*> iter(children);
	
	while (!iter.IsDone())
	{
		xmpChildFacet = *iter.CurrentItem();
		GetXMPFacet()->RemoveFacet(xmpChildFacet);
		delete xmpChildFacet;
		iter.Next();
	}
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::InvalidateAllFacets
//----------------------------------------------------------------------------------------
// invalidShape is in content coordinate

void FW_CFacet::InvalidateAllFacets(XMPShape* invalidShape, FW_Boolean skipThis)
{
	XMPShape* shape = NULL;
	if (invalidShape)
	{
		shape = ::NewXMPShape(invalidShape);
		shape->Transform(fFrame->GetXMPFrame()->GetInternalTransform()); 	// content -> frame coordinate
	}

	fFrame->GetPart()->InvalidateAllFrames(fFrame->GetIdentifier(), skipThis ? this : (FW_CFacet*)NULL, shape);
	
	delete shape;
}

//----------------------------------------------------------------------------------------
//	 FW_CFacet::ValidateAllFacets
//----------------------------------------------------------------------------------------
// validShape is in content coordinate

void FW_CFacet::ValidateAllFacets(XMPShape* validShape, FW_Boolean skipThis)
{
	XMPShape* shape = NULL;
	if (validShape)
	{
		shape = ::NewXMPShape(validShape);
		shape->Transform(fFrame->GetXMPFrame()->GetInternalTransform());	// content -> frame coordinate
	}

	fFrame->GetPart()->ValidateAllFrames(fFrame->GetIdentifier(), skipThis ? this :  (FW_CFacet*)NULL, shape);
	
	delete shape;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::GetEmbeddedXMPFacet
//------------------------------------------------------------------------------
//	Return the <first> facet belonging to proxy embedded in this 

XMPFacet* FW_CFacet::GetEmbeddedXMPFacet(FW_CProxyRun* proxy)
{
	XMPFacetIterator i(GetXMPFacet(), kXMPChildrenOnly, kXMPFrontToBack);

	for (XMPFacet* xmpFacet = i.First(); i.IsNotComplete(); xmpFacet = i.Next())
	{
		if (xmpFacet->GetFrame()->GetPart() == proxy->GetXMPPart())
			return xmpFacet;
	}
	
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CFacet::Invalidate
//------------------------------------------------------------------------------
// invalidShape is in frame coordinate

void FW_CFacet::Invalidate(XMPShape *invalidShape)
{
	if (fRedraw)
		GetXMPFacet()->Invalidate(invalidShape);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::Validate
//------------------------------------------------------------------------------
// validShape is in frame coordinate

void FW_CFacet::Validate(XMPShape *validShape)
{
	if (fRedraw)
		GetXMPFacet()->Validate(validShape);
}

//------------------------------------------------------------------------------
//	 FW_CFacet::SetRedraw
//------------------------------------------------------------------------------

FW_Boolean FW_CFacet::SetRedraw(FW_Boolean state)
{
	FW_Boolean temp = fRedraw;
	fRedraw = state;
	return temp;
}
