//========================================================================================
//
//	File:				FWFrame.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWWINDOW_H
#include "FWWindow.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

// ----- Graphic Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef FWPART_H
#include <FWPart.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _FOCUSSET_
#include <FocusSet.h>
#endif

#ifndef _FOCUSITR_
#include <FocusItr.h>
#endif

#ifndef _SUVIEW_
#include <SUView.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(mathRoutinesIncludes)
#include <math routines.h>
#endif

#pragma segment framework

//==============================================================================
//	 class FW_CFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CFrame::FW_CFrame
//------------------------------------------------------------------------------

FW_CFrame::FW_CFrame()
{
	fFocusSet = NULL;
	fGetFrontClicks = TRUE;
	fNeedsActivating = FALSE;
	fActiveFacet = NULL;
	fCanBeActiveFrame = TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CFrame::InitFrame
//
//	Should be called FIRST in CYourFrame::IYourFrame
//------------------------------------------------------------------------------

void FW_CFrame::InitFrame(XMPFrame* xmpFrame, FW_CPart* part)
{
	InitBaseFrame(xmpFrame);
	SetNextEventHandler(part);
	SetIdentifier(xmpFrame->GetPresentation());
	
	fFocusSet = new XMPFocusSet;
	fFocusSet->InitFocusSet();
	
	xmpFrame->SetPartInfo((XMPInfoType)this);
	
	fNeedsActivating = IsRoot();
}

//------------------------------------------------------------------------------
//	 FW_CFrame::~FW_CFrame
//------------------------------------------------------------------------------

FW_CFrame::~FW_CFrame()
{
	XMPFrame *xmpFrame = GetXMPFrame();
	if (xmpFrame)
		xmpFrame->SetPartInfo((XMPInfoType)NULL);
	
	delete fFocusSet;
	fFocusSet = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CFrame::CloseDisplayFrame
//
//	When overridden, call inherited::CloseDisplayFrame LAST
//------------------------------------------------------------------------------

void FW_CFrame::CloseDisplayFrame()
{
	// Relinquish what ever focus we have
	RelinquishFocusSet();
}

//------------------------------------------------------------------------------
//	 FW_CFrame::RemoveDisplayFrame
//
//	When overridden, call inherited::RemoveDisplayFrame LAST
//------------------------------------------------------------------------------

void FW_CFrame::RemoveDisplayFrame()
{
	// Relinquish what ever focus we have
	RelinquishFocusSet();
}

//------------------------------------------------------------------------------
//	 FW_CFrame::AddedToDisplayList
//------------------------------------------------------------------------------

void FW_CFrame::AddedToDisplayList()
{
}

//-------------------------------------------------------------------------
//	 FW_CFrame::HandleKeyDown
//-------------------------------------------------------------------------

FW_Boolean FW_CFrame::HandleKeyDown(XMPEventData event)
{
FW_UNUSED(event);
	
	return FALSE;
}

//-------------------------------------------------------------------------
//	 FW_CFrame::DoMenuEvent
//-------------------------------------------------------------------------

FW_Boolean FW_CFrame::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	FW_Boolean result = TRUE;
	FW_CSelection* selection = GetPart()->GetSelection();
	
	switch (commandID)
	{
		case kXMPCommandViewAsWin:
			GetPart()->Open(this->GetXMPFrame());
			break;
		
		case kXMPCommandCut:
			if (selection && selection->DoCut(this))
				GetPart()->Changed();
			break;
			
		case kXMPCommandCopy:
			if (selection)
				selection->DoCopy(this);
			break;
		
		default:
			result = GetPart()->DoMenuEvent(menuBar, commandID);	
	}
			
	return result;
}

//-------------------------------------------------------------------------
//	 FW_CFrame::DoIdle
//-------------------------------------------------------------------------

FW_Boolean FW_CFrame::DoIdle()
{
	return GetPart()->DoIdle();
}

//-------------------------------------------------------------------------
//	 FW_CFrame::InstallMenus
//
// 	By default call the part with a null menuBar
//-------------------------------------------------------------------------

void FW_CFrame::InstallMenus()
{
	GetPart()->InstallMenus(NULL);	// will install the part menubar
}

//-------------------------------------------------------------------------
//	FW_CFrame::DoAdjustMenus
//
//	When overridden, call inherited::DoAdjustMenus FIRST
//-------------------------------------------------------------------------

void FW_CFrame::DoAdjustMenus(XMPMenuBar* menuBar)
{
	GetPart()->DoAdjustMenus(menuBar);

	menuBar->EnableCommand(kXMPCommandViewAsWin, !IsRoot());
}

//-------------------------------------------------------------------------
//	 FW_CFrame::FocusStateChanged
//
//	When overridden, call inherited::FocusStateChanged FIRST
//-------------------------------------------------------------------------

void FW_CFrame::FocusStateChanged(XMPTypeToken focus, FW_Boolean newState)
{
	FW_CPart *itsPart = GetPart();
	if (newState)
	{
		if (focus == itsPart->GetSelectionFocusToken())
		{
			Activate();
			itsPart->SetActiveFrame(this);
		}
		if (focus == itsPart->GetMenuFocusToken())
		{
			InstallMenus();
		}
	}
	else
	{
		if (focus == itsPart->GetSelectionFocusToken())
		{
			Deactivate();
			itsPart->SetActiveFrame(NULL);
		}
	}
}

//-------------------------------------------------------------------------
// 	FW_CFrame::ActivateFrame
//-------------------------------------------------------------------------
//	Should not need to be override. Override instead FocusStateChanged.

FW_Boolean FW_CFrame::ActivateFrame(FW_CFacet* activatedFacet)
{ 
	FW_Boolean result = TRUE;
	
	SetActiveFacet(activatedFacet);
	
	if (!IsActive())
	{
		if (RequestFocusSet())
		{
			XMPFocusSetIterator* ite = fFocusSet->CreateIterator();
			for (XMPTypeToken token = ite->First(); ite->IsNotComplete(); token = ite->Next())
				GetPart()->FocusAcquired(token, GetXMPFrame());
			delete ite;
		}
		else
		{
			result = FALSE;	//	 Couln't activate it
		}
	}
	
	return result;
}

//-------------------------------------------------------------------------
//	 FW_CFrame::DeactivateFrame
//-------------------------------------------------------------------------
//	Should not need to be override. Override instead FocusStateChanged.

void FW_CFrame::DeactivateFrame()
{
	if (IsActive())
	{
		RelinquishFocusSet();
		
		XMPFocusSetIterator* ite = fFocusSet->CreateIterator();
		for (XMPTypeToken token = ite->First(); ite->IsNotComplete(); token = ite->Next())
			GetPart()->FocusLost(token, GetXMPFrame());
		delete ite;
	}
}

//-------------------------------------------------------------------------
//	 FW_CFrame::XMPtoFWFrame
//-------------------------------------------------------------------------

FW_CFrame* FW_CFrame::XMPtoFWFrame(XMPFrame* xmpFrame)
{
	return xmpFrame ? (FW_CFrame*)xmpFrame->GetPartInfo() : (FW_CFrame*)NULL;
}

//------------------------------------------------------------------------------
// 	FW_CFrame::FrameShapeChanged
//
//	When overridden, call inherited::FrameShapeChanged
//------------------------------------------------------------------------------

void FW_CFrame::FrameShapeChanged()
{
	UpdateUsedAndActiveShapes();
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::SetActiveFacet
//----------------------------------------------------------------------------------------
void FW_CFrame::SetActiveFacet(FW_CFacet*  activeFacet)
{
	fActiveFacet = activeFacet;
}

//------------------------------------------------------------------------------
//	 FW_CFrame::UpdateUsedAndActiveShapes
//------------------------------------------------------------------------------

void FW_CFrame::UpdateUsedAndActiveShapes()
{	
	UpdateUsedShape();

	FW_CFrameFacetIterator iter(this);	
	while (!iter.IsDone())
	{
		iter.CurrentItem()->UpdateActiveShape();
		iter.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CFrame::UpdateUsedShape
//------------------------------------------------------------------------------

void FW_CFrame::UpdateUsedShape()
{	
	XMPShape* scratch = ::NewXMPShape();
	GetFrameShape(scratch);
	this->GetXMPFrame()->ChangeUsedShape(scratch);
}

//------------------------------------------------------------------------------
//	 FW_CFrame::AddToFocusSet
//------------------------------------------------------------------------------

void FW_CFrame::AddToFocusSet(XMPTypeToken focus)
{
	fFocusSet->Add(focus);
}

//------------------------------------------------------------------------------
//	 FW_CFrame::RemoveFromFocusSet
//------------------------------------------------------------------------------

void FW_CFrame::RemoveFromFocusSet(XMPTypeToken focus)
{
	fFocusSet->Remove(focus);
}

//------------------------------------------------------------------------------
//	 FW_CFrame::RelinquishFocusSet
//
//	Just for practical purpose (this is a private method)
//------------------------------------------------------------------------------

void FW_CFrame::RelinquishFocusSet()
{
	GetPart()->GetSession()->GetArbitrator()->RelinquishFocusSet(fFocusSet, GetXMPFrame());
}

//------------------------------------------------------------------------------
//	 FW_CFrame::RequestFocusSet
//
//	Just for practical purpose (this is a private method)
//------------------------------------------------------------------------------

FW_Boolean FW_CFrame::RequestFocusSet()
{
	return GetPart()->GetSession()->GetArbitrator()->RequestFocusSet(fFocusSet, GetXMPFrame());
}

//------------------------------------------------------------------------------
// 	FW_CFrame::FacetAdded
//
//	When overridden call inherited::FacetAdded FIRST
//------------------------------------------------------------------------------

void FW_CFrame::FacetAdded(FW_CFacet* facet)
{
	// Install the frame with the newest facet as the active frame
	if (this->CanBeActiveFrame())
		this->GetPart()->SetActiveFrame(this);
		
	// Install the newest facet as the active facet
	this->SetActiveFacet(facet);
	
	// ----- If the facet's xmpWindow doesn't have a FW_CWindow,
	// ----- then create one
	XMPWindow* xmpWindow = facet->GetXMPWindow();	
	FW_CWindow* window = GetPart()->XMPtoFWWindow(xmpWindow);
	
	if (window == NULL)
	{
		window = new FW_CWindow;
		window->InitWindow(GetPart(), xmpWindow);
	}
	
	// ----- Notify the FW_CWindow
	window->FacetAdded(facet);
}

//------------------------------------------------------------------------------
// 	FW_CFrame::FacetRemoved
//
//	When overridden call inherited::FacetAdded LAST
//------------------------------------------------------------------------------

void FW_CFrame::FacetRemoved(FW_CFacet* facet)
{
	if (this->GetActiveFacet() == facet)
	{
		FW_CFrameFacetIterator iter(this);
		this->SetActiveFacet(iter.CurrentItem());
	}
	
	// ----- Notify the FW_CWindow
	FW_CWindow* window = GetPart()->XMPtoFWWindow(facet->GetXMPWindow());
	window->FacetRemoved(facet);
}

//------------------------------------------------------------------------------
//	 FW_CFrame::CreateChildWindow
//------------------------------------------------------------------------------

XMPWindow* FW_CFrame::CreateChildWindow(XMPISOStr windowTitle)
{
	FW_CRect windRect(ff(100), ff(100), ff(300), ff(300));
	
	return GetPart()->CreateXMPWindow(this,
									 GetPart()->GetSession()->Tokenize(kXMPViewAsFrame),
									 GetPresentation(),
									 windowTitle,
									 windRect,
									 TRUE,		// close box
									 TRUE,		// resizable
									 FALSE,		// floating
									 TRUE,		// should save
									 zoomDocProc);
}

//-------------------------------------------------------------------------
//	 FW_CFrame::InvalidateAllFacets
//-------------------------------------------------------------------------
//	invalidateShape must be in frame coordinate

void FW_CFrame::InvalidateAllFacets(FW_CFacet* skipFacet, XMPShape* invalidShape)
{
	FW_CFrameFacetIterator iter(this);	
	FW_CFacet* facet;
	while (!iter.IsDone())
	{
		facet = iter.CurrentItem();
		if (facet != skipFacet)
			facet->Invalidate(invalidShape);
		iter.Next();
	}
}

//-------------------------------------------------------------------------
//	 FW_CFrame::ValidateAllFacets
//-------------------------------------------------------------------------
//	validShape must be in frame coordinate

void FW_CFrame::ValidateAllFacets(FW_CFacet* skipFacet, XMPShape* validShape)
{
	FW_CFrameFacetIterator iter(this);	
	FW_CFacet* facet;
	while (!iter.IsDone())
	{
		facet = iter.CurrentItem();
		if (facet != skipFacet)
			facet->Validate(validShape);
		iter.Next();
	}
}

//-------------------------------------------------------------------------
//	 FW_CFrame::Invalidate
//-------------------------------------------------------------------------
//	invalidateShape must be in frame coordinate

void FW_CFrame::Invalidate(XMPShape* invalidShape)
{
	FW_CFrameFacetIterator iter(this);	
	while (!iter.IsDone())
	{
		iter.CurrentItem()->Invalidate(invalidShape);
		iter.Next();
	}
}

//-------------------------------------------------------------------------
//	 FW_CFrame::Validate
//-------------------------------------------------------------------------
//	validShape must be in frame coordinate

void FW_CFrame::Validate(XMPShape* validShape)
{
	FW_CFrameFacetIterator iter(this);	
	while (!iter.IsDone())
	{
		iter.CurrentItem()->Validate(validShape);
		iter.Next();
	}
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::HasSelectionFocus()
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HasSelectionFocus() const
{	
	return GetXMPFrame() == GetPart()->GetSession()->GetArbitrator()->GetFocusOwner(GetPart()->GetSelectionFocusToken());
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::ViewTypeChanged
//----------------------------------------------------------------------------------------

void FW_CFrame::ViewTypeChanged()
{	
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::PresentationChanged()
//----------------------------------------------------------------------------------------

void FW_CFrame::PresentationChanged()
{	
	SetIdentifier(GetPresentation());
}

//----------------------------------------------------------------------------------------
//	 FW_CFrame::SequenceChanged()
//----------------------------------------------------------------------------------------

void FW_CFrame::SequenceChanged()
{	
}

//-------------------------------------------------------------------------
//	 FW_CFrame::ChangeSelectionState
//-------------------------------------------------------------------------

void FW_CFrame::ChangeSelectionState(SelectionState state)
{
	SelectionStateChanged(fSelectionState, state);
	fSelectionState = state;
}

//-------------------------------------------------------------------------
//	 FW_CFrame::SelectionStateChanged
//-------------------------------------------------------------------------

void FW_CFrame::SelectionStateChanged(SelectionState oldState, SelectionState newState)
{
FW_UNUSED(oldState);
FW_UNUSED(newState);
}

//-------------------------------------------------------------------------
//	 FW_CFrame::WritePartInfo
//-------------------------------------------------------------------------
//	Save in the part info data specific to the frame

void FW_CFrame::WritePartInfo(XMPStorageUnitView* storageUnitView)
{
	FW_Boolean needsActivating = fNeedsActivating || IsActive();
	storageUnitView->SetValue(sizeof(FW_Boolean), (XMPValue)&needsActivating);
}

//-------------------------------------------------------------------------
//	 FW_CFrame::ReadPartInfo
//-------------------------------------------------------------------------
//	Read data specific to the frame

void FW_CFrame::ReadPartInfo(XMPStorageUnitView* storageUnitView)
{
	storageUnitView->GetValue(sizeof(FW_Boolean), (XMPValue)&fNeedsActivating);
}

//==============================================================================
//	 class FW_CFrameFacetIterator
//==============================================================================

//-------------------------------------------------------------------------
//	 FW_CFrameFacetIterator::FW_CFrameFacetIterator
//-------------------------------------------------------------------------

FW_CFrameFacetIterator::FW_CFrameFacetIterator(FW_CFrame* frame):
	fIte(frame->GetXMPFrame())
{
	fCurrentFacet = FW_CFacet::XMPtoFWFacet(fIte.First());
}

//-------------------------------------------------------------------------
//	 FW_CFrameFacetIterator::~FW_CFrameFacetIterator
//-------------------------------------------------------------------------

FW_CFrameFacetIterator::~FW_CFrameFacetIterator()
{
}

//-------------------------------------------------------------------------
//	 FW_CFrameFacetIterator::CurrentItem
//-------------------------------------------------------------------------

FW_CFacet* FW_CFrameFacetIterator::CurrentItem()
{
	return fCurrentFacet;
}

//-------------------------------------------------------------------------
//	 FW_CFrameFacetIterator::Next
//-------------------------------------------------------------------------

FW_Boolean FW_CFrameFacetIterator::Next()
{
	fCurrentFacet = FW_CFacet::XMPtoFWFacet(fIte.Next());
	return !this->IsDone();
}
