//========================================================================================
//
//	File:				FWFrming.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWPROXY_H
#include "FWProxy.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWPARTING_H
#include "FWParting.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef BCSTOREU_H
#include "BCStoreU.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#pragma segment framework

//==============================================================================
//	 class FW_CEmbeddingFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::FW_CEmbeddingFrame
//------------------------------------------------------------------------------

FW_CEmbeddingFrame::FW_CEmbeddingFrame()
{
	fProxyFrameList = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::InitEmbeddingFrame
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::InitEmbeddingFrame(XMPFrame* xmpFrame, 
											FW_CPart* part)
{
	InitFrame(xmpFrame, part);
	fProxyFrameList = new BC_TDynamicCollection<FW_CProxyFrame*, BC_CUnmanaged>(kChunkSize);
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::~FW_CEmbeddingFrame
//------------------------------------------------------------------------------

FW_CEmbeddingFrame::~FW_CEmbeddingFrame()
{
	if (fProxyFrameList)
	{
		BC_TCollectionActiveIterator<FW_CProxyFrame*> iter(*fProxyFrameList);
		
		FW_CProxyFrame* proxyFrame;
		while (!iter.IsDone())
		{
			proxyFrame = *iter.CurrentItem();
			delete proxyFrame;
			iter.Next();
		}
		
		delete fProxyFrameList;
		fProxyFrameList = NULL;
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::CreateEmbeddedFrame
//------------------------------------------------------------------------------

FW_CProxyFrame* FW_CEmbeddingFrame::CreateEmbeddedFrame(XMPPart *embeddedXMPPart,
												 FW_CProxyRun* proxyRun,
												 XMPShape* frameShape,												
												 XMPTypeToken viewType,
												 XMPTypeToken presentation,
												 XMPID frameGroupID,
												 FW_Boolean isOverlaid)
{
	XMPStorageUnit* su = GetPart()->GetStorageUnit();
		
	// ----- create the new frame
	XMPFrame* embeddedXMPFrame = su->GetDraft()->CreateFrame(GetXMPFrame(), frameShape, embeddedXMPPart, 
															viewType, presentation, 
															FALSE, isOverlaid);
	
	embeddedXMPFrame->SetFrameGroup(frameGroupID);
	
	// ----- Create the proxy Frame
	FW_CProxyFrame *proxyFrame = proxyRun->NewProxyFrame(embeddedXMPFrame);
	this->AddProxyFrame(proxyFrame);
	
	embeddedXMPFrame->Release();	// Release one of the references since CreateFrame gives
									// you a frame with a reference count of one instead of zero.
	
	return proxyFrame;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::AddToDisplayList
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::AddedToDisplayList()
{
	// ----- If it is the first display frame look if we don't have orphan embeddedframes
	if (GetPart()->CountDisplayFrame() == 1)
	{
		((FW_CEmbeddingPart*)GetPart())->SetEmbeddedFramesContainingFrame(this);
	}
	
	// ----- Call inherited -----
	FW_CFrame::AddedToDisplayList();
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::AttachSourceFrame
//------------------------------------------------------------------------------
//	FW_CEmbeddingFrame::AttachSourceFrame won't be called if sourceFrame == NULL

void FW_CEmbeddingFrame::AttachSourceFrame(FW_CEmbeddingFrame* sourceFrame)
{
	// ----- Duplicate all the embedded frames of sourceFrame -----
	// ----- CreateEmbeddedFrame doesn't create facets, wait for FacetAdded to duplicate my facet -----
	BC_TCollectionActiveIterator<FW_CProxyFrame*> iter(*(sourceFrame->GetProxyFrameList()));
	FW_CProxyFrame* proxyFrame;
	while (!iter.IsDone())
	{
		proxyFrame = *iter.CurrentItem();
		XMPShape* frameShape = ::NewXMPShape();
		proxyFrame->GetFrameShape(frameShape);
		
		XMPPart* embeddedXMPPart = proxyFrame->GetXMPPart();
		FW_CProxyRun* proxyRun = proxyFrame->GetProxyRun();
		
		// ----- Create the embedded Frame (proxyFrame) -----
		FW_CProxyFrame* newProxyFrame = CreateEmbeddedFrame(embeddedXMPPart,
														proxyRun,
														frameShape,
														proxyFrame->GetViewType(),
														proxyFrame->GetPresentation(),
														proxyFrame->GetFrameGroup(),
														proxyFrame->IsOverlaid());

		// ----- Add the embedded frame to the embedded part -----
		embeddedXMPPart->AttachSourceFrame(newProxyFrame->GetXMPFrame(), proxyFrame->GetXMPFrame());
		iter.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::RemoveEmbeddedFrame
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::RemoveEmbeddedFrame(FW_CProxyFrame* proxyFrame)
{
	XMPFrame *xmpEmbeddedFrame = proxyFrame->GetXMPFrame();
	
	// ----- Increment the refcount so at the end we can call Remove -----
	xmpEmbeddedFrame->IncrementRefCount();
	
	// ----- Delete its facet -----
	FW_CFrameFacetIterator ite(this);
	BC_TDynamicCollection<XMPFacet *, BC_CUnmanaged> tempCollect(kChunkSize);
	FW_CFacet* facet;
	while (!ite.IsDone())
	{
		facet = ite.CurrentItem();
		XMPFacetIterator ite2(facet->GetXMPFacet(), kXMPChildrenOnly, kXMPFrontToBack);
		for (XMPFacet *xmpFacet = ite2.First(); ite2.IsNotComplete(); xmpFacet = ite2.Next())
		{
			if (xmpFacet->GetFrame() == xmpEmbeddedFrame)
			{
				tempCollect.Append(xmpFacet);
			}
		}
		BC_TCollectionActiveIterator<XMPFacet *> ite3(tempCollect);
		XMPFacet* xmpFacet2;
		while (!ite3.IsDone())
		{
			xmpFacet2 = *ite3.CurrentItem();
			facet->GetXMPFacet()->RemoveFacet(xmpFacet2);
			delete xmpFacet2;
			ite3.Next();
		}
		
		tempCollect.Clear();
		ite.Next();
	}
		
	// ----- Remove it from the list -----
	this->RemoveProxyFrame(proxyFrame);
	delete proxyFrame;
		
	// ----- Tell the frame to remove itself -----
	// ----- Note: The frame will be automatically -----
	// ----- removed from the draft if its refcount is 1 -----
	xmpEmbeddedFrame->Remove();
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::FrameShapeChanged
//------------------------------------------------------------------------------
//	When overridden, call inherited::FrameShapeChanged

void FW_CEmbeddingFrame::FrameShapeChanged()
{
	FW_CFrame::FrameShapeChanged();
	ClipEmbeddedFrames();
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::UsedShapeChanged
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::UsedShapeChanged(FW_CProxyRun* proxyRun, XMPFrame* embeddedXMPFrame)
{
FW_UNUSED(proxyRun);
FW_UNUSED(embeddedXMPFrame);

	ClipEmbeddedFrames();
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::XMPtoProxyFrame
//------------------------------------------------------------------------------

FW_CProxyFrame* FW_CEmbeddingFrame::XMPtoProxyFrame(XMPFrame* xmpFrame) const
{
	BC_TCollectionActiveIterator<FW_CProxyFrame*> iter(*fProxyFrameList);
	FW_CProxyFrame* proxy;
	while (!iter.IsDone())
	{
		proxy = *iter.CurrentItem();
		if (proxy->GetXMPFrame() == xmpFrame)
			return proxy;
		iter.Next();
	}
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::ClipEmbeddedFrames
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::ClipEmbeddedFrames()
{
	FW_CFrameFacetIterator iter(this);
	while (!iter.IsDone())
	{
		iter.CurrentItem()->ClipEmbeddedFacets();
		iter.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::CloseDisplayFrame
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::CloseDisplayFrame()
{
	// ----- Close all my embedded frames -----
	FW_CProxyFrame* proxyFrame;
	while (!fProxyFrameList->IsEmpty())
	{
		proxyFrame = fProxyFrameList->First();
		proxyFrame->GetXMPFrame()->Close();
		this->RemoveProxyFrame(proxyFrame);
		delete proxyFrame;
	}

	// ----- Call inherited -----
	FW_CFrame::CloseDisplayFrame();
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::RemoveDisplayFrame
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::RemoveDisplayFrame()
{
	// ----- Remove all my embedded frames -----
	FW_CProxyFrame* proxyFrame;
	while (!fProxyFrameList->IsEmpty())
	{
		proxyFrame = fProxyFrameList->First();
		XMPFrame *embeddedXMPFrame = proxyFrame->GetXMPFrame();
		
		// ----- Increment the refcount to be able to call XMPFrame->Remove() -----
		embeddedXMPFrame->IncrementRefCount();
		
		// ----- Remove it from the list of proxyFrame -----
		this->RemoveProxyFrame(proxyFrame);
		delete proxyFrame;

		// ----- Remove it from the draft -----
		embeddedXMPFrame->Remove();
	}

	// ----- Call inherited -----
	FW_CFrame::RemoveDisplayFrame();
}


//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::FacetAdded
//------------------------------------------------------------------------------
//	When overridden call inherited::FacetAdded FIRST

void FW_CEmbeddingFrame::FacetAdded(FW_CFacet* facet)
{
	// ----- Call inherited first
	FW_CFrame::FacetAdded(facet);
	
	BC_TCollectionActiveIterator<FW_CProxyRun*> ite(*((FW_CEmbeddingPart*)GetPart())->GetProxyRunList());
	FW_CProxyRun* proxyRun;
	while (!ite.IsDone())
	{
		proxyRun = *ite.CurrentItem();
		BC_TCollectionActiveIterator<FW_CProxyFrame*> ite2(*proxyRun->GetProxyFrameList());
		FW_CProxyFrame* proxyFrame;
		while (!ite2.IsDone())
		{
			proxyFrame = *ite2.CurrentItem();
			if (proxyFrame->GetContainingFrame() == this)
			{
				XMPShape* clipShape = ::NewXMPShape();
				proxyFrame->GetFrameShape(clipShape);
				XMPTransform* externalTransform = ::NewXMPTransform();
				proxyRun->GetFacetExternalTransform(proxyFrame->GetXMPFrame(), externalTransform);
				facet->CreateEmbeddedFacet(proxyFrame,
										   clipShape, externalTransform,
											NULL, kXMPFrameBehind);
			}
			ite2.Next();
		}
		ite.Next();
	}
		
	// ----- Recalculate the clip of my embedded facets -----
	facet->ClipEmbeddedFacets();
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::FacetRemoved
//------------------------------------------------------------------------------
//	When overridden call inherited::FacetRemoved LAST

void FW_CEmbeddingFrame::FacetRemoved(FW_CFacet* facet)
{
	// ----- Remove the child facets of my facet
	facet->RemoveEmbeddedFacets();
	
	// ----- Call inherited
	FW_CFrame::FacetRemoved(facet);
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::DoMenuEvent
//------------------------------------------------------------------------------

FW_Boolean FW_CEmbeddingFrame::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	if (commandID == kXMPCommandGetPartInfo)
	{
		ShowPartInfo();
		return TRUE;
	}
			
	return FW_CFrame::DoMenuEvent(menuBar, commandID);
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::ShowPartInfo
//------------------------------------------------------------------------------

void FW_CEmbeddingFrame::ShowPartInfo()
{
	BC_TCollectionActiveIterator<FW_CProxyFrame*> iter(*fProxyFrameList);
	FW_CProxyFrame* proxyFrame;
	while (!iter.IsDone())
	{
		proxyFrame = *iter.CurrentItem();
		if (proxyFrame->GetProxyRun()->IsSelected())
			::ShowPartFrameInfo(proxyFrame->GetXMPFrame());
		iter.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::CountEmbeddedFrames
//------------------------------------------------------------------------------
unsigned long FW_CEmbeddingFrame::CountEmbeddedFrames() const
{
	return fProxyFrameList->Length();
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::AddProxyFrame
//------------------------------------------------------------------------------
void FW_CEmbeddingFrame::AddProxyFrame(FW_CProxyFrame* proxyFrame)
{
	fProxyFrameList->Append(proxyFrame);
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingFrame::RemoveProxyFrame
//------------------------------------------------------------------------------
void FW_CEmbeddingFrame::RemoveProxyFrame(FW_CProxyFrame* proxyFrame)
{
	fProxyFrameList->Remove(fProxyFrameList->Location(proxyFrame));
}

