//========================================================================================
//
//	File:				FWPart.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWWINDOW_H
#include "FWWindow.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef FWPRISTR_H
#include "FWPriStr.h"
#endif

#ifndef FWSTRING_H
#include "FWString.h"
#endif

// ----- Graphic Includes -----

#ifndef FWGRUTIL_H
#include "FWGrUtil.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _CLIPBD_
#include <Clipbd.h>
#endif

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _STORAGE_
#include <Storage.h>
#endif

#ifndef _DOCUMENT_
#include <Document.h>
#endif

#ifndef _SUCURSOR_
#include <SUCursor.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _WINSTAT_
#include <WinStat.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _XMPTYPES_
#include <XMPTypes.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#ifndef _FOCI_
#include <Foci.h>
#endif

#ifndef _LINKSRC_
#include <LinkSrc.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__TOOLUTILS__)
#include <ToolUtils.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <Quickdraw.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__EVENTS__)
#include <Events.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__STDLIB__)
#include <StdLib.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(mathRoutinesIncludes)
#include <math routines.h>
#endif

#pragma segment framework

//==============================================================================
// Constants
//==============================================================================

#define kOrderedList "Apple:OpenDoc:Framework:kOrderedList"
#define kUserPropertiesAdded "Apple:OpenDoc:Framework:kUserPropertiesAdded"

//==============================================================================
//	 class FW_CPart
//==============================================================================

FW_CPart::FW_CPart()
{
	fPartMenuBar = NULL;
	fMenuBar = NULL;
	fActiveFrame = NULL;
	fDisplayFrames = NULL;
	fWindowList = NULL;
	fSelection = NULL;
	
	// ----- tokenize 
	fSelectionFocusToken = 0;
	fMenuFocusToken = 0;
	fKeyFocusToken = 0;
	
	// ----- linking 
	fPendingPublishLink = NULL;
	fPublishLinkList = NULL;
	fSubscribLinkList = NULL;
	
	// ----- User properties added -----
	fUserPropertiesAdded = FALSE;
}

FW_CPart::~FW_CPart()
{	
	if (fPartMenuBar)
		fPartMenuBar->Release();
	
	delete fDisplayFrames;
	fDisplayFrames = NULL;

	delete fWindowList;
	fWindowList = NULL;
}

void FW_CPart::Release()
{	
	XMPRefCntObject::Release();

	if (this->GetRefCount() == 0)
	{
		// ----- Display Frames collection -----
		BC_TCollectionActiveIterator<FW_CFrame*> displayIter(*fDisplayFrames);
		while (!displayIter.IsDone())
		{
			FW_CFrame* displayFrame = *displayIter.CurrentItem();
			delete displayFrame;
			displayIter.Next();
		}
		fDisplayFrames->Clear();
				
		// ----- Window collection -----
		BC_TCollectionActiveIterator<FW_CWindow*> windowIter(*fWindowList);
		while (!windowIter.IsDone())
		{
			FW_CWindow* window = *windowIter.CurrentItem();
			delete window;
			windowIter.Next();
		}
		fWindowList->Clear();
	
		// ----- Publish link collection -----
		if (fPublishLinkList)
		{
			BC_TCollectionActiveIterator<FW_CPublishLink*> publishLinkIter(*fPublishLinkList);
			while (!publishLinkIter.IsDone())
			{
				FW_CPublishLink* publishLink = *publishLinkIter.CurrentItem();
				delete publishLink;
				publishLinkIter.Next();
			}
			delete fPublishLinkList;
			fPublishLinkList = NULL;
		}
		
		// ----- Subscrib link collection -----
		if (fSubscribLinkList)
		{
			BC_TCollectionActiveIterator<FW_CSubscribLink*> subscribLinkIter(*fSubscribLinkList);
			while (!subscribLinkIter.IsDone())
			{
				FW_CSubscribLink* subscribLink = *subscribLinkIter.CurrentItem();
				delete subscribLink;
				subscribLinkIter.Next();
			}
			delete fPublishLinkList;
			fPublishLinkList = NULL;
		}
	}
}

void FW_CPart::ReleaseAll()
{	
	XMPPersistentObject::ReleaseAll();
}

//-------------------------------------------------------------------------
//	 FW_CPart::Initialize
//-------------------------------------------------------------------------

void FW_CPart::Initialize()
{
	// ----- Initialize Graphic -----
	InitGraphic();

	// ----- Get the session -----
	XMPSession* session = GetSession();
	
	// ----- tokenize -----
	fSelectionFocusToken = session->Tokenize(kXMPSelectionFocus);
	fMenuFocusToken = session->Tokenize(kXMPMenuFocus);
	fKeyFocusToken = session->Tokenize(kXMPKeyFocus);
	
	// ----- menu bar -----
	fPartMenuBar = session->GetWindowState()->CopyBaseMenuBar();
	fMenuBar = fPartMenuBar;
	
	// ----- display collections -----
	fDisplayFrames = new BC_TDynamicCollection<FW_CFrame*, BC_CUnmanaged>(kDisplayFramesChunkSize);

	// ----- Window List -----
	fWindowList = new BC_TDynamicCollection<FW_CWindow*, BC_CUnmanaged>(kWindowsChunkSize);
	
	// ----- Selection -----
	fSelection = NewSelection();
	
}

//-------------------------------------------------------------------------
//	 FW_CPart::InitPart
//-------------------------------------------------------------------------

void FW_CPart::InitPart(XMPStorageUnit* storageUnit)
{
	if (fInitialized)
		return;
	
	XMPPersistentObject::InitPersistentObject(storageUnit);

	// Common initialization
	this->Initialize();
	
	this->CheckAndAddProperties(storageUnit);
}

//-------------------------------------------------------------------------
//	 FW_CPart::AddProperties
//-------------------------------------------------------------------------

void FW_CPart::AddProperties(XMPStorageUnit* storageUnit)
{
FW_UNUSED(storageUnit);
}

//-------------------------------------------------------------------------
//	 FW_CPart::CheckAndAddProperties
//-------------------------------------------------------------------------

void FW_CPart::CheckAndAddProperties(XMPStorageUnit* storageUnit)
{
	// ----- Check and add content property -----
	if (!storageUnit->Exists(kXMPPropContents, this->GetContentPropertyValueType(), 0))
		storageUnit->AddProperty(kXMPPropContents)->AddValue(this->GetContentPropertyValueType());

	// ----- Check and add Display frames property -----
	if (!storageUnit->Exists(kXMPPropDisplayFrames, kOrderedList, 0))
	{
		storageUnit->AddProperty(kXMPPropDisplayFrames)->AddValue(kOrderedList);
		unsigned long count = 0;
		storageUnit->SetValue(sizeof(count), (XMPValue)&count);
	}
	
	// ----- Check and add User properties -----
	if (!storageUnit->Exists(kUserPropertiesAdded, kXMPBoolean, 0))
	{
		this->AddProperties(storageUnit);
		storageUnit->AddProperty(kUserPropertiesAdded)->AddValue(kXMPBoolean);
		fUserPropertiesAdded = TRUE;
	}

}

//-------------------------------------------------------------------------
//	 FW_CPart::InitPartFromStorage
//-------------------------------------------------------------------------

void FW_CPart::InitPartFromStorage(XMPStorageUnit* storageUnit)
{
	if (fInitialized)
		return;
	
	XMPPersistentObject::InitPersistentObjectFromStorage(storageUnit);

	// ----- Common initialization -----
	this->Initialize();
	
	// ----- Check/add our properties -----
	CheckAndAddProperties(storageUnit);
	
	// ----- Read Framework Specific data -----
	InternalizeDisplayFrames(storageUnit);
	
	// ----- Read Part Specific data -----
	InternalizeContent(storageUnit);	
}

//-------------------------------------------------------------------------
//	 FW_CPart::Externalize
//-------------------------------------------------------------------------

void FW_CPart::Externalize()
{
	XMPPersistentObject::Externalize();

	XMPStorageUnit* partStorageUnit = this->GetStorageUnit();
	
	// ----- Externalize framework specific data -----
	ExternalizeDisplayFrames(partStorageUnit);
	
	// ----- Externalize part specific data -----
	ExternalizeContent(partStorageUnit);
}

//-------------------------------------------------------------------------
//	 FW_CPart::InternalizeContent
//-------------------------------------------------------------------------

void FW_CPart::InternalizeContent(XMPStorageUnit* storageUnit)
{
FW_UNUSED(storageUnit);
}

//-------------------------------------------------------------------------
//	 FW_CPart::ExternalizeContent
//-------------------------------------------------------------------------

void FW_CPart::ExternalizeContent(XMPStorageUnit* storageUnit)
{
FW_UNUSED(storageUnit);
}

//-------------------------------------------------------------------------
//	 FW_CPart::ExternalizeDisplayFrames
//-------------------------------------------------------------------------

void FW_CPart::ExternalizeDisplayFrames(XMPStorageUnit* storageUnit)
{
	// ----- Write list of displayFrames -----
	storageUnit->Focus(kXMPPropDisplayFrames, kXMPPosUndefined, kOrderedList, 0, kXMPPosUndefined);
	storageUnit->Remove();
	storageUnit->AddValue(kOrderedList);
		
	unsigned long count = fDisplayFrames->Length();
	storageUnit->SetValue(sizeof(count), (XMPValue)&count);
	
	FW_CPartFrameIterator ite1(this);
	FW_CFrame* frame;
	while (!ite1.IsDone())
	{
		frame = ite1.CurrentItem();
		XMPStorageUnitRef aSURef = storageUnit->GetWeakStorageUnitRef(frame->GetXMPStorageUnit());
		storageUnit->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		ite1.Next();
	}
}

//-------------------------------------------------------------------------
//	 FW_CPart::InternalizeDisplayFrames
//-------------------------------------------------------------------------

void FW_CPart::InternalizeDisplayFrames(XMPStorageUnit* storageUnit)
{
 	XMPStorageUnitRef aSURef;
	XMPFrame* xmpFrame;
	FW_VOLATILE(xmpFrame);
	
	// ----- Read the Display frames -----
	// We don't have to test because already done in CheckAndAddProperties -----
 	storageUnit->Focus(kXMPPropDisplayFrames, kXMPPosUndefined, kOrderedList, 0, kXMPPosUndefined);
	unsigned long count;
	storageUnit->GetValue(sizeof(count), (XMPValue)&count);
	for (unsigned long i = 0; i<count; i++)
	{
		storageUnit->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		
		TRY
		
			xmpFrame = storageUnit->GetDraft()->GetFrame(storageUnit->GetIDFromStorageUnitRef(aSURef));
		
			FW_CFrame* frame = NewFrame(xmpFrame, xmpFrame->GetPresentation());
			xmpFrame->Release();			// This is needed to balance an additional reference created when GetFrame
											// was called.  We keep our own reference in FW_CBaseFrame
	
			fDisplayFrames->Append(frame);
			
		CATCH_ALL
			
			xmpFrame = NULL;
			
		ENDTRY
	}
}

//-------------------------------------------------------------------------
//	 FW_CPart::CloneInto
//-------------------------------------------------------------------------

void FW_CPart::CloneInto(XMPDraftKey key, XMPStorageUnit* storageUnit, XMPStorageUnit* initiatingFrameSU)
{
	XMPStorageUnit*	su = this->GetStorageUnit();
	
	XMPDraft* draft = su->GetDraft();
	FW_VOLATILE(draft);

	this->Externalize();
	
#ifndef ASLMBUILD
	storageUnit->AddProperty(kXMPPropPart)->AddValue(kXMPISOStr);
	XMPISOStr partName = GetContentPropertyValueType();
	unsigned long size = FW_PrimitiveStringLength(partName) + 1;
	storageUnit->SetValue(size, partName);
#endif

	TRY
	
		su->CloneInto(key, storageUnit, initiatingFrameSU);
		
	CATCH_ALL
	
		draft->AbortClone(key);
		
	ENDTRY
}

//-------------------------------------------------------------------------
// From DragAndDrop protocol
//-------------------------------------------------------------------------

// --- FulfillPromise ---

void FW_CPart::FulfillPromise(XMPStorageUnitView *promiseSUView)
{
FW_UNUSED(promiseSUView);
}

// --- DropCompleted ---

void FW_CPart::DropCompleted(XMPPart* destPart,
							   XMPDropResult dropResult)
{
FW_UNUSED(destPart);
FW_UNUSED(dropResult);
}

// --- DragEnter ---

void FW_CPart::DragEnter(XMPDragItemIterator* dragInfo,  
						XMPFacet* xmpFacet,  
						XMPPoint where)
{
	FW_CFacet *facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	FW_CPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	facet->DragEnter(dragInfo, windowPoint);
}

// --- DragWithin ---

void FW_CPart::DragWithin(XMPDragItemIterator* dragInfo,
							XMPFacet* xmpFacet,
							XMPPoint where)
{
	FW_CFacet *facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	FW_CPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	facet->DragWithin(dragInfo, windowPoint);
}

// --- DragLeave ---

void FW_CPart::DragLeave(XMPFacet* xmpFacet,
						   XMPPoint where)
{
	FW_CFacet *facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	FW_CPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	facet->DragLeave(windowPoint);
}

// --- Drop ---

XMPDropResult FW_CPart::Drop(XMPDragItemIterator* dropInfo,
							   XMPFacet* xmpFacet,
							   XMPPoint where)
{
	FW_CFacet *facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	
	// ----- 'where' is in frame coordinate. Put it back in window coordinate
	FW_CPoint windowPoint = xmpFacet->GetWindowFrameTransform()->TransformPoint(where);
	
	return facet->Drop(dropInfo, windowPoint);
}

//-------------------------------------------------------------------------
// From Embedding protocol
//-------------------------------------------------------------------------

// --- ContainingPartPropertiesChanged ---

void FW_CPart::ContainingPartPropertiesChanged(XMPFrame* displayFrame,
												 XMPStorageUnit* propertyUnit)
{
FW_UNUSED(displayFrame);
FW_UNUSED(propertyUnit);
}

	// ********* for containing parts *********

// --- GetContainingPartProperties ---

XMPStorageUnit* FW_CPart::GetContainingPartProperties(XMPFrame* displayFrame)
{
FW_UNUSED(displayFrame);
	THROW_IF_ERROR(kXMPErrCannotEmbed);
	return nil;
}

// --- CreateEmbeddedFramesIterator ---

XMPEmbeddedFramesIterator* FW_CPart::CreateEmbeddedFramesIterator(XMPFrame* displayFrame)
{
FW_UNUSED(displayFrame);

	return NULL;
}

//-------------------------------------------------------------------------
// from Facet protocol:
//-------------------------------------------------------------------------

// --- FacetAdded ---

void FW_CPart::FacetAdded(XMPFacet* xmpFacet)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFacet->GetFrame());
	FW_CFacet* facet = frame->NewFacet(xmpFacet);	
	frame->FacetAdded(facet);
}

// --- FacetRemoved ---

void FW_CPart::FacetRemoved(XMPFacet* xmpFacet)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFacet->GetFrame());
	FW_CFacet* facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	frame->FacetRemoved(facet);
	delete facet;
}

// --- CanvasChanged ---

void FW_CPart::CanvasChanged(XMPFacet* xmpFacet)
{
FW_UNUSED(xmpFacet);
}

// --- ClipShapeChanged ---

void FW_CPart::GeometryChanged(XMPFacet* xmpFacet)
{
	FW_CFacet* facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	facet->GeometryChanged();
}

//-------------------------------------------------------------------------
// from Frame protocol:
//-------------------------------------------------------------------------

// --- AddDisplayFrame ---

void FW_CPart::AddDisplayFrame(XMPFrame* xmpFrame)
{
	if (xmpFrame->GetPresentation() == kNoPresentation)
		xmpFrame->SetPresentation(GetDefaultPresentation());
	
	FW_CFrame* frame = NewFrame(xmpFrame, xmpFrame->GetPresentation());
	fDisplayFrames->Append(frame);
	
	frame->AddedToDisplayList();
}

// --- AttachSourceFrame ---

void FW_CPart::AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame)
{
FW_UNUSED(xmpFrame);
FW_UNUSED(xmpSourceFrame);
}

// --- RemoveDisplayFrame ---

void FW_CPart::RemoveDisplayFrame(XMPFrame* xmpFrame)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	
	// ----- If the frame we remove is the activeFrame set it to the
	// ----- next activable frame
	if (frame == this->GetActiveFrame())
	{
		FW_CPartFrameIterator ite(this);
		FW_CFrame* frame;
		while (!ite.IsDone())
		{
			frame = ite.CurrentItem();
			if (frame->CanBeActiveFrame())
			{
				this->SetActiveFrame(frame);
				break;
			}
			ite.Next();
		}
	}
	
	fDisplayFrames->Remove(fDisplayFrames->Location(frame));	
	frame->RemoveDisplayFrame();
	delete frame;
}

// --- CloseDisplayFrame ---

void FW_CPart::CloseDisplayFrame(XMPFrame* xmpFrame)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	fDisplayFrames->Remove(fDisplayFrames->Location(frame));	
	frame->CloseDisplayFrame();
	delete frame;
}

// --- RevealFrame ---

void FW_CPart::RevealFrame(XMPFrame* embeddedFrame, XMPShape* revealShape)
{
FW_UNUSED(embeddedFrame);
FW_UNUSED(revealShape);
}

// --- FrameShapeChanged ---

void FW_CPart::FrameShapeChanged(XMPFrame* xmpFrame)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	frame->FrameShapeChanged();
}

// --- WritePartInfo ---

void FW_CPart::WritePartInfo(void* partInfo,
							XMPStorageUnitView* storageUnitView)
{
	// ----- The part info is a FW_CFrame -----
	FW_CFrame* frame = (FW_CFrame*)partInfo;
	frame->WritePartInfo(storageUnitView);	
}

// --- ReadPartInfo ---

void* FW_CPart::ReadPartInfo(XMPFrame* xmpFrame, XMPStorageUnitView* storageUnitView)
{	
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	frame->ReadPartInfo(storageUnitView);
	
	return (void*)frame;
}

// --- Open ---

XMPID FW_CPart::Open(XMPFrame* xmpFrame)
{
FW_UNUSED(xmpFrame);

	XMPWindow* window;
	
	XMPISOStr title = "Untitled-1";
	if (xmpFrame)
	{
		FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
		window = frame->CreateChildWindow(title);
	}
	else
	{
		window = CreateRootWindow(title);
	}
	
	XMPID windowID = 0;
	
	if (window)
	{
		// ATTENTION: the order is very important [HLX]
		window->Open();
		window->Show();
		window->Select();
	
		windowID =  window->GetID();
	}
	
	return windowID;
}

	// ********* for containing parts *********

// --- RequestEmbeddedFrame ---

XMPFrame* FW_CPart::RequestEmbeddedFrame(XMPFrame* containingFrame,
										  XMPFrame* baseFrame,
										  XMPShape* frameShape,
										  XMPPart* embedPart,
										  XMPTypeToken viewType,
										  XMPTypeToken presentation,
										  FW_Boolean isOverlaid)
{
FW_UNUSED(containingFrame);
FW_UNUSED(baseFrame);
FW_UNUSED(frameShape);
FW_UNUSED(embedPart);
FW_UNUSED(viewType);
FW_UNUSED(presentation);
FW_UNUSED(isOverlaid);

	THROW(kXMPErrCannotEmbed);
	return NULL;
}

// --- RemoveEmbeddedFrame ---

void FW_CPart::RemoveEmbeddedFrame(XMPFrame* embeddedXMPFrame)
{
FW_UNUSED(embeddedXMPFrame);

	THROW(kXMPErrCannotEmbed);
}

// --- RequestFrameShape ---

XMPShape* FW_CPart::RequestFrameShape(XMPFrame* embeddedXMPFrame, XMPShape* frameShape)
{
FW_UNUSED(embeddedXMPFrame);
	return frameShape;
}

// --- UsedShapeChanged ---

void FW_CPart::UsedShapeChanged(XMPFrame* xmpEmbeddedFrame)
{
FW_UNUSED(xmpEmbeddedFrame);
	THROW_IF_ERROR(kXMPErrCannotEmbed);
}

// --- AdjustBorderShape ---

XMPShape* FW_CPart::AdjustBorderShape(XMPFrame* xmpFrame, XMPShape* shape)
{
FW_UNUSED(xmpFrame);

	return shape;
}

// --- EmbeddedFrameSpec ---

void FW_CPart::EmbeddedFrameSpec(XMPFrame* embeddedFrame, XMPObjectSpec* spec)
{
FW_UNUSED(embeddedFrame);
FW_UNUSED(spec);
}

// --- ViewTypeChanged ---

void FW_CPart::ViewTypeChanged(XMPFrame* xmpFrame)
{
FW_UNUSED(xmpFrame);
}
	
// --- PresentationChanged ---

void FW_CPart::PresentationChanged(XMPFrame* xmpFrame)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	frame->PresentationChanged();
}

// --- SequenceChanged ---

void FW_CPart::SequenceChanged(XMPFrame* xmpFrame)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	frame->SequenceChanged();
}

//-------------------------------------------------------------------------
// From Binding protocol
//-------------------------------------------------------------------------

void FW_CPart::ExternalizeKinds(XMPTypeList* kindList)
{
FW_UNUSED(kindList);
}

void FW_CPart::ChangeKind(XMPType kind)
{
FW_UNUSED(kind);
}

//-------------------------------------------------------------------------
// From Imaging protocol
//-------------------------------------------------------------------------

// --- CanvasUpdate ---

void FW_CPart::CanvasUpdated(XMPCanvas* canvas)
{
FW_UNUSED(canvas);

	DebugStr("\pCPart::CanvasUpdate(XMPCanvas* canvas) called");
}

// --- Draw ---

void FW_CPart::Draw(XMPFacet* xmpFacet, XMPShape* invalidShape)
{
	FW_CFacet* facet = FW_CFacet::XMPtoFWFacet(xmpFacet);

	XMPShape* testShape = NULL;
	if (invalidShape)
	{
		testShape = ::NewXMPShape(invalidShape);
		testShape->Intersect(xmpFacet->GetAggregateClipShape());
	}	

	FW_CGraphicContext gc(xmpFacet, testShape);
	facet->Draw(&gc);
	
	delete testShape;
}

// --- HighlightChanged ---

void FW_CPart::HighlightChanged(XMPFacet* facet)
{
FW_UNUSED(facet);
}

// --- GetPrintResolution ---

unsigned long FW_CPart::GetPrintResolution(XMPFrame* xmpFrame)
{
FW_UNUSED(xmpFrame);
	return 0;
}

//==============================================================================
//	 Linking protocol
//

//------------------------------------------------------------------------------
//	 FW_CPart::CreateLink
//------------------------------------------------------------------------------

XMPLink* FW_CPart::CreateLink(void* data, unsigned long size)
{
FW_UNUSED(size);

	XMPChangeID changeID = *((XMPChangeID*)data);
	
	XMPLinkSource* xmpLinkSource = this->GetStorageUnit()->GetDraft()->CreateLinkSource(this);
	
	WASSERTM(fPendingPublishLink->GetChangeID() == changeID, "Error in CreateLink");
	
	fPendingPublishLink->SetXMPLinkSource(xmpLinkSource);
	
	AddToPublishLinkList(fPendingPublishLink);
	fPendingPublishLink->Publish();

	fPendingPublishLink = NULL;
		
	return xmpLinkSource->GetLink();
}

//------------------------------------------------------------------------------
//	 FW_CPart::LinkUpdated
//------------------------------------------------------------------------------

void FW_CPart::LinkUpdated(XMPLink* updatedLink, XMPChangeID id)
{
	FW_CSubscribLink *link = XMPtoSubscribLink(updatedLink);
	if (link)
		link->LinkUpdated(id);
}

//------------------------------------------------------------------------------
//	 FW_CPart::RevealLink
//------------------------------------------------------------------------------

void FW_CPart::RevealLink(XMPLinkSource* xmpLinkSource)
{
FW_UNUSED(xmpLinkSource);
}

//------------------------------------------------------------------------------
//	 FW_CPart::EmbeddedFrameChanged
//------------------------------------------------------------------------------

void FW_CPart::EmbeddedFrameChanged(XMPFrame* xmpFrame, XMPChangeID change)
{
FW_UNUSED(xmpFrame);
FW_UNUSED(change);
}

//------------------------------------------------------------------------------
//	 FW_CPart::LinkStatusChanged
//------------------------------------------------------------------------------

void FW_CPart::LinkStatusChanged(XMPFrame* xmpFrame)
{
FW_UNUSED(xmpFrame);
}

//------------------------------------------------------------------------------
//	 FW_CPart::AddToPublishLinkList
//------------------------------------------------------------------------------

void FW_CPart::AddToPublishLinkList(FW_CPublishLink* publishLink)
{
	if (fPublishLinkList == NULL)
	{
		fPublishLinkList = new BC_TDynamicCollection<FW_CPublishLink*, BC_CUnmanaged>(kPublishLinksChunkSize);
	}
	
	fPublishLinkList->Append(publishLink);
}

//------------------------------------------------------------------------------
//	 FW_CPart::RemoveFromPublishLinkList
//------------------------------------------------------------------------------

void FW_CPart::RemoveFromPublishLinkList(FW_CPublishLink* publishLink)
{
	fPublishLinkList->Remove(fPublishLinkList->Location(publishLink));
	
	if (fPublishLinkList->Length() == 0)
	{
		delete fPublishLinkList;
		fPublishLinkList = NULL;
	}
}

//------------------------------------------------------------------------------
//	 FW_CPart::AddToSubscribLinkList
//------------------------------------------------------------------------------

void FW_CPart::AddToSubscribLinkList(FW_CSubscribLink* subscribLink)
{
	if (fSubscribLinkList == NULL)
	{
		fSubscribLinkList = new BC_TDynamicCollection<FW_CSubscribLink*, BC_CUnmanaged>(kSubscribeLinksChunkSize);
	}
	
	fSubscribLinkList->Append(subscribLink);
}

//------------------------------------------------------------------------------
//	 FW_CPart::RemoveFromSubscribLinkList
//------------------------------------------------------------------------------

void FW_CPart::RemoveFromSubscribLinkList(FW_CSubscribLink* subscribLink)
{
	fSubscribLinkList->Remove(fSubscribLinkList->Location(subscribLink));
	
	if (fSubscribLinkList->Length() == 0)
	{
		delete fSubscribLinkList;
		fSubscribLinkList = NULL;
	}
}

//------------------------------------------------------------------------------
//	 FW_CPart::XMPtoSubscribLink
//------------------------------------------------------------------------------

FW_CSubscribLink* FW_CPart::XMPtoSubscribLink(XMPLink* xmpLink)
{
	BC_TCollectionActiveIterator<FW_CSubscribLink*> ite(*fSubscribLinkList);
	FW_CSubscribLink* link;
	while (!ite.IsDone())
	{
		link = *ite.CurrentItem();
		if (link->GetXMPLink() == xmpLink)
			return link;
		ite.Next();
	}
	return NULL;
}

//==============================================================================
//	 Memory Management protocol
//

//------------------------------------------------------------------------------
//	 FW_CPart::Purge
//------------------------------------------------------------------------------

XMPSize FW_CPart::Purge(XMPSize size)
{
	return size; // return fStorageUnit->Purge(size);
}

//==============================================================================
//	 Part Activation protocol
//

// --- BeginRelinquishFocus ---

FW_Boolean FW_CPart::BeginRelinquishFocus(XMPTypeToken focus,
											XMPFrame* ownerFrame,
											XMPFrame* proposedFrame)
{
FW_UNUSED(focus);
FW_UNUSED(ownerFrame);
FW_UNUSED(proposedFrame);

	return TRUE;
}
	
// --- CommitRelinquishFocus ---

void FW_CPart::CommitRelinquishFocus(XMPTypeToken focus,
									   XMPFrame* ownerFrame,
									   XMPFrame* proposedFrame)
{
FW_UNUSED(proposedFrame);

	FocusLost(focus, ownerFrame);
	
	if (proposedFrame == NULL || proposedFrame->GetPart() != this)
	{
		if (fSelection)
			fSelection->CloseSelection();
		HideShowFloatingWindows(FALSE);
	}
}
							
// --- AbortRelinquishFocus ---

void FW_CPart::AbortRelinquishFocus(XMPTypeToken focus,
									  XMPFrame* ownerFrame,
									  XMPFrame* proposedFrame)
{
FW_UNUSED(focus);
FW_UNUSED(ownerFrame);
FW_UNUSED(proposedFrame);
}
							  
// --- FocusAcquired ---

void FW_CPart::FocusAcquired(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(ownerFrame);
	frame->FocusStateChanged(focus, TRUE);
	HideShowFloatingWindows(TRUE);
}	

// --- FocusLost ---

void FW_CPart::FocusLost(XMPTypeToken focus, XMPFrame* ownerFrame)
{
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(ownerFrame);
	frame->FocusStateChanged(focus, FALSE);
}

//-------------------------------------------------------------------------
// From UI Events protocol
//-------------------------------------------------------------------------


// --- HandleEvent ---

FW_Boolean FW_CPart::HandleEvent(XMPEventData event,  XMPFrame* xmpFrame,  XMPFacet* xmpFacet)
{
	FW_Boolean handled = false;
	FW_CFrame* frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	FW_CFacet* facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	
	switch (event->what)
	{
		case nullEvent:
			handled = HandleIdle(frame);
			break;
		
		case mouseDown:
			handled = facet->HandleMouseDown(NULL, event);
			break;
		case mouseUp:
			handled = facet->HandleMouseUp(event);
			break;
			
		case kXMPEvtMouseDownEmbedded:
		case kXMPEvtMouseUpEmbedded:
		case kXMPEvtMouseDownBorder:
		case kXMPEvtMouseUpBorder:
			break;
		
		case keyDown:
		case autoKey:
			handled = frame->HandleKeyDown(event);
			break;
	
		case activateEvt:
			handled = HandleActivate(frame, facet, (event->modifiers & activeFlag ) != 0);
			break;
		
		case kXMPEvtWindow: 
			{
				FW_CWindow* window = XMPtoFWWindow(facet->GetXMPWindow());
				if (window)
				{
					switch (event->message)
					{
						case inGoAway:
							if (window->IsFloating()) {						
								window->HideWindow();
								handled = TRUE;
							}
							break;
						case inZoomIn:
						case inZoomOut:
							SysBeep(2); 
							handled = TRUE;
							break;
						default:
							break;
					}
				}
			}
			break;
		
		case keyUp:
			break;
						
		case osEvt:
			break;
	
		case kHighLevelEvent:
			break;

		case kXMPEvtMenu:
			handled = HandleMenuEvent(frame, (XMPCommandID)event->message);
			break;
			
		default:
			return false;
	}
	
	return handled;
}

// --- HandleEventInEmbedded ---

FW_Boolean FW_CPart::HandleEventInEmbedded(XMPEventData event,
											XMPFrame* xmpFrame,
											XMPFacet* xmpfacet,
											XMPFrame* embeddedXMPFrame,
											XMPFacet* embeddedXMPFacet)
{
FW_UNUSED(event);
FW_UNUSED(xmpFrame);
FW_UNUSED(xmpfacet);
FW_UNUSED(embeddedXMPFrame);
FW_UNUSED(embeddedXMPFacet);

	return FALSE;
}

// --- MouseEnter ---

void FW_CPart::MouseEnter(XMPFacet* xmpFacet, XMPPoint where)
{
	FW_CFacet *facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	facet->MouseEnter(where);
}

// ----- MouseWithin

void FW_CPart::MouseWithin(XMPFacet* xmpFacet, XMPPoint where)
{
	FW_CFacet *facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	facet->MouseWithin(where);
}

// --- MouseLeave ---

void FW_CPart::MouseLeave(XMPFacet* xmpFacet)
{
	FW_CFacet *facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	facet->MouseLeave();
}

// --- AdjustMenus ---

void FW_CPart::AdjustMenus(XMPFrame* xmpFrame)
{
	FW_CFrame *frame = FW_CFrame::XMPtoFWFrame(xmpFrame);
	frame->DoAdjustMenus(GetMenuBar());
}

//-------------------------------------------------------------------------
// From Undo protocol
//-------------------------------------------------------------------------

// --- UndoAction ---

void FW_CPart::UndoAction(XMPActionData actionState)
{
FW_UNUSED(actionState);
}

// --- RedoAction ---

void FW_CPart::RedoAction(XMPActionData actionState)
{
FW_UNUSED(actionState);
}

// --- DisposeAction ---

void FW_CPart::DisposeActionState(XMPActionData actionState,
									XMPDoneState doneState)
{
FW_UNUSED(actionState);
FW_UNUSED(doneState);
}


// --- WriteAction ---

void FW_CPart::WriteActionState(void* actionState,
								  XMPStorageUnitView* storageUnitView)
{
FW_UNUSED(actionState);
FW_UNUSED(storageUnitView);
}

// --- ReadAction ---

void* FW_CPart::ReadActionState(XMPStorageUnitView* storageUnitView)
{
FW_UNUSED(storageUnitView);
	return NULL;
}

//==============================================================================
// New API
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CPart::HandleMenuEvent
//------------------------------------------------------------------------------

FW_Boolean FW_CPart::HandleMenuEvent(FW_CFrame* frame, long message)
{
	XMPMenuBar* menuBar = GetMenuBar();
	
	XMPMenuID menu = HiWord(message);
	XMPMenuItemID item = LoWord(message);
	XMPCommandID commandID = menuBar->GetCommand(menu, item);

	FW_Boolean result = frame->DoMenuEvent(menuBar, commandID);
	
	::HiliteMenu(0);
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CPart::DoMenuEvent
//------------------------------------------------------------------------------

FW_Boolean FW_CPart::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
FW_UNUSED(menuBar);

	FW_Boolean result = TRUE;

	switch (commandID)
	{
		case kXMPCommandSelectAll:
			if (fSelection)
				fSelection->SelectAll();
			break;
			
		case  kXMPCommandClear:
			if (fSelection && fSelection->DoClear())
				this->Changed();
			break;
						
		case kXMPCommandPaste:
			if (fSelection && fSelection->DoPaste())
				this->Changed();
			break;
		
		case kXMPCommandPasteAs:
			if (fSelection && fSelection->DoPasteWithLink())
				this->Changed();
			break;

		default:
			result = FALSE;
	};
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CPart::HandleIdle
//------------------------------------------------------------------------------

FW_Boolean FW_CPart::HandleIdle(FW_CFrame* frame)
{
	if (frame)
		return frame->DoIdle();
	else
		return this->DoIdle();
}

//------------------------------------------------------------------------------
//	 FW_CPart::DoIdle
//------------------------------------------------------------------------------

FW_Boolean FW_CPart::DoIdle()
{
	return TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CPart::InstallMenus
//------------------------------------------------------------------------------

void FW_CPart::InstallMenus(XMPMenuBar* menuBar)
{
	if (menuBar == NULL)
		menuBar = GetPartMenuBar();
		
	if (menuBar)
		menuBar->Display();
}

//------------------------------------------------------------------------------
//	 FW_CPart::DoAdjustMenus
//------------------------------------------------------------------------------

void FW_CPart::DoAdjustMenus(XMPMenuBar* menuBar)
{
	FW_Boolean hasSelection = fSelection && !fSelection->IsEmpty();

	// ----- Edit Menu -----
	menuBar->EnableCommand(kXMPCommandClear, hasSelection);
	menuBar->EnableCommand(kXMPCommandCut, hasSelection);
	menuBar->EnableCommand(kXMPCommandCopy, hasSelection);
	menuBar->EnableCommand(kXMPCommandPaste, FALSE);
	menuBar->EnableCommand(kXMPCommandPasteAs, HasPropertyOnClipboard(kXMPPropLinkSpec, kXMPLinkSpec));
}

//------------------------------------------------------------------------------
//	 FW_CPart::HandleActivate
//------------------------------------------------------------------------------

FW_Boolean FW_CPart::HandleActivate(FW_CFrame *frame, FW_CFacet* facet, FW_Boolean state)
{
	FW_UNUSED(frame);
	
	if (state)
		facet->WindowActivated();
	else
		facet->WindowDeActivated();

	return TRUE;		// we just don't have anuthing to do
}

//------------------------------------------------------------------------------
//	 FW_CPart::AddWindow
//------------------------------------------------------------------------------

void FW_CPart::AddWindow(FW_CWindow* window)
{
	fWindowList->Append(window);
}

//------------------------------------------------------------------------------
//	 FW_CPart::RemoveWindow
//------------------------------------------------------------------------------

void FW_CPart::RemoveWindow(FW_CWindow* window)
{
	fWindowList->Remove(fWindowList->Location(window));
}

//------------------------------------------------------------------------------
//	 FW_CPart::XMPtoFWWindow
//------------------------------------------------------------------------------

FW_CWindow* FW_CPart::XMPtoFWWindow(XMPWindow *xmpWindow) const
{
	BC_TCollectionActiveIterator<FW_CWindow*> ite(*fWindowList);
	FW_CWindow* window;
	while (!ite.IsDone())
	{
		window = *ite.CurrentItem();
		if (window->GetXMPWindow() == xmpWindow)
			return window;
		ite.Next();
	}
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CPart::HideShowFloatingWindows();
//------------------------------------------------------------------------------

void FW_CPart::HideShowFloatingWindows(FW_Boolean state)
{
	BC_TCollectionActiveIterator<FW_CWindow*> ite(*fWindowList);
	FW_CWindow* window;
	while (!ite.IsDone())
	{
		window = *ite.CurrentItem();
		if (window->GetXMPWindow() && window->IsFloating() && window->IsShown())
		{
			if (state)
				window->GetXMPWindow()->Show();
			else
				window->GetXMPWindow()->Hide();
		}
		ite.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CPart::CreateRootWindow
//------------------------------------------------------------------------------

XMPWindow* FW_CPart::CreateRootWindow(XMPISOStr windowTitle)
{
	// ----- Taken from the shell. Should be replaced -----
	// Get the menu bar height, for figuring the top of our windows
	short	mbHeight	= 24; //(*(short*)MBarHeight);	//get it from lomem
	FW_CRect windRect(ff(4), ff(mbHeight+16), 
					 				 ff(XMPASLMQDGlobals.screenBits.bounds.right - 64), ff(XMPASLMQDGlobals.screenBits.bounds.bottom - 4));
	if (windRect.right-windRect.left > ff(600))
	{
		windRect.right = windRect.left + ff(600);
	}

	return CreateXMPWindow(NULL,
						 	GetSession()->Tokenize(kXMPViewAsFrame),
							kNoPresentation,
							windowTitle,
							windRect,
							TRUE,		// close box
							TRUE,		// resizable
							FALSE,		// floating
							TRUE,		// should save
							zoomDocProc);
}

//------------------------------------------------------------------------------
//	 FW_CPart::CreateXMPWindow
//------------------------------------------------------------------------------

XMPWindow* FW_CPart::CreateXMPWindow(FW_CFrame* itsFrame,
								XMPTypeToken viewType,
								XMPTypeToken presentation,
								XMPISOStr windowTitle,
								const FW_CRect& windowRect,
								FW_Boolean hasCloseBox,
								FW_Boolean isResizable,
								FW_Boolean isFloating,
								FW_Boolean shouldSave,
								short procID)
{
	FW_SPlatformRect qdWinRect = windowRect;
	
	Str255 pascalTitle;
	FW_CDynamicString title(windowTitle);
	title.ExportPascal(pascalTitle);
	
	XMPPlatformWindow platformWindow;
	
	{
		FW_CAcquireASLMResourceAccess qr;
	
		platformWindow = ::NewCWindow(NULL, 
									&qdWinRect, 
									pascalTitle, 
									FALSE, 
									procID, 
									(WindowPtr)-1L, 
									hasCloseBox, 
									NULL);
	}
	
	return GetSession()->GetWindowState()->CreateWindow(platformWindow, 
														itsFrame == NULL && !isFloating,		// root frame
														isResizable,
														isFloating,
														shouldSave,
														this,
														viewType, 
														presentation,
														itsFrame ? itsFrame->GetXMPFrame() : (XMPFrame*)NULL);
}

//------------------------------------------------------------------------------
//	 FW_CPart::InvalidateAllFrames
//------------------------------------------------------------------------------

void FW_CPart::InvalidateAllFrames(XMPTypeToken whichFrames, FW_CFacet* skipFacet, XMPShape* invalidShape)
{
	FW_CPartFrameIterator ite(this, whichFrames);
	while (!ite.IsDone())
	{
		ite.CurrentItem()->InvalidateAllFacets(skipFacet, invalidShape);
		ite.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CPart::ValidateAllFrames
//------------------------------------------------------------------------------

void FW_CPart::ValidateAllFrames(XMPTypeToken whichFrames, FW_CFacet* skipFacet, XMPShape* validShape)
{
	FW_CPartFrameIterator ite(this, whichFrames);
	while (!ite.IsDone())
	{
		ite.CurrentItem()->ValidateAllFacets(skipFacet, validShape);
		ite.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CPart::Changed
//------------------------------------------------------------------------------

void FW_CPart::Changed()
{
	this->GetStorageUnit()->GetDraft()->SetChangedFromPrev();
	
	DeletePendingPublish();
}

//------------------------------------------------------------------------------
//	 FW_CPart::GetDefaultPresentation
//------------------------------------------------------------------------------

XMPTypeToken FW_CPart::GetDefaultPresentation() const
{
	return kNoPresentation;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::SetActiveFrame
//----------------------------------------------------------------------------------------
void FW_CPart::SetActiveFrame(FW_CFrame*  activeFrame)
{
	fActiveFrame = activeFrame;
}

//----------------------------------------------------------------------------------------
//	 FW_CPart::GetActiveFacet
//----------------------------------------------------------------------------------------
FW_CFacet* FW_CPart::GetActiveFacet() const
{
	FW_CFacet* facet = NULL;
	if (fActiveFrame)
		facet = fActiveFrame->GetActiveFacet();
	return facet;
}

//------------------------------------------------------------------------------
//	 FW_CPart::NewSelection
//------------------------------------------------------------------------------

FW_CSelection* FW_CPart::NewSelection()
{
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CPart::HasPropertyOnClipboard
//------------------------------------------------------------------------------

FW_Boolean FW_CPart::HasPropertyOnClipboard(XMPPropertyName propertyName, XMPValueType type)
{
	XMPClipboard* clipboard = GetSession()->GetClipboard();
	FW_Boolean result = FALSE;

	XMPClipboardKey key;
	if (clipboard->Lock(0, &key))
	{
		TRY
			XMPStorageUnit* su = clipboard->GetContentStorageUnit(key);
			result =  su->Exists(propertyName, type, 0);
		CATCH_ALL
		ENDTRY
		clipboard->Unlock(key);
	}
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CPart::DeletePendingPublish
//------------------------------------------------------------------------------
// Remove the link spec from the clipboard

void FW_CPart::DeletePendingPublish()
{
	if (fPendingPublishLink != NULL)
	{
		XMPClipboard* clipboard = GetSession()->GetClipboard();
		
		if (fPendingPublishLink->GetChangeID() == clipboard->GetChangeID()) 
		{
			XMPClipboardKey key;
			if (clipboard->Lock(0, &key)) 
			{
				TRY
					XMPStorageUnit* clipboardSU = clipboard->GetContentStorageUnit(key);
					if (clipboardSU->Exists(kXMPPropLinkSpec, (XMPValueType) NULL, 0) == TRUE) 
					{
						clipboardSU->Focus(kXMPPropLinkSpec, kXMPPosUndefined, (XMPValueType) NULL, 0, kXMPPosUndefined);
						clipboardSU->Remove();
					}
				CATCH_ALL
				ENDTRY
				clipboard->Unlock(key);
			}
		}
		
		delete fPendingPublishLink;
		fPendingPublishLink = NULL;
	}
}

//------------------------------------------------------------------------------
//	 FW_CPart::Dragging
//------------------------------------------------------------------------------

void FW_CPart::Dragging(FW_CFacet* facet, FW_Boolean state)
{
FW_UNUSED(facet);
FW_UNUSED(state);
}

//------------------------------------------------------------------------------
//	 FW_CPart::ClonePartFrom
//------------------------------------------------------------------------------
//	A FW_CPart can't embed anything

XMPPart* FW_CPart::ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind)
{
FW_UNUSED(storageUnit);
FW_UNUSED(kind);
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CPart::IsEmbeddingPart
//------------------------------------------------------------------------------
//	This needs to be removed

FW_Boolean FW_CPart::IsEmbeddingPart() const
{
	return FALSE;
}

//==============================================================================
//	 class FW_CPartFrameIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CPartFrameIterator::FW_CPartFrameIterator
//------------------------------------------------------------------------------

FW_CPartFrameIterator::FW_CPartFrameIterator(FW_CPart* part, XMPTypeToken presentation) :
	fIterator(*part->GetDisplayFrameList()),
	fPresentation(presentation)
{
	if (!fIterator.IsDone())
	{
		fCurrentFrame = *fIterator.CurrentItem();
		FindNext();
	}
	else
		fCurrentFrame = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CPartFrameIterator::~FW_CPartFrameIterator
//------------------------------------------------------------------------------

FW_CPartFrameIterator::~FW_CPartFrameIterator()
{
}

//------------------------------------------------------------------------------
//	 FW_CPartFrameIterator::CurrentItem
//------------------------------------------------------------------------------

FW_CFrame* FW_CPartFrameIterator::CurrentItem()
{
	return fCurrentFrame;
}
	
//------------------------------------------------------------------------------
//	 FW_CPartFrameIterator::Next
//------------------------------------------------------------------------------

FW_Boolean FW_CPartFrameIterator::Next()
{
	if (fIterator.Next())
	{
		fCurrentFrame = *fIterator.CurrentItem();
		FindNext();
	}
	else
		fCurrentFrame = NULL;
	return !this->IsDone();
}

//------------------------------------------------------------------------------
//	 FW_CPartFrameIterator::IsDone
//------------------------------------------------------------------------------

FW_Boolean FW_CPartFrameIterator::IsDone()
{
	return fCurrentFrame == NULL;
}

//------------------------------------------------------------------------------
//	 FW_CPartFrameIterator::FindNext
//------------------------------------------------------------------------------

void FW_CPartFrameIterator::FindNext()
{
	while (fCurrentFrame != NULL)
	{
		if (fPresentation == kNoPresentation || fCurrentFrame->GetPresentation() == fPresentation)
		{
			return;
		}
		
		if (fIterator.Next())
			fCurrentFrame = *fIterator.CurrentItem();
		else
			fCurrentFrame = NULL;
	} 
}

//========================================================================================
//	 class FW_CPartFacetIterator
//========================================================================================

//---------------------------------------------------------------------------------------
//	 FW_CPartFacetIterator::FW_CPartFacetIterator
//---------------------------------------------------------------------------------------

FW_CPartFacetIterator::FW_CPartFacetIterator(FW_CPart* part, XMPTypeToken presentation) :
	fFrameIterator(part, presentation),
	fCurrentFacet(NULL),
	fFacetIterator(NULL)	
{
	fCurrentFrame = fFrameIterator.CurrentItem();
	NewFacetIterator();
}

//---------------------------------------------------------------------------------------
//	 FW_CPartFacetIterator::~FW_CPartFacetIterator
//---------------------------------------------------------------------------------------

FW_CPartFacetIterator::~FW_CPartFacetIterator()
{
	delete fFacetIterator;
}

//---------------------------------------------------------------------------------------
//	 FW_CPartFacetIterator::CurrentItem
//---------------------------------------------------------------------------------------

FW_CFacet* FW_CPartFacetIterator::CurrentItem()
{
	if (fFacetIterator == NULL)
	{
		NewFacetIterator();		
	}
	
	return fCurrentFacet;
}
	
//---------------------------------------------------------------------------------------
//	 FW_CPartFacetIterator::Next
//---------------------------------------------------------------------------------------

FW_Boolean FW_CPartFacetIterator::Next()
{
	if (fFacetIterator)
	{
		fFacetIterator->Next();
		fCurrentFacet = fFacetIterator->CurrentItem();
		if (fCurrentFacet == NULL)
		{
			delete fFacetIterator;
			fFacetIterator = NULL;
			fFrameIterator.Next();
			fCurrentFrame = fFrameIterator.CurrentItem();
			NewFacetIterator();
		}
	}
	return !this->IsDone();
}

//---------------------------------------------------------------------------------------
//	 FW_CPartFacetIterator::IsDone
//---------------------------------------------------------------------------------------

FW_Boolean FW_CPartFacetIterator::IsDone()
{
	return fCurrentFacet == NULL;
}

//---------------------------------------------------------------------------------------
//	 FW_CPartFacetIterator::NewFacetIterator
//---------------------------------------------------------------------------------------

void FW_CPartFacetIterator::NewFacetIterator()
{
	if (fCurrentFrame != NULL)
	{
		fFacetIterator = new FW_CFrameFacetIterator(fCurrentFrame);
		fCurrentFacet = fFacetIterator->CurrentItem();		
	}
}

