//========================================================================================
//
//	File:				FWParting.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPARTING_H
#include "FWParting.h"
#endif

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWPROXY_H
#include "FWProxy.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef BCSTOREU_H
#include "BCStoreU.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _CMDDEFS_
#include <CmdDefs.h>
#endif

#ifndef _SUCURSOR_
#include <SUCursor.h>
#endif

#ifndef _ARBITRAT_
#include <Arbitrat.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _POINT_
#include <FW_SPlatformPoint.h>
#endif

#ifndef _MENUBAR_
#include <MenuBar.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _WINSTAT_
#include <WinStat.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _DISPTCH_
#include <Disptch.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__TOOLUTILS__)
#include <ToolUtils.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <Quickdraw.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__EVENTS__)
#include <Events.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__STDLIB__)
#include <StdLib.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(mathRoutinesIncludes)
#include <math routines.h>
#endif

#pragma segment framework

//==============================================================================
//	 class FW_CPart
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::FW_CEmbeddingPart
//------------------------------------------------------------------------------

FW_CEmbeddingPart::FW_CEmbeddingPart() :
	fProxyRunList(NULL),
	fProxyRunCount(0),
	fSupportRequestEmbeddedFrame(TRUE)
{
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::~FW_CEmbeddingPart
//------------------------------------------------------------------------------

FW_CEmbeddingPart::~FW_CEmbeddingPart()
{
	delete fProxyRunList;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::Release
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::Release()
{
	FW_CPart::Release();	
	
	if (this->GetRefCount() == 0)
	{
		WASSERTM(fProxyRunCount == 0, "All proxyRuns should have been deleted");
		
		BC_TCollectionActiveIterator<FW_CProxyRun*> iter(*fProxyRunList);
		FW_CProxyRun* proxyRun;
		while (!iter.IsDone())
		{
			proxyRun = *iter.CurrentItem();
			delete proxyRun;
			iter.Next();
		}
		fProxyRunCount = 0;
	}	
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::Initialize
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::Initialize()
{
	FW_CPart::Initialize();
	
	fProxyRunList = new BC_TDynamicCollection<FW_CProxyRun*, BC_CUnmanaged>(kChunkSize);
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::AttachSourceFrame
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::AttachSourceFrame(XMPFrame* xmpFrame, XMPFrame* xmpSourceFrame)
{
	if (xmpSourceFrame && xmpFrame)
	{
		FW_CEmbeddingFrame* sourceFrame = (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(xmpSourceFrame);	
		FW_CEmbeddingFrame* frame = (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(xmpFrame);
		frame->AttachSourceFrame(sourceFrame);
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::CreateEmbeddedFramesIterator
//------------------------------------------------------------------------------

XMPEmbeddedFramesIterator* FW_CEmbeddingPart::CreateEmbeddedFramesIterator(XMPFrame* displayFrame)
{
	return new FW_CEmbeddedXMPFramesIterator(this, displayFrame ? (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(displayFrame) : NULL);
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::UsedShapeChanged
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::UsedShapeChanged(XMPFrame* embeddedXMPFrame)
{
	FW_CEmbeddingFrame *containingFrame = (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(embeddedXMPFrame->GetContainingFrame());	
	FW_CProxyRun *proxyRun = XMPFrametoProxyRun(embeddedXMPFrame);	

	containingFrame->UsedShapeChanged(proxyRun, embeddedXMPFrame);
	proxyRun->UsedShapeChanged(embeddedXMPFrame);
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::EmbedPart
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::EmbedPart(XMPPart *embeddedXMPPart,
								FW_CProxyRun* proxyRun, 
								XMPTypeToken inWhichPresentation,
								XMPTypeToken viewType,
								XMPTypeToken presentation,
								XMPTransform* externalXForm, 
								XMPShape* frameShape,
								FW_Boolean invalidate)
{	
	// ----- Create a shape if neccessary -----
	XMPShape *invalidShape = ::NewXMPShape();

	// ----- Create an embedded frame for each of my display frame -----		
	FW_CPartFrameIterator ite(this);
	FW_CEmbeddingFrame* frame;
	XMPFrame* sourceFrame = NULL;
	while (!ite.IsDone())
	{
		frame = (FW_CEmbeddingFrame*)ite.CurrentItem();
		if (inWhichPresentation == kNoPresentation || frame->GetPresentation() == inWhichPresentation)
		{
			// ----- Create the frame -----
			XMPShape *itsShape = ::NewXMPShape(frameShape);
			FW_CProxyFrame* proxyFrame = frame->CreateEmbeddedFrame(embeddedXMPPart, proxyRun, itsShape, viewType, presentation, 0, FALSE);
						
			// ----- Attach source frames if needed -----
			if (sourceFrame)
				embeddedXMPPart->AttachSourceFrame(proxyFrame->GetXMPFrame(), sourceFrame);
			sourceFrame = proxyFrame->GetXMPFrame();
	
			// ----- Create its facets -----
			FW_CFrameFacetIterator ite2(frame);
			FW_CFacet* facet;
			while (!ite2.IsDone())
			{
				facet = ite2.CurrentItem();
				XMPShape* clip = ::NewXMPShape();
				proxyFrame->GetFrameShape(clip);
				XMPTransform* xform = ::NewXMPTransform(externalXForm);
				
				FW_Boolean svRedraw = facet->SetRedraw(invalidate);
				
				facet->CreateEmbeddedFacet(proxyFrame, clip, xform, NULL, kXMPFrameInFront);
				facet->ClipEmbeddedFacets();

				proxyFrame->GetFrameShape(invalidShape); 	// May have changed when the facet was added
				invalidShape->Transform(externalXForm);
				facet->Invalidate(invalidShape);

				facet->SetRedraw(svRedraw);
				ite2.Next();
			}
		}
		ite.Next();
	}
	
	delete invalidShape;
		
	delete frameShape;
	delete externalXForm;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::EmbedFrame
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::EmbedFrame(XMPFrame* embeddedXMPFrame,
								FW_CProxyRun* proxyRun, 
								XMPTypeToken inWhichPresentation, 		
								XMPTransform* externalXForm, 
								FW_Boolean invalidate)
{	
	XMPPart *embeddedXMPPart = embeddedXMPFrame->GetPart();
	
	// ----- Create a shape if neccessary -----
	XMPShape *invalidShape = ::NewXMPShape();
	
	// ----- Get Info from embeddedXMPFrame -----
	XMPShape* frameShape = ::NewXMPShape(embeddedXMPFrame->GetFrameShape());
	XMPTypeToken viewType = embeddedXMPFrame->GetViewType();
	XMPTypeToken presentation = embeddedXMPFrame->GetPresentation();
	
	// ----- Create an embedded frame for each of my display frame -----		
	FW_CPartFrameIterator ite(this);
	FW_CEmbeddingFrame* frame;
	XMPFrame* sourceFrame = NULL;
	while (!ite.IsDone())
	{
		frame = (FW_CEmbeddingFrame*)ite.CurrentItem();
		if (inWhichPresentation == kNoPresentation || frame->GetPresentation() == inWhichPresentation)
		{
			FW_CProxyFrame* proxyFrame;
			
			// ----- Create the frame -----
			if (embeddedXMPFrame)
			{
				proxyFrame = proxyRun->NewProxyFrame(embeddedXMPFrame);
				embeddedXMPFrame->Release();
				embeddedXMPFrame->SetContainingFrame(frame->GetXMPFrame());
				frame->AddProxyFrame(proxyFrame);
				embeddedXMPFrame = NULL;
			}
			else
			{
				XMPShape *itsShape = ::NewXMPShape(frameShape);
				proxyFrame = frame->CreateEmbeddedFrame(embeddedXMPPart, proxyRun, 
														itsShape, viewType, presentation, 
														0, FALSE);
			}
			
			// ----- Attach source frames if needed -----
			if (sourceFrame)
				embeddedXMPPart->AttachSourceFrame(proxyFrame->GetXMPFrame(), sourceFrame);
			sourceFrame = proxyFrame->GetXMPFrame();
	
			// ----- Create its facets -----
			FW_CFrameFacetIterator ite2(frame);
			FW_CFacet* facet;
			while (!ite2.IsDone())
			{
				facet = ite2.CurrentItem();
				XMPShape* clip = ::NewXMPShape();
				proxyFrame->GetFrameShape(clip);
				XMPTransform* xform = ::NewXMPTransform(externalXForm);
				
				FW_Boolean svRedraw = facet->SetRedraw(invalidate);
				
				facet->CreateEmbeddedFacet(proxyFrame, clip, xform, NULL, kXMPFrameInFront);
				facet->ClipEmbeddedFacets();

				proxyFrame->GetFrameShape(invalidShape); 	// May have changed when the facet was added
				invalidShape->Transform(externalXForm);
				facet->Invalidate(invalidShape);

				facet->SetRedraw(svRedraw);
				ite2.Next();
			}
		}
		ite.Next();
	}
	
	delete invalidShape;
		
	delete frameShape;
	delete externalXForm;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::RequestEmbeddedFrame
//------------------------------------------------------------------------------

XMPFrame* FW_CEmbeddingPart::RequestEmbeddedFrame(XMPFrame* containingFrame,
										   			XMPFrame* baseFrame,
										   			XMPShape* frameShape,
										   			XMPPart* embeddedXMPPart,
										   			XMPTypeToken viewType,
										   			XMPTypeToken presentation,
										   			FW_Boolean isOverlaid)
{
	if (fSupportRequestEmbeddedFrame)
	{
		FW_CEmbeddingFrame* frame = (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(containingFrame);
		
		// ----- Create ProxyRun -----
		FW_CProxyRun* proxyRun = NewProxyRun();
		
		// ----- Create and return the embedded frame -----
		FW_CProxyFrame* proxyFrame = frame->CreateEmbeddedFrame(embeddedXMPPart, proxyRun, frameShape, 
														     	viewType, presentation, baseFrame->GetFrameGroup(), 
															 	isOverlaid);
		return proxyFrame->GetXMPFrame();
	}
	else
	{
		return NULL;
	}
}
	
//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::RemoveEmbeddedFrame
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::RemoveEmbeddedFrame(XMPFrame* xmpFrame)
{
	XMPFrame* containingXMPFrame = xmpFrame->GetContainingFrame();
	FW_CEmbeddingFrame* containingFrame = (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(containingXMPFrame);
	FW_CProxyFrame* proxyFrame = containingFrame->XMPtoProxyFrame(xmpFrame);
	
	containingFrame->RemoveEmbeddedFrame(proxyFrame);	// Note: will remove the XMPEmbeddedFrame from the draft
	
	// Note: the proxyRun will be automatically removed (and deleted) when the last proxyFrame
	// is deleted
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::HandleEvent
//------------------------------------------------------------------------------

FW_Boolean FW_CEmbeddingPart::HandleEvent(XMPEventData event, XMPFrame* xmpFrame, XMPFacet* xmpFacet)
{
	FW_Boolean handled = false;
	FW_CFacet* facet = FW_CFacet::XMPtoFWFacet(xmpFacet);
	
	switch (event->what)
	{
		case kXMPEvtMouseUpEmbedded:
			break;
			
		case kXMPEvtMouseUpBorder:
			break;

		case kXMPEvtMouseDownEmbedded:
			handled = facet->HandleMouseDown((XMPFacet*)event->message, event);
			break;
			
		case kXMPEvtMouseDownBorder:
			handled = facet->HandleMouseDownInEmbeddedFrameBorder((XMPFacet*)event->message, event);
			break;
		
		default:
			return FW_CPart::HandleEvent(event, xmpFrame, xmpFacet);
	}
	
	return handled;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::DoAdjustMenus
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::DoAdjustMenus(XMPMenuBar* menuBar)
{
	FW_CPart::DoAdjustMenus(menuBar);

	menuBar->EnableCommand(kXMPCommandGetPartInfo, HasSelectedProxyRun());	
	menuBar->EnableCommand(kXMPCommandInsert, TRUE);	
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::DoMenuEvent
//------------------------------------------------------------------------------

FW_Boolean FW_CEmbeddingPart::DoMenuEvent(XMPMenuBar *menuBar, XMPCommandID commandID)
{
	FW_Boolean result = TRUE;
	FW_CSelection* selection = NULL;

	switch (commandID)
	{
		case kXMPCommandInsert:
			selection = GetSelection();
			if (selection != NULL)
				result = selection->InsertNewPart();
			else
				SysBeep(1);
			break;
			
		default:
			result = FW_CPart::DoMenuEvent(menuBar, commandID);
	};
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::ClipEmbeddedFrames
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::ClipEmbeddedFrames(XMPTypeToken whichFrames)
{
	FW_CPartFrameIterator ite(this, whichFrames);
	while (!ite.IsDone())
	{
		((FW_CEmbeddingFrame*)ite.CurrentItem())->ClipEmbeddedFrames();
		ite.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::CloneFrameFrom
//------------------------------------------------------------------------------

XMPFrame* FW_CEmbeddingPart::CloneFrameFrom(XMPStorageUnit* storageUnit, XMPStorageUnitRef frameSURef, XMPCloneKind kind)
{
	XMPDraft* myDraft = GetStorageUnit()->GetDraft();
	XMPStorageUnit* newSU = myDraft->CreateStorageUnit();
	
	XMPDraft* srcDraft = storageUnit->GetDraft();
	XMPStorageUnit* srcSU = srcDraft->GetStorageUnit(storageUnit->GetIDFromStorageUnitRef(frameSURef));

	XMPDraftKey key = srcDraft->BeginClone(kind);
	srcSU->CloneInto(key, newSU, NULL);
	srcDraft->EndClone(key);

	srcSU->Release();

	XMPFrame* xmpFrame = myDraft->GetFrame(newSU->GetID());
	
	newSU->Release();
	
	return xmpFrame;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::XMPFrametoProxyRun
//------------------------------------------------------------------------------

FW_CProxyRun* FW_CEmbeddingPart::XMPFrametoProxyRun(XMPFrame* embeddedXMPFrame) const
{
	BC_TCollectionActiveIterator<FW_CProxyRun*> ite(*fProxyRunList);
	FW_CProxyRun* proxyRun;
	while (!ite.IsDone())
	{
		proxyRun = *ite.CurrentItem();
		if (proxyRun->GetXMPPart() == embeddedXMPFrame->GetPart())
		{
			BC_TCollectionActiveIterator<FW_CProxyFrame*> ite2(*proxyRun->GetProxyFrameList());
			FW_CProxyFrame* proxyFrame;
			while (!ite2.IsDone())
			{
				proxyFrame = *ite2.CurrentItem();
				if (proxyFrame->GetXMPFrame() == embeddedXMPFrame)
					return proxyRun;
				ite2.Next();
			}
		}
		ite.Next();
	}
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::XMPtoProxyFrame
//------------------------------------------------------------------------------

FW_CProxyFrame* FW_CEmbeddingPart::XMPtoProxyFrame(XMPFrame* embeddedXMPFrame) const
{
	BC_TCollectionActiveIterator<FW_CProxyRun*> ite(*fProxyRunList);
	FW_CProxyRun* proxyRun;
	while (!ite.IsDone())
	{
		proxyRun = *ite.CurrentItem();
		if (proxyRun->GetXMPPart() == embeddedXMPFrame->GetPart())
		{
			BC_TCollectionActiveIterator<FW_CProxyFrame*> ite2(*proxyRun->GetProxyFrameList());
			FW_CProxyFrame* proxyFrame;
			while (!ite2.IsDone())
			{
				proxyFrame = *ite2.CurrentItem();
				if (proxyFrame->GetXMPFrame() == embeddedXMPFrame)
					return proxyFrame;
				ite2.Next();
			}
		}
		ite.Next();
	}
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::HasSelectedProxyRun
//------------------------------------------------------------------------------

FW_Boolean FW_CEmbeddingPart::HasSelectedProxyRun() const
{
	FW_Boolean result = FALSE;
	BC_TCollectionActiveIterator<FW_CProxyRun*> ite(*fProxyRunList);
	FW_CProxyRun* proxyRun;
	while (!ite.IsDone())
	{
		proxyRun = *ite.CurrentItem();
		if (proxyRun->IsSelected())
		{
			result = TRUE;
			break;
		}
		ite.Next();
	}
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::SetEmbeddedFramesContainingFrame
//------------------------------------------------------------------------------
//	This is call by AddDisplayFrame when the first display frame is added. The part
//	may have orphaned embedded frames that we have to attach to the frame passed as
// 	parameter

void FW_CEmbeddingPart::SetEmbeddedFramesContainingFrame(FW_CEmbeddingFrame* frame)
{
	FW_CEmbeddedXMPFramesIterator ite(this);
	for (XMPFrame* xmpFrame = ite.First(); ite.IsNotComplete(); xmpFrame = ite.Next())
	{
		if (xmpFrame->GetContainingFrame() == NULL)
		{
			xmpFrame->SetContainingFrame(frame->GetXMPFrame());
			FW_CProxyFrame* proxyFrame = XMPtoProxyFrame(xmpFrame);
			frame->AddProxyFrame(proxyFrame);
		}
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::RemoveEmbeddedFrames
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::RemoveEmbeddedFrames(FW_CProxyRun* proxyRun)
{
	//  We have to collect first all the embedded frames
	//  Because the proxyRun will be deleted when the last proxyFrame
	//  will be
	BC_TDynamicCollection<FW_CProxyFrame*, BC_CUnmanaged> tempCollect(kChunkSize);
	BC_TCollectionActiveIterator<FW_CProxyFrame*> ite(*proxyRun->GetProxyFrameList());
	FW_CProxyFrame* proxyFrame;
	while (!ite.IsDone())
	{
		proxyFrame = *ite.CurrentItem();
		tempCollect.Append(proxyFrame);
		ite.Next();
	}
	
	// ----- Now we can call RemoveEmbeddedFrame -----
	BC_TCollectionActiveIterator<FW_CProxyFrame*> ite2(tempCollect);
	while (!ite2.IsDone())
	{
		proxyFrame = *ite2.CurrentItem();
		this->RemoveEmbeddedFrame(proxyFrame->GetXMPFrame());
		ite2.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::StartDragging
//------------------------------------------------------------------------------

void FW_CEmbeddingPart::Dragging(FW_CFacet* facet, FW_Boolean state)
{
	FW_CFrame* frame = facet->GetFrame();
	
	BC_TCollectionActiveIterator<FW_CProxyRun*> ite(*fProxyRunList);
	FW_CProxyRun* proxyRun;
	while (!ite.IsDone())
	{
		proxyRun = *ite.CurrentItem();
		if (proxyRun->IsSelected())
		{
			BC_TCollectionActiveIterator<FW_CProxyFrame*> ite2(*proxyRun->GetProxyFrameList());
			FW_CProxyFrame* proxyFrame;
			while (!ite2.IsDone())
			{
				proxyFrame = *ite2.CurrentItem();
				if (proxyFrame->GetContainingFrame() == frame)
					proxyFrame->SetDragging(state);
				ite2.Next();
			}
		}
		ite.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::ClonePartFrom
//------------------------------------------------------------------------------

XMPPart* FW_CEmbeddingPart::ClonePartFrom(XMPStorageUnit* storageUnit, XMPCloneKind kind)
{
	XMPDraft* myDraft = GetStorageUnit()->GetDraft();
	XMPDraft* srcDraft = storageUnit->GetDraft();

	XMPDraftKey key = srcDraft->BeginClone(kind);
	XMPStorageUnit* newSU = storageUnit->CloneTo(key, myDraft, NULL);
	srcDraft->EndClone(key);
	
	XMPPart* xmpPart = myDraft->GetPart(newSU->GetID());
	
	newSU->Release();
	
	return xmpPart;	
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::IsEmbeddingPart
//------------------------------------------------------------------------------
//	This needs to be removed

FW_Boolean FW_CEmbeddingPart::IsEmbeddingPart() const
{
	return TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::AddProxyRun
//------------------------------------------------------------------------------
void FW_CEmbeddingPart::AddProxyRun(FW_CProxyRun* proxyRun)
{
	fProxyRunList->Append(proxyRun);
	fProxyRunCount++;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddingPart::RemoveProxyRun
//------------------------------------------------------------------------------
void FW_CEmbeddingPart::RemoveProxyRun(FW_CProxyRun* proxyRun)
{
	fProxyRunList->Remove(fProxyRunList->Location(proxyRun));
	fProxyRunCount--;
}

//==============================================================================
//	 FW_CEmbeddedXMPFramesIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFramesIterator::FW_CEmbeddedXMPFramesIterator
//------------------------------------------------------------------------------

FW_CEmbeddedXMPFramesIterator::FW_CEmbeddedXMPFramesIterator(FW_CEmbeddingPart* part, FW_CEmbeddingFrame* displayFrame)
{
	fDisplayFrame = displayFrame;
	fPart = part;
	fCurrentProxyRun = NULL;
	fCurrentProxyFrame = NULL;
	fProxyRunIter = NULL;
	fProxyFrameIter = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFramesIterator::~FW_CEmbeddedXMPFramesIterator
//------------------------------------------------------------------------------

FW_CEmbeddedXMPFramesIterator::~FW_CEmbeddedXMPFramesIterator()
{
	delete fProxyRunIter;
	delete fProxyFrameIter;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFramesIterator::First
//------------------------------------------------------------------------------

XMPFrame* FW_CEmbeddedXMPFramesIterator::First()
{
	delete fProxyRunIter;
	
	delete fProxyFrameIter;
	fProxyFrameIter = NULL;

	fProxyRunIter = new BC_TCollectionActiveIterator<FW_CProxyRun*>(*fPart->GetProxyRunList());
	if (!fProxyRunIter->IsDone())
	{
		fCurrentProxyRun = *fProxyRunIter->CurrentItem();
		fProxyFrameIter = new BC_TCollectionActiveIterator<FW_CProxyFrame*>(*fCurrentProxyRun->GetProxyFrameList());
		if (!fProxyFrameIter->IsDone())
		{
			fCurrentProxyFrame = *fProxyFrameIter->CurrentItem();
			TestEmbedded();
		}
		else
			fCurrentProxyFrame = NULL;
	}
	else
		fCurrentProxyRun = NULL;
	return fCurrentProxyFrame ? fCurrentProxyFrame->GetXMPFrame() : (XMPFrame*)NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFramesIterator::Next
//------------------------------------------------------------------------------

XMPFrame* FW_CEmbeddedXMPFramesIterator::Next()
{
	if (fProxyFrameIter)
	{
		if (fProxyFrameIter->Next())
		{
			fCurrentProxyFrame = *fProxyFrameIter->CurrentItem();
			TestEmbedded();
		}
		else
			fCurrentProxyFrame = NULL;

		if (fCurrentProxyFrame == NULL)
		{
			delete fProxyFrameIter;

			if (fProxyRunIter->Next())
			{
				fCurrentProxyRun = *fProxyRunIter->CurrentItem();
				fProxyFrameIter = new BC_TCollectionActiveIterator<FW_CProxyFrame*>(*fCurrentProxyRun->GetProxyFrameList());
				if (!fProxyFrameIter->IsDone())
				{
					fCurrentProxyFrame = *fProxyFrameIter->CurrentItem();
					TestEmbedded();
				}
				else
					fCurrentProxyFrame = NULL;
			}
			else
				fCurrentProxyRun = NULL;
		}
	}
	
	return fCurrentProxyFrame ? fCurrentProxyFrame->GetXMPFrame() : (XMPFrame*)NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFramesIterator::TestEmbedded
//------------------------------------------------------------------------------

void FW_CEmbeddedXMPFramesIterator::TestEmbedded()
{
	if (fDisplayFrame != NULL)
	{
		FW_CProxyFrame** currentProxyFrame = &fCurrentProxyFrame;
		while (currentProxyFrame != NULL && (*currentProxyFrame)->GetContainingFrame() != fDisplayFrame)
		{
			fProxyFrameIter->Next();
			currentProxyFrame = fProxyFrameIter->CurrentItem();
		}
		fCurrentProxyFrame = currentProxyFrame ? *currentProxyFrame : NULL;
	}
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFramesIterator::IsNotComplete
//------------------------------------------------------------------------------

FW_Boolean	FW_CEmbeddedXMPFramesIterator::IsNotComplete()
{
	return fCurrentProxyFrame != NULL;
}

