//========================================================================================
//
//	File:				FRProxy.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPROXY_H
#include "FWProxy.h"
#endif

#ifndef FWPARTING_H
#include "FWParting.h"
#endif

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWBORDER_H
#include "FWBorder.h"
#endif

// ----- Graphic Includes -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

// ----- OPF Foundation Includes -----

#ifndef BCSTOREU_H
#include <BCStoreU.h>
#endif

// ----- OpenDoc Includes -----

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _FRAME_
#include <Frame.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#pragma segment framework

//==============================================================================
//	 class FW_CProxyRun
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CProxyRun::FW_CProxyRun
//------------------------------------------------------------------------------

FW_CProxyRun::FW_CProxyRun()
{
	fProxyFrameList 	= NULL;
	fProxyBorder		= NULL;
	fSelected			= FALSE;
	fCachedProxyFrame 	= NULL;
	fCount				= 0;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::InitProxyRun
//------------------------------------------------------------------------------

void FW_CProxyRun::InitProxyRun(FW_CEmbeddingPart* embeddingPart)
{
	fEmbeddingPart = embeddingPart;
	fEmbeddingPart->AddProxyRun(this);
	
	fProxyFrameList = new BC_TDynamicCollection<FW_CProxyFrame*, BC_CUnmanaged>(kChunkSize);
	
	fProxyBorder = NewProxyBorder();
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::~FW_CProxyRun
//------------------------------------------------------------------------------

FW_CProxyRun::~FW_CProxyRun()
{
	if (fEmbeddingPart)
		fEmbeddingPart->RemoveProxyRun(this);
		
	delete fProxyBorder;
	
	if (fProxyFrameList)
	{
		WASSERTM(fCount == 0, "All proxyFrame should be already deleted");
		BC_TCollectionActiveIterator<FW_CProxyFrame*> iter(*fProxyFrameList);
		FW_CProxyFrame* proxyFrame;
		while (!iter.IsDone())
		{
			proxyFrame = *iter.CurrentItem();
			delete proxyFrame;
			iter.Next();
		}
		fCount = 0;
		delete fProxyFrameList;
		fProxyFrameList = NULL;
	}
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::AddProxyFrame
//------------------------------------------------------------------------------

void FW_CProxyRun::AddProxyFrame(FW_CProxyFrame* proxyFrame)
{
	if (fCachedProxyFrame == NULL)
		fCachedProxyFrame = proxyFrame;
		
	fProxyFrameList->Append(proxyFrame);
	
	fCount++;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::RemoveProxyFrame
//------------------------------------------------------------------------------

void FW_CProxyRun::RemoveProxyFrame(FW_CProxyFrame* proxyFrame)
{
	if (fCachedProxyFrame == proxyFrame)
		fCachedProxyFrame = NULL;
		
	fProxyFrameList->Remove(fProxyFrameList->Location(proxyFrame));
	
	fCount--;
	
	if (fCount == 0)
		delete this;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::UsedShapeChanged
//------------------------------------------------------------------------------

void FW_CProxyRun::UsedShapeChanged(XMPFrame* xmpFrame)
{
FW_UNUSED(xmpFrame);

	fProxyBorder->BorderHasChanged(TRUE);
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::WhichEmbeddedFacet
//------------------------------------------------------------------------------
// mouse in window coordinate system

XMPFacet* FW_CProxyRun::WhichEmbeddedFacet(FW_CFacet* embeddingFacet, const FW_CPoint& mouse)
{	
	XMPFacet *result = NULL;
	FW_CEmbeddedXMPFacetsIterator ite(this, embeddingFacet, kXMPFrontToBack);
	for (XMPFacet* xmpFacet = ite.First(); ite.IsNotComplete(); xmpFacet = ite.Next())
	{
		if (xmpFacet->ContainsPoint(mouse))
		{
			result = xmpFacet;
			break;
		}
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::NewProxyBorder
//------------------------------------------------------------------------------

FW_CBaseProxyBorder* FW_CProxyRun::NewProxyBorder()
{
	FW_CProxyBorder* border = new FW_CProxyBorder;
	border->InitProxyBorder(this);
	return border;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::SetSelectState
//------------------------------------------------------------------------------

void FW_CProxyRun::SetSelectState(FW_Boolean state)
{
	fSelected = state;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::GetXMPPart
//------------------------------------------------------------------------------
//	Can't be const because of the cache

XMPPart* FW_CProxyRun::GetXMPPart()
{
	if (fCachedProxyFrame == NULL)
		fCachedProxyFrame = fProxyFrameList->First();
	
	
	return fCachedProxyFrame->GetXMPPart();
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::WhichProxyFrame
//------------------------------------------------------------------------------

FW_CProxyFrame* FW_CProxyRun::WhichProxyFrame(FW_CEmbeddingFrame* embeddingFrame)
{
	FW_CProxyFrame *result = NULL;
	
	BC_TCollectionActiveIterator<FW_CProxyFrame*> ite(*fProxyFrameList);
	FW_CProxyFrame* proxyFrame;
	while (!ite.IsDone())
	{
		proxyFrame = *ite.CurrentItem();
		if (proxyFrame->GetContainingFrame() == embeddingFrame)
		{
			result = proxyFrame;
			break;
		}
		ite.Next();
	}
	return result;
}

//----------------------------------------------------------------------------------------
// FW_CProxyRun::NewProxyFrame
//----------------------------------------------------------------------------------------

FW_CProxyFrame* FW_CProxyRun::NewProxyFrame(XMPFrame* xmpFrame)
{
	FW_CProxyFrame* proxyFrame =  new FW_CProxyFrame;
	proxyFrame->InitProxyFrame(this, xmpFrame);
	return proxyFrame;
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::Externalize
//------------------------------------------------------------------------------

void FW_CProxyRun::Externalize(XMPStorageUnit* storageUnit)
{
	unsigned long count = fProxyFrameList->Length();	
	storageUnit->SetValue(sizeof(unsigned long), &count);
	
	BC_TCollectionActiveIterator<FW_CProxyFrame*> ite(*fProxyFrameList);
	FW_CProxyFrame* proxyFrame;
	while (!ite.IsDone())
	{
		proxyFrame = *ite.CurrentItem();
		XMPStorageUnitRef aSURef = storageUnit->GetStrongStorageUnitRef(proxyFrame->GetXMPFrame()->GetStorageUnit());		
		storageUnit->SetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);			
		ite.Next();
	}
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::CloneTo
//------------------------------------------------------------------------------

void FW_CProxyRun::CloneTo(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind)
{
	XMPStorageUnit* newSU = storageUnit->GetDraft()->CreateStorageUnit();

	// ----- Look for the frame embedded in commandFrame -----
	FW_CProxyFrame *proxyFrame = WhichProxyFrame((FW_CEmbeddingFrame*)commandFrame);
	XMPStorageUnit* InitiatingFrameSU = proxyFrame ? proxyFrame->GetXMPFrame()->GetStorageUnit() : NULL;

	XMPDraft* fromDraft = fEmbeddingPart->GetStorageUnit()->GetDraft();
	XMPDraftKey	key = fromDraft->BeginClone(cloneKind);
	
	// ----- Clone the embedded part -----
	GetXMPPart()->CloneInto(key, newSU, InitiatingFrameSU);

	// ----- Clone the initiating frame -----
	XMPStorageUnit* frameSU = proxyFrame->GetXMPFrame()->CloneTo(key, storageUnit->GetDraft());
		
	fromDraft->EndClone(key);
	
	newSU->Release();
	
	XMPStorageUnitRef frameSURef = storageUnit->GetStrongStorageUnitRef(frameSU);
	frameSU->Release();
	
	// ----- Write out -----
	unsigned long count = 1;
	storageUnit->SetValue(sizeof(count), (XMPValue)&count);		
	storageUnit->SetValue(sizeof(frameSURef), (XMPValue)&frameSURef);	
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::CloneInto
//------------------------------------------------------------------------------

void FW_CProxyRun::CloneInto(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind)
{
	FW_CProxyFrame *proxyFrame = WhichProxyFrame((FW_CEmbeddingFrame*)commandFrame);
	XMPStorageUnit* InitiatingFrameSU = proxyFrame ? proxyFrame->GetXMPFrame()->GetStorageUnit() : NULL;

	XMPDraft* fromDraft = fEmbeddingPart->GetStorageUnit()->GetDraft();
	
	XMPDraftKey	key = fromDraft->BeginClone(cloneKind);
	GetXMPPart()->CloneInto(key, storageUnit, InitiatingFrameSU);
	fromDraft->EndClone(key);
	
	// ----- Add the shape of the frame -----
	if (proxyFrame)
	{
		XMPShape *shape = proxyFrame->GetXMPFrame()->GetFrameShape();
		storageUnit->AddProperty(kXMPPropFrameShape)->AddValue(kXMPFrame);
		shape->WriteShape(storageUnit);
	}
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::CloneFrom
//------------------------------------------------------------------------------

XMPFrame* FW_CProxyRun::CloneFrom(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	XMPStorageUnitRef ref;
	unsigned long count;
	storageUnit->GetValue(sizeof(count), (XMPValue)&count);
	storageUnit->GetValue(sizeof(ref), (XMPValue)&ref);
	
	return fEmbeddingPart->CloneFrameFrom(storageUnit, ref, cloneKind);
}

//------------------------------------------------------------------------------
//	 FW_CProxyRun::Internalize
//------------------------------------------------------------------------------

void FW_CProxyRun::Internalize(XMPStorageUnit* storageUnit)
{
	XMPFrame* xmpEmbeddedFrame;
	FW_VOLATILE(xmpEmbeddedFrame);
	
	unsigned long count;	
	storageUnit->GetValue(sizeof(unsigned long), (XMPValue)&count);
	
	unsigned long nbProxyFrame = 0;
	for (unsigned long i = 0; i<count; i++)
	{
		XMPStorageUnitRef aSURef;
		storageUnit->GetValue(sizeof(XMPStorageUnitRef), (XMPValue)&aSURef);
		TRY
			XMPStorageUnitID id = storageUnit->GetIDFromStorageUnitRef(aSURef);
			xmpEmbeddedFrame = storageUnit->GetDraft()->GetFrame(id);
		CATCH_ALL
			xmpEmbeddedFrame = NULL;
		ENDTRY
		
		if (xmpEmbeddedFrame)
		{
			nbProxyFrame++;
			
			FW_CProxyFrame* proxyFrame = NewProxyFrame(xmpEmbeddedFrame);
			FW_CEmbeddingFrame* embeddingFrame = (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(xmpEmbeddedFrame->GetContainingFrame());
			if (embeddingFrame)
				embeddingFrame->AddProxyFrame(proxyFrame);
			
			xmpEmbeddedFrame->Release();			// This is needed to balance an additional reference created when GetFrame
													// was called.  We keep our own reference in proxyFrame
		}
	}
}

//==============================================================================
//	 class FW_CProxyFrame
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CProxyFrame::FW_CProxyFrame
//------------------------------------------------------------------------------

FW_CProxyFrame::FW_CProxyFrame()
{
	fProxyRun = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CProxyFrame::IProxyFrame
//------------------------------------------------------------------------------

void FW_CProxyFrame::InitProxyFrame(FW_CProxyRun *proxyRun, XMPFrame *xmpFrame)
{
	InitBaseFrame(xmpFrame);
	
	fProxyRun = proxyRun;
	fProxyRun->AddProxyFrame(this);
}

//------------------------------------------------------------------------------
//	 FW_CProxyFrame::~FW_CProxyFrame
//------------------------------------------------------------------------------

FW_CProxyFrame::~FW_CProxyFrame()
{
	if (fProxyRun)
		fProxyRun->RemoveProxyFrame(this);
}

//------------------------------------------------------------------------------
//	 FW_CProxyFrame::GetContainingFrame
//------------------------------------------------------------------------------

FW_CEmbeddingFrame* FW_CProxyFrame::GetContainingFrame() const
{
	XMPFrame* containingFrame = GetXMPFrame()->GetContainingFrame();
	if (containingFrame)
		return (FW_CEmbeddingFrame*)FW_CFrame::XMPtoFWFrame(containingFrame);
	else
		return NULL;
}

//==============================================================================
//	 CEmbeddedXMPFacetIterator
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFacetsIterator::FW_CEmbeddedXMPFacetsIterator
//------------------------------------------------------------------------------

FW_CEmbeddedXMPFacetsIterator::FW_CEmbeddedXMPFacetsIterator(FW_CProxyRun* proxyRun, 
													FW_CFacet* facet, 
													XMPSiblingOrder siblingOrder) :
	fXMPFacetIterator(facet->GetXMPFacet(),
					  kXMPChildrenOnly,
					  siblingOrder)
{
	fProxyRun = proxyRun;
	fXMPFacet = NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFacetsIterator::~FW_CEmbeddedXMPFacetsIterator
//------------------------------------------------------------------------------

FW_CEmbeddedXMPFacetsIterator::~FW_CEmbeddedXMPFacetsIterator()
{
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFacetsIterator::First
//------------------------------------------------------------------------------

XMPFacet* FW_CEmbeddedXMPFacetsIterator::First()
{
	fXMPFacet = fXMPFacetIterator.First();
	FindNext();
	return fXMPFacet;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFacetsIterator::Next
//------------------------------------------------------------------------------

XMPFacet* FW_CEmbeddedXMPFacetsIterator::Next()
{
	fXMPFacet = fXMPFacetIterator.Next();
	FindNext();
	return fXMPFacet;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFacetsIterator::IsNotComplete
//------------------------------------------------------------------------------

FW_Boolean FW_CEmbeddedXMPFacetsIterator::IsNotComplete()
{
	return fXMPFacet != NULL;
}

//------------------------------------------------------------------------------
//	 FW_CEmbeddedXMPFacetsIterator::FindNext
//------------------------------------------------------------------------------

void FW_CEmbeddedXMPFacetsIterator::FindNext()
{
	while (fXMPFacet != NULL)
	{
		if (fXMPFacet->GetFrame()->GetPart() == fProxyRun->GetXMPPart())
		{
			return;
		}
		fXMPFacet = fXMPFacetIterator.Next();
	} 
}

