//========================================================================================
//
//	File:				FWSelect.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWPARTING_H
#include "FWParting.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWPROXY_H
#include "FWProxy.h"
#endif

// ----- Framework Includes -----

#ifndef FWPRISTR_H
#include <FWPriStr.h>
#endif

#ifndef FWSTRING_H
#include <FWString.h>
#endif

#ifndef FWMEMMGR_H
#include <FWMemMgr.h>
#endif

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _CLIPBD_
#include <Clipbd.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _STDPROPS_
#include <StdProps.h>
#endif

#ifndef _DRAGDROP_
#include <DragDrop.h>
#endif

#ifndef _STDTYPES_
#include <StdTypes.h>
#endif

#ifndef _TRNSFORM_
#include <Trnsform.h>
#endif

#ifndef _LINK_
#include <Link.h>
#endif

#ifndef _LINKSPEC_
#include <LinkSpec.h>
#endif

#ifndef _LINKSRC_
#include <LinkSrc.h>
#endif

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _STDDEFS_
#include <StdDefs.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__STANDARDFILE__)
#include <StandardFile.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__RESOURCES__)
#include <Resources.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(mathRoutinesIncludes)
#include <math routines.h>
#endif

#pragma segment framework

//==============================================================================
//	 class FW_CSelection
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CSelection::FW_CSelection
//------------------------------------------------------------------------------

FW_CSelection::FW_CSelection()
{
}

//------------------------------------------------------------------------------
//	 FW_CSelection::InitSelection
//------------------------------------------------------------------------------

void FW_CSelection::InitSelection(FW_CPart* part,  FW_Boolean allowPublish, FW_Boolean allowSubscrib)
{
	fPart = part;
	fAllowPublish = allowPublish;
	fAllowSubscrib = allowSubscrib;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::~FW_CSelection
//------------------------------------------------------------------------------

FW_CSelection::~FW_CSelection()
{
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoCut
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::DoCut(FW_CFrame* frame)
{
	Copy(FALSE, frame, kXMPCloneCut);	// we dont't allow linking for a cut
	return DoClear();
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoCopy
//------------------------------------------------------------------------------

void FW_CSelection::DoCopy(FW_CFrame* frame)
{
	Copy(fAllowPublish, frame, kXMPCloneCopy);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::IsSelectionOnlyOneProxyRun
//------------------------------------------------------------------------------

FW_CProxyRun* FW_CSelection::IsSelectionOnlyOneProxyRun() const
{
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::Copy
//------------------------------------------------------------------------------

void FW_CSelection::Copy(FW_Boolean allowPublish, FW_CFrame* frame, XMPCloneKind cloneKind)
{
	XMPClipboard* clipboard = fPart->GetSession()->GetClipboard();
	XMPClipboardKey key;
	if (clipboard->Lock(0, &key))
	{
		clipboard->Clear(key);
		XMPStorageUnit* clipBSU = clipboard->GetContentStorageUnit(key);
		
		this->WriteSelection(clipBSU, frame, cloneKind);			
		
// NOTE: This code is commented out until the linking API's stabalize.
/*
		// ----- Write out link Spec if there is one -----
		if (allowPublish && IsSelectionPublishable())
		{
			XMPChangeID changeID = clipboard->GetChangeID();

			FW_CPublishLink* publishLink = NewPublishLink(changeID);
			fPart->SetPendingPublish(publishLink);	
	
			XMPLinkSpec* linkSpec = fPart->GetStorageUnit()->GetDraft()->CreateLinkSpec(fPart, &changeID, sizeof(XMPChangeID));

			clipBSU->AddProperty(kXMPPropLinkSpec);
			linkSpec->WriteToProperty(clipBSU);
			delete linkSpec;
		}
*/
		
		clipboard->Unlock(key);
	}

}

//------------------------------------------------------------------------------
//	 FW_CSelection::GetSelectionShape
//------------------------------------------------------------------------------

XMPShape* FW_CSelection::GetSelectionShape()
{
	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::IsSelectionPublishable
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::IsSelectionPublishable()
{
	return TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::NewPublishLink
//------------------------------------------------------------------------------
//	Must overridden if supporting publishing

FW_CPublishLink* FW_CSelection::NewPublishLink(XMPChangeID changeID)
{
FW_UNUSED(changeID);

	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::NewSubscribLink
//------------------------------------------------------------------------------
//	Must overridden if supporting subscribing

FW_CSubscribLink* FW_CSelection::NewSubscribLink(XMPLink *xmpLink)
{
FW_UNUSED(xmpLink);

	return NULL;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoPaste
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::DoPaste()
{
	return Paste(FALSE, kXMPClonePaste);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoPasteWithLink
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::DoPasteWithLink()
{
	return Paste(fAllowSubscrib, kXMPClonePaste);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoPaste
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::Paste(FW_Boolean allowSubscrib, XMPCloneKind cloneKind)
{
	FW_Boolean result = FALSE;
	
	XMPClipboard* clipboard = fPart->GetSession()->GetClipboard();
	XMPClipboardKey key;
	if (clipboard->Lock(0, &key))
	{
		XMPStorageUnit* clipBSU = clipboard->GetContentStorageUnit(key);

		result = this->ReadSelection(clipBSU, cloneKind);
				
		// ----- Link -----
// NOTE: This code is commented out until the linking API's stabalize.
/*
		if (result && allowSubscrib && clipBSU->Exists(kXMPPropLinkSpec, (XMPValueType)NULL, 0))
		{
			XMPLinkSpec* linkSpec = new XMPLinkSpec;
			clipBSU->Focus(kXMPPropLinkSpec, kXMPPosUndefined, (XMPValueType)NULL, 0, kXMPPosUndefined);
			linkSpec->ReadFromProperty(clipBSU);
			XMPLink *xmpLink = fPart->GetStorageUnit()->GetDraft()->GetLink((XMPStorageUnitID)NULL, linkSpec);	
			
			FW_CSubscribLink *subscribLink = NewSubscribLink(xmpLink);
			fPart->AddToSubscribLinkList(subscribLink);
			subscribLink->Subscrib();
			
			xmpLink->RegisterDependent(fPart, 0);
			delete linkSpec;
		}
*/		
		clipboard->Unlock(key);
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::Drag
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::Drag(FW_CFacet* facet, XMPEventData event)
{
	XMPPart* destPart;
	XMPID frameID;
	XMPDragAndDrop*	drag;
	XMPStorageUnit*	dragSU;
	XMPDropResult dropResult;

	if (!::WaitMouseMoved(event->where))
		return FALSE;
	
	// ----- In the following block we focus on the window -----
	{
		FW_CGraphicContext gc(facet->GetXMPWindow());
	
		XMPRgnHandle dragRgn = this->CalcDragRgn(facet);
	
		FW_CPoint translation(0,0);
		translation = facet->GetWindowFrameTransform()->TransformPoint(translation);
		
		FW_SPlatformPoint qdTranslation = translation;
		::LocalToGlobal(&qdTranslation);
		
		::OffsetRgn(dragRgn, qdTranslation.h, qdTranslation.v);
		
		//  Fill Drag and Drop SU with Props n' Vals
		drag = fPart->GetSession()->GetDragAndDrop();
		drag->Clear();
		dragSU = drag->GetContentStorageUnit();
		
		this->WriteSelection(dragSU, facet->GetFrame(), kXMPCloneCut);
		
		// save ID of frame being dragged
		frameID = facet->GetFrame()->GetXMPFrame()->GetID();
		dragSU->AddProperty(kXMPPropFrameInfo)->AddValue(kXMPID);
		dragSU->SetValue(sizeof(XMPID), (XMPValue) &frameID);
		
		// save offset between mousedown pt and topLeft pt of selection
		FW_SPlatformPoint mouseOffset = {0,0};
		FW_SPlatformPoint topLeft;
		
		// Top left of selection
		topLeft.v = (*dragRgn)->rgnBBox.top;
		topLeft.h = (*dragRgn)->rgnBBox.left;
		
		mouseOffset = event->where;	
		::SubPt(topLeft,&mouseOffset);
		
		dragSU->AddProperty(kXMPPropMouseDownOffset)->AddValue(kQDPoint);
		dragSU->SetValue(sizeof(FW_SPlatformPoint), (XMPValue) &mouseOffset);
		
		fPart->Dragging(facet, TRUE);
		
		dropResult = drag->StartDrag(facet->GetFrame()->GetXMPFrame(), "RGNH", dragRgn, &destPart, event);
	
		if (dropResult == kXMPDropMove || dropResult == kXMPDropCopy)
		{
			if ((destPart != fPart) && (!::TestOptionKey(event) || ::TestControlKey(event)))
				this->DoClear();
		}
		
		fPart->Dragging(facet, FALSE);
	
		::DisposeRgn(dragRgn);
	}
	
	return TRUE;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::WriteSelection
//------------------------------------------------------------------------------
//	Here we have two cases: 
//		1) The selection consist of only one embedded part. In this case
//			we just want to clone it into the clipboard
//		2) The selection is two or more embedded parts or one or more
//			embedded parts plus content. In this case we externalize the selection

void FW_CSelection::WriteSelection(XMPStorageUnit* storageUnit, FW_CFrame* commandFrame, XMPCloneKind cloneKind)
{
	FW_CProxyRun *selectedProxyRun = IsSelectionOnlyOneProxyRun();
	if (selectedProxyRun)		// if the selection is only one embedded part clone it into the clipboard 
	{
		selectedProxyRun->CloneInto(storageUnit, commandFrame, cloneKind);
	}
	else
	{		
#ifndef ASLMBUILD
		// ----- Add a kXMPPropPart Property -----
		storageUnit->AddProperty(kXMPPropPart)->AddValue(kXMPISOStr);
		XMPISOStr partName = fPart->GetContentPropertyValueType();
		unsigned long size = FW_PrimitiveStringLength(partName) + 1;
		storageUnit->SetValue(size, partName);
#endif
	
		this->ExternalizeSelection(storageUnit, commandFrame, cloneKind);

		// ----- If supported, write a shape property -----
		XMPShape* selectionShape = GetSelectionShape();
		if (selectionShape)
		{
			// ----- Note: We only add a property, no value because WriteShape will do it
			storageUnit->AddProperty(kXMPPropFrameShape);
			selectionShape->WriteShape(storageUnit);
		}
	}
}

//------------------------------------------------------------------------------
//	 FW_CSelection::ReadSelection
//------------------------------------------------------------------------------
//	ReadSelection tries first to internalize the selection. If the selection is
//	not of a kind that this part can read, an embedding part is read.
//	Note: If the part is a non embedding part ClonePartFrom will return NULL so no
//	embedding will happended.

FW_Boolean FW_CSelection::ReadSelection(XMPStorageUnit* storageUnit, XMPCloneKind cloneKind)
{
	FW_Boolean result = FALSE;
	
	if ((result = InternalizeSelection(storageUnit, cloneKind)) == FALSE)
	{
		XMPPart *xmpPart = fPart->ClonePartFrom(storageUnit, cloneKind);
		if ((result = (xmpPart != NULL)) == TRUE)
		{
			// ----- Look for a frame shape property -----
			XMPShape* frameShape = NULL;
			if (storageUnit->Exists(kXMPPropFrameShape, (XMPValueType)NULL, 0))
			{
				// ----- Note: we focus on the property, ReadShape will focus on the value
				storageUnit->Focus(kXMPPropFrameShape, kXMPPosUndefined, 
							   		(XMPValueType)NULL, (XMPValueIndex)1, kXMPPosUndefined);
				frameShape = ::NewXMPShape(storageUnit);
			}
			
			result = EmbedSelection(xmpPart, frameShape);
			xmpPart->Release();
			delete frameShape;
		}
	}
	
	return result;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::CalcDragRgn
//------------------------------------------------------------------------------

XMPRgnHandle FW_CSelection::CalcDragRgn(FW_CFacet* facet)
{
	FW_CGraphicContext gc(facet->GetXMPFacet());
	
	FW_PlatformRegion dragRgn = facet->GetXMPFacet()->GetActiveShape()->CopyQDRegion();
	FW_PlatformRegion tempRgn = ::NewRgn();
	
	::CopyRgn(dragRgn, tempRgn);
	::InsetRgn(tempRgn, 1, 1);
	::DiffRgn(dragRgn, tempRgn, dragRgn);
	::DisposeRgn(tempRgn);
	
	return dragRgn;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoDroppedInSameFrame
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::DoDroppedInSameFrame(XMPStorageUnit* dropSU, FW_CFacet* facet,
											   const FW_CPoint& originPoint, const FW_CPoint& dropPoint)
{
	FW_UNUSED(dropSU);
	FW_UNUSED(facet);
	FW_UNUSED(originPoint);
	FW_UNUSED(dropPoint);
	
	return FALSE;
}

//------------------------------------------------------------------------------
//	 FW_CSelection::DoDrop
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::DoDrop(XMPStorageUnit* dropSU, FW_CFacet* facet, const FW_CPoint& originPoint, const FW_CPoint& dropPoint)
{
	FW_UNUSED(facet);
	FW_UNUSED(originPoint);
	FW_UNUSED(dropPoint);
	
	return this->ReadSelection(dropSU, kXMPClonePaste);
}

//------------------------------------------------------------------------------
//	 FW_CSelection::EmbedSelection
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::EmbedSelection(XMPPart* xmpPart, XMPShape* frameShape)
{
	FW_UNUSED(xmpPart);
	FW_UNUSED(frameShape);
	
	return FALSE;
}

/* 
Removed until OpenDoc fixes the problem of XMPFile
//------------------------------------------------------------------------------
//	 FW_CSelection::InsertNewPart
//------------------------------------------------------------------------------

FW_Boolean FW_CSelection::InsertNewPart()
{
	FW_Boolean result = kXMPFalse;
	XMPStorageUnit* su = GetPart()->GetStorageUnit();
	XMPSession* session = su->GetSession();
	XMPWindowState* windowState = session->GetWindowState();
	XMPContainer* insertContainer = kXMPNULL;
	XMPDocument* insertDocument = kXMPNULL;
	
	XMPType	partkind = kXMPNULL;

	// StdGetFile please select an editor for the root part
	StandardFileReply sfReply;
	SFTypeList sfTypes;
	
	windowState->DeactivateFrontWindows();
	StandardGetFile((FileFilterUPP)kXMPNULL,-1,sfTypes,&sfReply);
	windowState->ActivateFrontWindows();
	
	if (sfReply.sfGood) 
	{
		XMPFile* file = new XMPFile();
		file->Specify(&(sfReply.sfFile));
		
		TRY
			insertContainer = session->GetStorageSystem()->GetContainer(kXMPDefaultFileContainer, file);
			// IF file->GetContainerType doesn't work then use kXMPDefaultFileContainer
		CATCH_ALL
			insertContainer = kXMPNULL;
			result = kXMPFalse;
		ENDTRY
	
		// If the user chose an OpenDoc XMPContainer
		if (insertContainer != kXMPNULL)
		{
			// ???? Should use a document method to get the top draft? What does that mean?
			insertDocument = insertContainer->GetDocument(kXMPDefaultDocument);
			XMPDraft* nextDraft = kXMPNULL;
			XMPDraft* insertDraft = insertDocument->GetBaseDraft(kDPExclusiveWrite);
		
			{
			TRY
				while(kXMPTrue)
				{
				nextDraft = insertDocument->GetDraft(kDPReadOnly, kXMPNULL, insertDraft, kXMPPosLastAbove, kXMPTrue);
				insertDraft = nextDraft;
				}
			CATCH_ALL
				nextDraft = kXMPNULL;
			ENDTRY
			}
		
			XMPStorageUnit* draftSU = insertDraft->GetDraftProperties();
			XMPStorageUnitRef suRef;
			draftSU->Focus(kXMPPropRootPartSU, kXMPPosUndefined, kXMPStorageUnit, 0, kXMPPosUndefined);
			draftSU->GetValue(sizeof(XMPStorageUnitRef), &suRef);
			XMPStorageUnit* insertSU = insertDraft->GetStorageUnit(draftSU->GetIDFromStorageUnitRef(suRef));
		
			// Clone the root part from the insertDraft to this Draft. As per Insert recipe.
			XMPStorageUnit* newSU = kXMPNULL;
			XMPDraftKey key = insertDraft->BeginClone(kXMPClonePaste);
			{
			TRY
				newSU = insertSU->CloneTo(key, su->GetDraft(), kXMPNULL);
			CATCH_ALL
				// If an exception was raised while cloning, abort the insert
				insertDraft->AbortClone(key);
				RERAISE;
			ENDTRY
			}
			insertDraft->EndClone(key);
			
			XMPPart* newPart = su->GetDraft()->GetPart(newSU->GetID());
			
			// Get the frame shape
			XMPShape* frameShape = ::NewXMPShape();
			if (insertSU->Exists(kXMPPropFrameShape, kXMPNULL, 0))
			{
				insertSU->Focus(kXMPPropFrameShape, kXMPPosUndefined, kXMPNULL, 0, kXMPPosUndefined);
				frameShape->ReadShape(insertSU);
			}
			else
			{
				FW_CRect frameRect(0, 0, ff(80), ff(80));
				frameShape->SetRectangle(&frameRect);
			}
			
			result = EmbedSelection(newPart, frameShape);
			
			// ----- Release -----
			newPart->Release();
			
			newSU->Release();
			insertSU->Release();
			draftSU->Release();
			insertDraft->Release();
			if (insertDocument)
				insertDocument->Release();
			insertContainer->Release();
			
			// ----- Release Shape -----
			delete frameShape;
			
			result = kXMPTrue;
			
			delete file;		
		}
		
	}
	
	
	return result;
}
*/

//------------------------------------------------------------------------------
//	 FW_CSelection::InsertNewPart
//------------------------------------------------------------------------------
// This is the wrong way to do Insert but it's fine for now

FW_Boolean FW_CSelection::InsertNewPart()
{
	XMPPart* embeddedPart;
	FW_Boolean result = FALSE;
	XMPStorageUnit* su = GetPart()->GetStorageUnit();
	
	XMPType	partkind = NULL;
	FW_CDynamicString partString;
	
	// StdGetFile please select an editor for the root part
	StandardFileReply sfReply;
	SFTypeList sfTypes;
	sfTypes[0] = 'libr';
	sfTypes[1] = 'libr';
	sfTypes[2] = 'libr';
	sfTypes[3] = 'libr';
	
	::StandardGetFile((FileFilterProcPtr)NULL,1,sfTypes,&sfReply);
	
	if (sfReply.sfGood) 
	{
		short saveResFile = CurResFile();
		short resRefNum = FSpOpenResFile(&(sfReply.sfFile),fsRdPerm);

		FW_PlatformHandle NMAPHdl = Get1Resource('NMAP',kNMAPid + 1);
		if (NULL != NMAPHdl)
		{

			unsigned char offset = sizeof(unsigned short) + sizeof(kXMPKind) + sizeof(unsigned short);
			unsigned char length = (*(char**)(NMAPHdl))[offset];
			partkind = (XMPValueType)FW_CMemoryManager::AllocateBlock(length + 1);
			partString.Append(&((*(char**)(NMAPHdl))[offset+1]), length);
			partString.Export(partkind);
		}
		ReleaseResource(NMAPHdl);
		CloseResFile(resRefNum);
		UseResFile(saveResFile);

		embeddedPart = su->GetDraft()->CreatePart(partkind, NULL);
		
		// ----- Create new Shape -----
		FW_CRect frameRect(0, 0, ff(80), ff(80));
		XMPShape* frameShape = ::NewXMPShape(frameRect);
		
		result = EmbedSelection(embeddedPart, frameShape);
		
		// ----- Release Shape -----
		delete frameShape;
		
		// ----- Release Part -----
		embeddedPart->Release();
	}
	
	return result;
}

//========================================================================================
//	 class FW_CPublishLink
//========================================================================================

//---------------------------------------------------------------------------------------
//	 FW_CPublishLink::FW_CPublishLink
//---------------------------------------------------------------------------------------

FW_CPublishLink::FW_CPublishLink(XMPChangeID changeID)
{
	fXMPLinkSource = NULL;
	fChangeID = changeID;
}

//---------------------------------------------------------------------------------------
//	 FW_CPublishLink::SetXMPLinkSource
//---------------------------------------------------------------------------------------

void FW_CPublishLink::SetXMPLinkSource(XMPLinkSource* xmpLinkSource)
{
	WASSERTM(xmpLinkSource!=NULL, "xmpLinkSource can't be null");
	WASSERTM(fXMPLinkSource==NULL, "fXMPLinkSource must be null");
	
	fXMPLinkSource = xmpLinkSource;
	xmpLinkSource->IncrementRefCount();
}

//---------------------------------------------------------------------------------------
//	 FW_CPublishLink::~FW_CPublishLink
//---------------------------------------------------------------------------------------

FW_CPublishLink::~FW_CPublishLink()
{
	if (fXMPLinkSource)
		fXMPLinkSource->Release();
}

//---------------------------------------------------------------------------------------
//	 FW_CPublishLink::ContentChanged
//---------------------------------------------------------------------------------------

void FW_CPublishLink::ContentChanged(XMPChangeID changeID)
{
	XMPLinkKey key;
	if (fXMPLinkSource->Lock(0, &key)) 
	{
		XMPStorageUnit* linkSU = fXMPLinkSource->GetContentStorageUnit(key);
		if (!linkSU->Exists(kXMPPropContents, (XMPValueType) NULL, 0)) 
		{
			linkSU->AddProperty(kXMPPropContents);
		}
		
		if (!linkSU->Exists(kXMPPropContents, GetPublishFormat(), 0)) 
		{
			linkSU->AddValue(GetPublishFormat());
		}
		else 
		{
			unsigned long valueSize;
			linkSU->Focus(kXMPPropContents, kXMPPosUndefined, GetPublishFormat(), 0, kXMPPosUndefined);
			valueSize = linkSU->GetSize();
			linkSU->DeleteValue(valueSize);
		}
		
		linkSU->Focus(kXMPPropContents, kXMPPosUndefined, GetPublishFormat(), 0, kXMPPosUndefined);

		ExternalizeLinkContent(linkSU);
		
		fXMPLinkSource->Unlock(key);		
		fXMPLinkSource->ContentChanged(changeID);	
	}
}

//========================================================================================
//	 class FW_CSubscribLink
//========================================================================================

//---------------------------------------------------------------------------------------
//	 FW_CSubscribLink::FW_CSubscribLink
//---------------------------------------------------------------------------------------

FW_CSubscribLink::FW_CSubscribLink(XMPLink *xmpLink)
{
	fXMPLink = xmpLink;
	fXMPLink->IncrementRefCount();
}

//---------------------------------------------------------------------------------------
//	 FW_CSubscribLink::~FW_CSubscribLink
//---------------------------------------------------------------------------------------

FW_CSubscribLink::~FW_CSubscribLink()
{
	if (fXMPLink)
		fXMPLink->Release();
}

//---------------------------------------------------------------------------------------
//	 FW_CSubscribLink::~FW_CSubscribLink
//---------------------------------------------------------------------------------------

void FW_CSubscribLink::LinkUpdated(XMPChangeID id)
{
FW_UNUSED(id);
}

