//========================================================================================
//
//	File:				FWUtil.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

// ----- Graphix Includes -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWGC_H
#include "FWGC.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _SHAPE_
#include <Shape.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__LIBRARYMANAGERUTILITIES__)
#include <LibraryManagerUtilities.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__LIBRARYMANAGERCLASSES__)
#include <LibraryManagerClasses.h>
#endif

#pragma segment framework

//----------------------------------------------------------------------------------------
// Abs
//----------------------------------------------------------------------------------------

long Abs(long value)
{
	return value<0 ? -1 * value : value;
}

//----------------------------------------------------------------------------------------
// ::IsKeyPressed
//----------------------------------------------------------------------------------------

FW_Boolean IsKeyPressed(unsigned short theKey)
{
	unsigned char theKeys[16];
	::GetKeys((long*)theKeys);
	return ((theKeys[theKey >> 3] >> (theKey & 7)) & 1);
}

//----------------------------------------------------------------------------------------
// ::NewXMPShape
//----------------------------------------------------------------------------------------
// The storage unit must have been focused first

XMPShape* NewXMPShape(XMPStorageUnit* su)
{
	XMPShape *shape  =  NULL;
	FW_VOLATILE(shape);

	TRY
		shape = new XMPShape;
		FailNULL(shape, kXMPErrOutOfMemory);
		shape = shape->ReadShape(su);
		
	CATCH_ALL
		
		if (shape!=NULL)
			delete shape;
			
		RERAISE;
		
	ENDTRY
	
	return shape;
}

//----------------------------------------------------------------------------------------
// ::CenterRectOnScreen
//----------------------------------------------------------------------------------------

void CenterRectOnScreen(FW_SPlatformRect& aRect, Boolean horizontally, Boolean vertically, Boolean forDialog)
{
	short newSize = 0;
	FW_SPlatformPoint rectSize, screenSize;

	::SetPt(&rectSize, aRect.right - aRect.left, aRect.bottom - aRect.top);
	::SetPt(&screenSize, XMPASLMQDGlobals.screenBits.bounds.right - XMPASLMQDGlobals.screenBits.bounds.left,
						XMPASLMQDGlobals.screenBits.bounds.bottom - XMPASLMQDGlobals.screenBits.bounds.top);
	screenSize.v -= GetMBarHeight();

	// Calculate screen size minus menu bar 
	if (horizontally)
		aRect.left = (screenSize.h - rectSize.h) / 2;
	if (vertically)
		if (forDialog)
		{
			newSize = (screenSize.v - rectSize.v) / 5;
			aRect.top = (short)(Max(newSize, 10) + GetMBarHeight());
		}
		else
			aRect.top = (screenSize.v - rectSize.v) / 2;
	
	aRect.bottom = aRect.top + rectSize.v;
	aRect.right = aRect.left + rectSize.h;
}

//========================================================================================
//	 class FW_CAcquireASLMResourceAccess
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CAcquireASLMResourceAccess::FW_CAcquireASLMResourceAccess
//----------------------------------------------------------------------------------------

FW_CAcquireASLMResourceAccess::FW_CAcquireASLMResourceAccess()
{
#ifdef ASLMBUILD
	fShellLibFile = GetLocalLibraryFile();
	fShellLibFile->Preflight(fShellLibRefNum);
#endif // ASLMBUILD
}

//----------------------------------------------------------------------------------------
// FW_CAcquireASLMResourceAccess::~FW_CAcquireASLMResourceAccess
//----------------------------------------------------------------------------------------

FW_CAcquireASLMResourceAccess::~FW_CAcquireASLMResourceAccess()
{
#ifdef ASLMBUILD
 	fShellLibFile->Postflight(fShellLibRefNum);
#endif // ASLMBUILD
}

//========================================================================================
//	 class FW_CDrawInitiator
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CDrawInitiator::FW_CDrawInitiator
//----------------------------------------------------------------------------------------

FW_CDrawInitiator::FW_CDrawInitiator(FW_CFacet *facet, XMPShape* clipShape) :
	FW_CGraphicContext(*facet, clipShape)
{
	fFacet = facet;
}

//----------------------------------------------------------------------------------------
// FW_CDrawInitiator::FW_CDrawInitiator
//----------------------------------------------------------------------------------------

FW_CDrawInitiator::FW_CDrawInitiator(XMPWindow* xmpWindow) :
	FW_CGraphicContext(xmpWindow)
{
}

//----------------------------------------------------------------------------------------
// FW_CDrawInitiator::~FW_CDrawInitiator
//----------------------------------------------------------------------------------------

FW_CDrawInitiator::~FW_CDrawInitiator()
{
}


