//========================================================================================
//
//	File:				FWView.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

// ----- Framework Includes -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWVIEW_H
#include "FWView.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _EXCEPT_
#include "Except.h"
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _POINT_
#include "FW_SPlatformPoint.h"
#endif

#ifndef _RECT_
#include "FW_SPlatformRect.h"
#endif

#ifndef _SHAPE_
#include "Shape.h"
#endif

#ifndef _TRNSFORM_
#include "Trnsform.h"
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(mathRoutinesIncludes)
#include <math routines.h>
#endif

//========================================================================================
// CLASS FW_CView
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CView::FW_CView
//----------------------------------------------------------------------------------------

FW_CView::FW_CView() :
	fFacet(NULL),
	fEnclosure(NULL),
	fShape(NULL),
	fTransform(NULL),
	fSubviewList(NULL)
{
}

//----------------------------------------------------------------------------------------
// FW_CView::~FW_CView
//----------------------------------------------------------------------------------------

FW_CView::~FW_CView()
{
	delete fShape;
	delete fTransform;
}

//----------------------------------------------------------------------------------------
// FW_CView::InitView
//----------------------------------------------------------------------------------------

void FW_CView::InitView(CFacet* facet, FW_CView* enclosure,
						XMPID identifier,
						XMPTransform *transform, XMPShape* shape)
{
	fFacet = facet;
	fEnclosure = enclosure;
	fShape = shape;
	fTransform = transform;

	SetIdentifier(identifier);
	
	if (enclosure != NULL)
		enclosure->AddSubview(this);
}


//----------------------------------------------------------------------------------------
//	FW_CView::HandleDraw
//----------------------------------------------------------------------------------------

void FW_CView::HandleDraw(XMPShape* area)
{	
	// ----- Draw subviews
	XMPShape* invalid = ::NewXMPShape();
	FW_CView *view = GetFirstSubview();
	while (view)
	{
		invalid->CopyFrom(area);
		view->EnclosureToView(*invalid);
		view->HandleDraw(invalid);
		view = GetNextSubview(view);
	}
	delete invalid;

	// ----- Draw this view
	XMPShape* viewShape = ::NewXMPShape(GetShape());
	FW_CViewDrawInitiator di(this);	
	Draw(viewShape);

#if 0
	// ----- Draw this view
	XMPShape* viewShape = ::NewXMPShape(GetShape());
	viewShape->Intersect(area);
	if (!viewShape->IsEmpty())
	{
		FW_CViewDrawInitiator di(this);		
		Draw(viewShape);
	}
#endif

}

//----------------------------------------------------------------------------------------
//	FW_CView::Draw
//----------------------------------------------------------------------------------------

void FW_CView::Draw(XMPShape* area)
{
FW_UNUSED(area);
}

//----------------------------------------------------------------------------------------
//	FW_CView::HandleMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_CView::HandleMouseDown(const XMPPoint& where, XMPEventData event)
{
	FW_CView* view = GetFirstSubview();
	while (view)
	{
		if (view->IsPointInView(where))
		{
			XMPPoint pt(where);
			view->EnclosureToView(pt);
			return view->HandleMouseDown(pt, event);
		}
		view = GetNextSubview(view);
	}
	
	return MouseDown(where, event);
}

//----------------------------------------------------------------------------------------
//	FW_CView::MouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_CView::MouseDown(const XMPPoint& where, XMPEventData event)
{
FW_UNUSED(where);
FW_UNUSED(event);

	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CView::HandleMouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_CView::HandleMouseUp(const XMPPoint& where, XMPEventData event)
{
	FW_CView* view = GetFirstSubview();
	while (view)
	{
		if (view->IsPointInView(where))
		{
			XMPPoint pt(where);
			view->EnclosureToView(pt);
			return view->HandleMouseUp(pt, event);
		}
		view = GetNextSubview(view);
	}
	
	return MouseUp(where, event);
}


//----------------------------------------------------------------------------------------
//	FW_CView::MouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_CView::MouseUp(const XMPPoint& where, XMPEventData event)
{
FW_UNUSED(where);
FW_UNUSED(event);

	return FALSE;
}
//----------------------------------------------------------------------------------------
//	FW_CView::IsPointInView
//----------------------------------------------------------------------------------------

FW_Boolean FW_CView::IsPointInView(const XMPPoint &point)
{
	XMPPoint pt = fTransform->InvertPoint(point);
	return GetShape()->ContainsPoint(pt);
}

//----------------------------------------------------------------------------------------
//	FW_CView::GetShape
//----------------------------------------------------------------------------------------

XMPShape* FW_CView::GetShape() const
{
	return fShape;
}

//----------------------------------------------------------------------------------------
//	FW_CView::SetShape
//----------------------------------------------------------------------------------------

void FW_CView::SetShape(XMPShape* shape)
{
	delete fShape;
	fShape = shape;
}

//----------------------------------------------------------------------------------------
//	FW_CView::EnclosureChangedShape
//----------------------------------------------------------------------------------------

void FW_CView::EnclosureChangedShape(XMPShape* oldShape, XMPShape* newShape)
{
FW_UNUSED(oldShape);
FW_UNUSED(newShape);
}

//----------------------------------------------------------------------------------------
//	FW_CView::ViewToEnclosure
//----------------------------------------------------------------------------------------

void FW_CView::ViewToEnclosure(XMPShape& shape) const
{
	shape.Transform(fTransform);
}

//----------------------------------------------------------------------------------------
//	FW_CView::EnclosureToView
//----------------------------------------------------------------------------------------

void FW_CView::EnclosureToView(XMPShape& shape) const
{
	shape.InverseTransform(fTransform);
}

//----------------------------------------------------------------------------------------
//	FW_CView::ViewToEnclosure
//----------------------------------------------------------------------------------------

void FW_CView::ViewToEnclosure(XMPPoint& point) const
{
	point = fTransform->TransformPoint(point);
}

//----------------------------------------------------------------------------------------
//	FW_CView::EnclosureToView
//----------------------------------------------------------------------------------------

void FW_CView::EnclosureToView(XMPPoint& point) const
{
	point = fTransform->InvertPoint(point);
}

//----------------------------------------------------------------------------------------
//	FW_CView::ComputeViewTransform
//----------------------------------------------------------------------------------------

XMPTransform* FW_CView::ComputeViewTransform() const
{
	XMPTransform* aggTransform = ::NewXMPTransform(fTransform);
	if (fEnclosure)
	{
		XMPTransform* transform = fEnclosure->ComputeViewTransform();
		aggTransform->PostCompose(transform);
		delete transform;
	}
	return aggTransform;
}

//----------------------------------------------------------------------------------------
// FW_CView::AddSubview
//----------------------------------------------------------------------------------------

void FW_CView::AddSubview(FW_CView *subview)
{
	if (fSubviewList == NULL)
		fSubviewList = new OrderedCollection;

	fSubviewList->AddLast(subview);
}

//----------------------------------------------------------------------------------------
// FW_CView::RemoveSubview
//----------------------------------------------------------------------------------------

void FW_CView::RemoveSubview(FW_CView *subview)
{
	ASSERT(fSubviewList != NULL, 0);
	
	fSubviewList->Remove(subview);
}

//----------------------------------------------------------------------------------------
// FW_CView::CountSubviews
//----------------------------------------------------------------------------------------

unsigned long FW_CView::CountSubviews() const
{
	return fSubviewList == NULL ? 0 : fSubviewList->Count();
}

//----------------------------------------------------------------------------------------
// FW_CView::HasSubviews
//----------------------------------------------------------------------------------------

FW_Boolean FW_CView::HasSubviews() const
{
	return fSubviewList != NULL;
}

//----------------------------------------------------------------------------------------
// FW_CView::IsInSubviewList
//----------------------------------------------------------------------------------------

FW_Boolean FW_CView::IsInSubviewList(FW_CView *subview) const
{
	return fSubviewList == NULL ? FALSE : fSubviewList->Contains(subview);
}

//----------------------------------------------------------------------------------------
// FW_CView::GetFirstSubview
//----------------------------------------------------------------------------------------

FW_CView* FW_CView::GetFirstSubview() const
{
	return fSubviewList == NULL ? NULL : (FW_CView *) fSubviewList->First();
}

//----------------------------------------------------------------------------------------
// FW_CView::GetSubviewAt
//----------------------------------------------------------------------------------------

FW_CView* FW_CView::GetSubviewAt(unsigned long index) const
{
	ASSERT(fSubviewList != NULL, 0);
	ASSERT(fSubviewList->Count() > index, 0);

	FW_CView *subview;
	unsigned long i = 0;
	
	OrderedCollectionIterator ite(fSubviewList);
	for (subview = (FW_CView *) ite.First();
			ite.IsNotComplete() && i < index;
				subview = (FW_CView *) ite.Next(), i++);
	
	return subview;
}

//----------------------------------------------------------------------------------------
// FW_CView::GetNextSubview
//----------------------------------------------------------------------------------------

FW_CView* FW_CView::GetNextSubview(FW_CView* subview, FW_Boolean circle) const
{
	ASSERT(fSubviewList != NULL, 0);

	if (circle && subview == fSubviewList->Last())
		return (FW_CView *) fSubviewList->First();
	else
		return (FW_CView *) fSubviewList->After(subview);
}

//----------------------------------------------------------------------------------------
// FW_CView::GetLastSubview
//----------------------------------------------------------------------------------------

FW_CView* FW_CView::GetLastSubview() const
{
	return fSubviewList == NULL ? NULL : (FW_CView *) fSubviewList->Last();
}

//----------------------------------------------------------------------------------------
// FW_CView::GetLastSubview
//----------------------------------------------------------------------------------------

FW_CView* FW_CView::FindSubview(XMPID identifier) const
{
	if (fSubviewList == NULL)
		return NULL;
		
	FW_CView *subview;
	
	OrderedCollectionIterator ite(fSubviewList);
	for (subview = (FW_CView *) ite.First();
			ite.IsNotComplete();
				subview = (FW_CView *) ite.Next())
	{
		if (subview->GetIdentifier() == identifier)
			return subview;
	}
	
	return NULL;
}

//----------------------------------------------------------------------------------------
// FW_CView::SetFacet
//----------------------------------------------------------------------------------------

void FW_CView::SetFacet(CFacet* facet)
{
	fFacet = facet;
}


//========================================================================================
// CLASS FW_CViewDrawInitiator
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CViewDrawInitiator::FW_CViewDrawInitiator
//----------------------------------------------------------------------------------------

FW_CViewDrawInitiator::FW_CViewDrawInitiator(FW_CView* view) :
	fDrawInitiator(view->GetFacet(), NULL)
{
	::GetPort(&fCurPort);
	
	fCurOrigin.h = XMPASLMQDGlobals.thePort->portRect.left;
	fCurOrigin.v = XMPASLMQDGlobals.thePort->portRect.top;

	XMPPoint offset(ff(fCurOrigin.h), ff(fCurOrigin.v));
	
	XMPTransform* transform = view->ComputeViewTransform();
	offset = transform->InvertPoint(offset);
	delete transform;
	
	::SetOrigin(offset.IntX(), offset.IntY());
}

//----------------------------------------------------------------------------------------
// FW_CViewDrawInitiator::~FW_CViewDrawInitiator
//----------------------------------------------------------------------------------------

FW_CViewDrawInitiator::~FW_CViewDrawInitiator()
{
	::SetOrigin(fCurOrigin.h, fCurOrigin.v);

	::SetPort(fCurPort);
}
