//========================================================================================
//
//	File:				FWWindow.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWWINDOW_H
#include "FWWindow.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFACET_H
#include "FWFacet.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

// ----- OpenDoc Includes -----
#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _WINDOW_
#include <Window.h>
#endif

#ifndef _STORAGEU_
#include <StorageU.h>
#endif

#ifndef _DRAFT_
#include <Draft.h>
#endif

#ifndef _XMPSESSN_
#include <XMPSessM.h>
#endif

#ifndef _INFO_
#include <Info.h>
#endif

#pragma segment framework

//==============================================================================
// CLASS FW_CWindow
//==============================================================================

//------------------------------------------------------------------------------
// FW_CWindow::FW_CWindow
//------------------------------------------------------------------------------

FW_CWindow::FW_CWindow() :
	fXMPWindow(NULL),
	fFacetCount(0)
{
}

//------------------------------------------------------------------------------
// FW_CWindow::InitWindow
//------------------------------------------------------------------------------

void FW_CWindow::InitWindow(FW_CPart* part, const FW_CRect& windowRect)
{
	fPart = part;
	fWindowRect = windowRect;
	fPart->AddWindow(this);
}

//------------------------------------------------------------------------------
// FW_CWindow::InitWindow
//------------------------------------------------------------------------------

void FW_CWindow::InitWindow(FW_CPart* part, XMPWindow* xmpWindow)
{
	fPart = part;
	fXMPWindow = xmpWindow;
	fXMPWindow->IncrementRefCount();
	fPart->AddWindow(this);
}

//------------------------------------------------------------------------------
// FW_CWindow::~FW_CWindow
//------------------------------------------------------------------------------

FW_CWindow::~FW_CWindow()
{
	if (fXMPWindow) 
	{
		fXMPWindow->Release();
		fXMPWindow = NULL;
	}
}

//------------------------------------------------------------------------------
// FW_CWindow::CloseWindow
//
// Never call delete myWindow to close a window, call CloseWindow
//------------------------------------------------------------------------------

void FW_CWindow::CloseWindow()
{
	if (fXMPWindow)
	{
		HideWindow();
		fXMPWindow->Close();	// 'this' will be deleted automatically
	}		
}

//------------------------------------------------------------------------------
// FW_CWindow::ShowWindow
//------------------------------------------------------------------------------

void FW_CWindow::ShowWindow()
{
	if (fXMPWindow)
		fXMPWindow->Show();
}

//------------------------------------------------------------------------------
// FW_CWindow::HideWindow
//------------------------------------------------------------------------------

void FW_CWindow::HideWindow()
{
	if (fXMPWindow)
		fXMPWindow->Hide();
}

//------------------------------------------------------------------------------
//	FW_CWindow::IsShown
//------------------------------------------------------------------------------

FW_Boolean FW_CWindow::IsShown() const
{
	return fXMPWindow ? fXMPWindow->IsShown() : FALSE;
}

//------------------------------------------------------------------------------
//	 FW_CWindow::FacetAdded
//------------------------------------------------------------------------------

void FW_CWindow::FacetAdded(FW_CFacet *facet)
{
	fFacetCount++;
	
	if (fXMPWindow == NULL)
	{
		fXMPWindow = facet->GetXMPWindow();
		fXMPWindow->IncrementRefCount();
	}
}

//------------------------------------------------------------------------------
//	 FW_CWindow::FacetRemoved
//------------------------------------------------------------------------------

void FW_CWindow::FacetRemoved(FW_CFacet *facet)
{
	fFacetCount--;
	
	if (fFacetCount == 0)
	{
		facet->GetFrame()->GetPart()->RemoveWindow(this);
		delete this;
	}
}

//==============================================================================
// CLASS FW_CFloatingWindow
//==============================================================================

//------------------------------------------------------------------------------
// FW_CFloatingWindow::FW_CFloatingWindow
//------------------------------------------------------------------------------

FW_CFloatingWindow::FW_CFloatingWindow()
{
	fFloatingShown = FALSE;
}

//------------------------------------------------------------------------------
// FW_CFloatingWindow::InitFloatingWindow
//------------------------------------------------------------------------------

void FW_CFloatingWindow::InitFloatingWindow(FW_CPart* part,
										 XMPTypeToken presentation,
										 const FW_CRect& windowRect)
{
	fPresentation = presentation;
	InitWindow(part, windowRect);
}

//------------------------------------------------------------------------------
// FW_CFloatingWindow::InitFloatingWindow
//------------------------------------------------------------------------------

void FW_CFloatingWindow::InitFloatingWindow(FW_CPart* part, XMPWindow* xmpWindow)
{
	InitWindow(part, xmpWindow);
	fFloatingShown = xmpWindow->GetWasVisible();
}

//------------------------------------------------------------------------------
// FW_CFloatingWindow::~FW_CFloatingWindow
//------------------------------------------------------------------------------

FW_CFloatingWindow::~FW_CFloatingWindow()
{
}

//------------------------------------------------------------------------------
// FW_CFloatingWindow::CreateFloatingWindow
//------------------------------------------------------------------------------

void FW_CFloatingWindow::CreateFloatingWindow(XMPTypeToken presentation)
{
	fXMPWindow = fPart->CreateXMPWindow(NULL,
									  fPart->GetSession()->Tokenize(kXMPViewAsFrame),
									  presentation,
									  "",
									  fWindowRect,
									  TRUE,			// close box
									  FALSE,		// resizable
									  TRUE,			// floating
									  TRUE,			// should save
									  kWindoidWDEF + kWindoidNotResizable);
	
	fXMPWindow->IncrementRefCount();
	
	fXMPWindow->Open();
}

//------------------------------------------------------------------------------
// FW_CFloatingWindow::ShowWindow
//------------------------------------------------------------------------------

void FW_CFloatingWindow::ShowWindow()
{
	if (!fFloatingShown)
	{
		if (fXMPWindow == NULL)
			CreateFloatingWindow(fPresentation);
		
		FW_CWindow::ShowWindow();
		fFloatingShown = TRUE;
	}
}

//------------------------------------------------------------------------------
// FW_CFloatingWindow::HideWindow
//------------------------------------------------------------------------------

void FW_CFloatingWindow::HideWindow()
{
	if (fFloatingShown)
	{		
		FW_CWindow::HideWindow();
		fFloatingShown = FALSE;
	}
}

//------------------------------------------------------------------------------
// FW_CFloatingWindow::HideShowFloating
//------------------------------------------------------------------------------

void FW_CFloatingWindow::HideShowFloating()
{
	if (fFloatingShown)
		HideWindow();	
	else
		ShowWindow();
}

//------------------------------------------------------------------------------
//	FW_CFloatingWindow::IsShown
//------------------------------------------------------------------------------

FW_Boolean FW_CFloatingWindow::IsShown() const
{
	return fFloatingShown;
}

